/***************************************************************
 * 
 *  takeover ext4 interface
 *
 *	date: 2019/06/24
 *
 * 	xxx_xxxx@apowertec.com
 *
 ****************************************************************/
#include <common.h>
#include <exports.h>
#include <part.h>
#include <config.h>
#include <command.h>
#include <image.h>
#include <linux/ctype.h>
#include <asm/byteorder.h>
#include <ext4fs.h>
#include <linux/stat.h>
#include <malloc.h>
#include <fs.h>
#include <fw_info.h>
#include <asm/arch/takeoverlib_ext4.h>
#include <asm/arch/system.h>
#include <gpt_parse.h>

/***************************************************************
*
*	marco define 
*
****************************************************************/
#define ENABLE_TAKEOVER_CONSOLE_MODE
#define ENABLE_TAKEOVER_DEBUG

#define TAKEOVER_PRINTF(fmt, args...)	printf(fmt, ##args)

#ifdef	ENABLE_TAKEOVER_DEBUG
#define FAC_DEBUG(fmt, args...)		printf(fmt, ##args)
#else
#define FAC_DEBUG(fmt, args...)
#endif

#define TAKEOVER_PART_NAME           "takeover"
#define TAKEOVER_OLD_PATH_PREFIX		"takeover/"

#define TAKEOVER_BACKUP_PATH_PREFIX  "/.000@bk@"
#define TAKEOVER_BACKUP_PATH_PREFIX1  ".000@bk@"

/***************************************************************
*
*	variable define 
*
****************************************************************/
static char *takeover_ext4_buffer = NULL;
static unsigned int takeover_ext4_size = 0x0;

static disk_partition_t takeover_ext4_info;
static block_dev_desc_t *takeover_ext4_dev_desc = NULL;

static int takeover_ext4_is_ready = -1;

/***************************************************************
*
*	takeover ext4 api
*
****************************************************************/
static int takeover_is_ready(void)
{
	return takeover_ext4_is_ready;
}

static void takeover_path_refine(char *path)
{
	char tmpname[256] = {0};
	char *pos = NULL;
	
	pos = strstr(path, TAKEOVER_OLD_PATH_PREFIX);
	if(pos == path){
		memset(tmpname, 0x00, sizeof(tmpname));
		memcpy(tmpname, path + strlen(TAKEOVER_OLD_PATH_PREFIX), strlen(path) - strlen(TAKEOVER_OLD_PATH_PREFIX));
		memset(path, 0x00, strlen(path));
		memcpy(path, tmpname, strlen(tmpname));
	}

	if(path[0] != '/'){
		memset(tmpname, 0x00, sizeof(tmpname));
		tmpname[0] = '/';
		memcpy(tmpname + 1, path, strlen(path));
		memset(path, 0x00, strlen(path));
		memcpy(path, tmpname, strlen(tmpname));
	}
	
	return;
}

static void takeover_get_backup_filename(char *path)
{
	char tmpname[256] = {0};
	char *pos = NULL;
	unsigned int i = 0, max = 0;

	max = strlen(path);
	memset(tmpname, 0x00, sizeof(tmpname));
	memcpy(tmpname, TAKEOVER_BACKUP_PATH_PREFIX, strlen(TAKEOVER_BACKUP_PATH_PREFIX));

	pos = tmpname + strlen(tmpname);
	
	for(i = 0; i < max; i++){
		if(path[i] == '/')
			pos[i] = '\\';
		else
			pos[i] = path[i];
	}

	memset(path, 0x00, strlen(path));
	memcpy(path, tmpname, strlen(tmpname));
	
	return;
}

static int takeover_parse_backup_filename(char *path)
{
	char tmpname[256] = {0};
	unsigned int i = 0, j = 0, max = 0;
	
	memset(tmpname, 0x00, sizeof(tmpname));

	if(strstr(path, TAKEOVER_BACKUP_PATH_PREFIX) == NULL)
		return -1;

	i = strlen(TAKEOVER_BACKUP_PATH_PREFIX);
	max = strlen(path);

	for( ; i < max; i++){
		if(path[i] == '\\')
			tmpname[j] = '/';
		else
			tmpname[j] = path[i];
		j++;
	}

	memset(path, 0x00,strlen(path));
	memcpy(path, tmpname, strlen(tmpname));
	
	return 0;
}

int takeover_read(char *path, char**buffer, int *length)
{
	int ret = 0;
	int file_len = 0;
	int read_len = 0;
	char filename[256] = {0};
	
	if(takeover_is_ready() < 0)
		return -1;

	if((path == NULL) || ((strlen(path) + 1) > sizeof(filename)))
		return -1;

	memset(filename, 0x00, sizeof(filename));
	memcpy(filename, path, strlen(path));
	
	/*delete takeover path header: takeovers/ */
	takeover_path_refine(filename);
    TAKEOVER_PRINTF("filename is =%s", filename);
	/* set the device as block device */
	ext4fs_set_blk_dev(takeover_ext4_dev_desc, &takeover_ext4_info);

	/* mount the filesystem */
	if (!ext4fs_mount(takeover_ext4_info.size)) {
		TAKEOVER_PRINTF("mount ext4 partition %s fail!\r\n", TAKEOVER_PART_NAME);
		goto fail;
	}

	/*open file and get file size*/
	ret = ext4fs_open(filename, (loff_t *)&file_len);
	if (ret < 0) {
		TAKEOVER_PRINTF("file %s not found in %s partition!\r\n", filename, TAKEOVER_PART_NAME);
		goto fail;
	}

	/*check file size*/
	if(file_len >= (int)takeover_ext4_size){
		TAKEOVER_PRINTF("file size is too big, size = 0x%x!\r\n", (unsigned int)file_len);
		goto fail;
	}

	/*read data from takeover*/
	ret = ext4fs_read(takeover_ext4_buffer,0, (loff_t)file_len, (loff_t *)&read_len);
	if((ret <0) || (read_len != file_len)){
		TAKEOVER_PRINTF("load file %s to ddr fail!\r\n", filename);
		goto fail;
	}

	*length = file_len;
	*buffer = takeover_ext4_buffer;
	
	ext4fs_close();
	return 0;
fail:
	ext4fs_close();
	return -1;
}

int takeover_write(char *path, char *buffer, int length)
{
	char filename[256] = {0};
	char tmp_filename[256] = {0};

	if(takeover_is_ready() < 0)
		return -1;

	if((path == NULL) || ((strlen(path) + 1) > sizeof(filename)))
		return -1;

	memset(filename, 0x00, sizeof(filename));
	memcpy(filename, path, strlen(path));

	/*delete takeover path header: takeovers */
	takeover_path_refine(filename);

	/*get tmp backup file path */
	memset(tmp_filename, 0x00, sizeof(tmp_filename));
	memcpy(tmp_filename, filename, strlen(filename));
	takeover_get_backup_filename(tmp_filename);

	/* set the device as block device */
	ext4fs_set_blk_dev(takeover_ext4_dev_desc, &takeover_ext4_info);

	/* mount the filesystem */
	if (!ext4fs_mount(takeover_ext4_info.size)) {
		TAKEOVER_PRINTF("mount ext4 partition %s fail!\r\n", TAKEOVER_PART_NAME);
		goto fail;
	}

	/*gen current timestamp*/
	ext4fs_timestamp_gen();
	
	/* first, update tmp backup file to filesystem*/
	if (ext4fs_write(tmp_filename, (char *)buffer, (unsigned long)length, FILETYPE_REG)) {
		TAKEOVER_PRINTF("write %s to %s partition fail!\r\n", tmp_filename, TAKEOVER_PART_NAME);
		goto fail;
	}

	/* second, write data to filesystem */
	if (ext4fs_write(filename, (char *)buffer, (unsigned long)length, FILETYPE_REG)) {
		TAKEOVER_PRINTF("write %s to %s partition fail!\r\n", filename, TAKEOVER_PART_NAME);
		goto fail;
	}

	/*last, delete tmp file from filesystem*/
	ext4fs_unlink(tmp_filename);
	
	ext4fs_close();
	return 0;
fail:
	ext4fs_close();
	return -1;
}

#if 0
int takeover_save(void)
{
	if(takeover_is_ready() < 0)
		return -1;
	
	return 0;
}
#endif

/*file recovery mechanism*/
int takeover_delete(char *path)
{
	char filename[256] = {0};
	
	if(takeover_is_ready() < 0)
		return -1;

	if((path == NULL) || ((strlen(path) + 1) > sizeof(filename)))
		return -1;

	memset(filename, 0x00, sizeof(filename));
	memcpy(filename, path, strlen(path));
	
	/*delete takeover header: takeover*/
	takeover_path_refine(filename);

	/* set the device as block device */
	ext4fs_set_blk_dev(takeover_ext4_dev_desc, &takeover_ext4_info);

	/* mount the filesystem */
	if (!ext4fs_mount(takeover_ext4_info.size)) {
		TAKEOVER_PRINTF("mount ext4 partition %s fail!\r\n", TAKEOVER_PART_NAME);
		goto fail;
	}

	/* unlink file */
	if (ext4fs_unlink(filename)) {
		TAKEOVER_PRINTF("unlink %s fail!\r\n", filename);
		goto fail;
	}
	
	ext4fs_close();
	return 0;
fail:
	ext4fs_close();
	return -1;
}

int takeover_find_file(char *dir, char *prefix, char *filename)
{
	char dir_name[256] = {0};
	
	if(takeover_is_ready() < 0)
		return -1;
	
	if(dir == NULL || prefix == NULL || filename == NULL)
		return -1;

	memset(dir_name, 0x00, sizeof(dir_name));
	memcpy(dir_name, dir, strlen(dir));
	
	/*delete takeover path header: takeovers*/
	takeover_path_refine(dir_name);
	
	/* set the device as block device */
	ext4fs_set_blk_dev(takeover_ext4_dev_desc, &takeover_ext4_info);

	/* mount the filesystem */
	if (!ext4fs_mount(takeover_ext4_info.size)) {
		TAKEOVER_PRINTF("mount ext4 partition %s fail!\r\n", TAKEOVER_PART_NAME);
		goto fail;
	}

	if(ext4fs_find_filepath(dir_name, prefix, filename)){
		TAKEOVER_PRINTF("%s partition can not find %s%s***!\r\n",TAKEOVER_PART_NAME, dir_name, prefix);
		goto fail;
	}
	
	ext4fs_close();
	return 0;
fail:
	ext4fs_close();
	return -1;
}

static int takeover_get_file_mtime(char *path, unsigned int *mtime)
{
	char filename[256] = {0};
	
	if(takeover_is_ready() < 0)
		return -1;

	if((path == NULL) || ((strlen(path) + 1) > sizeof(filename)))
		return -1;

	memset(filename, 0x00, sizeof(filename));
	memcpy(filename, path, strlen(path));
	
	/*delete takeover path header: takeover*/
	takeover_path_refine(filename);

	/* set the device as block device */
	ext4fs_set_blk_dev(takeover_ext4_dev_desc, &takeover_ext4_info);

	/* mount the filesystem */
	if (!ext4fs_mount(takeover_ext4_info.size)) {
		TAKEOVER_PRINTF("mount ext4 partition %s fail!\r\n", TAKEOVER_PART_NAME);
		goto fail;
	}

	/* get mtime */
	if (ext4fs_mtime(filename, mtime)) {
		TAKEOVER_PRINTF("get mtime %s fail!\r\n", filename);
		goto fail;
	}
		
	ext4fs_close();
	return 0;
fail:
	ext4fs_close();
	return -1;

}

static int takeover_auto_check(void)
{
	int ret1 = 0, ret2 = 0;
	char *buffer1, *buffer2;
	int len1 = 0, len2 = 0;
	
	char filename[256] = {0};
	char tmp_filename[256] = {0};
	int recovery_flag = 0;

	memset(filename, 0x00, sizeof(filename));
	memset(tmp_filename, 0x00, sizeof(tmp_filename));

	/*get tmp backup file status*/
	ret1 = takeover_find_file("/", TAKEOVER_BACKUP_PATH_PREFIX1, tmp_filename);
	if(ret1 < 0){
		TAKEOVER_PRINTF("TAKEOVER no need recovery!\r\n");
		return 0;
	}

	takeover_path_refine(tmp_filename);

	/*get recovery file path*/
	memcpy(filename, tmp_filename, strlen(tmp_filename));
	ret1 = takeover_parse_backup_filename(filename);
	if(ret1 < 0){
		TAKEOVER_PRINTF("Cannot parse recovery file name!\r\n");
		return -1;
	}

	/*recovery */
	ret1 = takeover_read(tmp_filename, &buffer1, &len1);
	ret2 = takeover_read(filename, &buffer2, &len2);
	if((ret2 < 0) && (ret1 == 0)){
		TAKEOVER_PRINTF("TAKEOVER recovery run case1, save %s to %s!\r\n", tmp_filename, filename);
		takeover_write(filename, buffer1, len1);
		recovery_flag = 1;
	}else if((ret1 == 0) && (ret2 == 0)){
		unsigned int mtime1 = 0, mtime2 = 0;

		takeover_get_file_mtime(tmp_filename, &mtime1);
		takeover_get_file_mtime(filename, &mtime2);

		//TAKEOVER_DEBUG("mtime1 = 0x%x mtime2 = 0x%x len1 = 0x%x len2 = 0x%x\r\n", mtime1, mtime2, len1, len2);
		if(mtime1 == mtime2){
			TAKEOVER_PRINTF("TAKEOVER recovery run case2, save %s to %s!\r\n", tmp_filename, filename);
			takeover_write(filename, buffer1, len1);
			recovery_flag = 1;
		}
	}

	if(recovery_flag == 0)
		takeover_delete(tmp_filename);
	
	return 0;
}


int takeover_init(void)
{
	int ret = 0;
	unsigned int dev_index = 0;
	char dev_string[32] = {0};
	int dev = -1, part = -1;
	
	if(boot_flash_type != BOOT_EMMC){
		TAKEOVER_PRINTF("Not support current flash, flash type = %d !\r\n", (int)boot_flash_type);
		return -1;
	}

	takeover_ext4_size = 2048;
	takeover_ext4_buffer = (char *)malloc(takeover_ext4_size);

	TAKEOVER_PRINTF("[EXT4] takeover_buffer:%p, takeover_size:0x%x!\n", takeover_ext4_buffer, takeover_ext4_size);

	/*get takeover partition index*/
	ret = get_dev_index_by_name(TAKEOVER_PART_NAME, &dev_index);
	if(ret < 0){
		TAKEOVER_PRINTF("Get %s partition block device index fail!\r\n", TAKEOVER_PART_NAME);
		TAKEOVER_PRINTF("No takeover partition!\r\n");
		return 0;
	}
	
	/*get takeover partition info*/
	memset(&takeover_ext4_info, 0x00, sizeof(disk_partition_t));
	memset(dev_string, 0x00, sizeof(dev_string));
	snprintf(dev_string, sizeof(dev_string), "0:0x%x", dev_index);
	part = get_device_and_partition("mmc", dev_string, &takeover_ext4_dev_desc, &takeover_ext4_info, 1);
	if (part < 0){
		TAKEOVER_PRINTF("Get %s partition desc info fail!\r\n", TAKEOVER_PART_NAME);
		return -1;
	}
	dev = takeover_ext4_dev_desc->dev;

	TAKEOVER_PRINTF("Partiton %s located at %d:%d \r\n", TAKEOVER_PART_NAME, dev, part);
	
	takeover_ext4_is_ready = 1;

	takeover_auto_check();
	
	return 0;
}

#ifdef ENABLE_TAKEOVER_CONSOLE_MODE
int takeover_dump_info(void)
{
	if(takeover_is_ready() < 0)
		return -1;
	
	TAKEOVER_PRINTF("****** dump takeover info ******\n");
	TAKEOVER_PRINTF("boot_flash_type    = ");
	switch(boot_flash_type){
		case BOOT_NOR_SERIAL:
			TAKEOVER_PRINTF("NOR\n");
			break;

		case BOOT_NAND:
			TAKEOVER_PRINTF("NAND\n");
			break;

		case BOOT_EMMC:
			TAKEOVER_PRINTF("MMC\n");
			break;

		case BOOT_SPI_NAND:
            TAKEOVER_PRINTF("SPI NAND\n");
			break;

		default:
			TAKEOVER_PRINTF("UNKNOWN\n");
			break;
	}

	TAKEOVER_PRINTF("takeover_buffer     = %16ld\n", (ulong)takeover_ext4_buffer);
	TAKEOVER_PRINTF("takeover_total_size = 0x%08x\n", takeover_ext4_size);

	return 0;
}

int takeover_dump_list(void)
{
	if(takeover_is_ready() < 0)
		return -1;
	
	/* set the device as block device */
	ext4fs_set_blk_dev(takeover_ext4_dev_desc, &takeover_ext4_info);

	/* mount the filesystem */
	if (!ext4fs_mount(takeover_ext4_info.size)) {
		TAKEOVER_PRINTF("mount ext4 partition %s fail!\r\n", TAKEOVER_PART_NAME);
		goto fail;
	}

	/* show takeover file list */
	if (ext4fs_ls("/")) {
		TAKEOVER_PRINTF("show %s partition file list fail!\r\n", TAKEOVER_PART_NAME);
		goto fail;
	}
	
	ext4fs_close();
	return 0;
fail:
	ext4fs_close();
	return -1;
}

int do_ext4_takeover(cmd_tbl_t *cmdtp, int flag, int argc, char * const argv[])
{
	char *buffer;
	unsigned long dst_buffer;//for rtice tool takeover read
	int length;
	int rc;

	if (argc < 2) {
		return CMD_RET_USAGE;
	}
	else {
		if (strcmp(argv[1], "info") == 0) {
			takeover_dump_info();
		}
		else if (strcmp(argv[1], "list") == 0) {
			takeover_dump_list();
		}
		else if (strcmp(argv[1], "read") == 0) {
			rc = takeover_read(argv[2], &buffer, &length);

			if (rc)
				TAKEOVER_PRINTF("%s: file not found\n", argv[2]);
			else
				TAKEOVER_PRINTF("%s: %d bytes at 0x%08x\n", argv[2], length, (uint)buffer);

			if (argc == 4) {
				dst_buffer = simple_strtoul(argv[3], NULL, 16);
				memcpy((void *)dst_buffer, (void *)buffer, length);
				TAKEOVER_PRINTF("src[0x%08x] dst[0x%08x]\n", (uint)buffer, (uint)dst_buffer);
			}
		}
		else if (strcmp(argv[1], "write") == 0) {

			buffer = (char *)simple_strtoul(argv[3], NULL, 16);
			length = (int)simple_strtoul(argv[4], NULL, 16);

			takeover_write(argv[2], buffer, length);
		}
		else {
			return CMD_RET_USAGE;
		}
	}

	return 0;
}


U_BOOT_CMD(
	takeover,	5,	1,	do_ext4_takeover,
	"takeover sub system",
	"read path\n"
	"takeover write path addr length\n"
	"takeover info\n"
	"takeover list"
);
#endif
