
#include <common.h>
#include <command.h>
#include <io.h>

#define MEM_TRASH_DBG_START0    0x1805C020
#define MEM_TRASH_DBG_END0      0x1805C030
#define MEM_TRASH_DBG_CTRL          0x1805C040

#define MEM_TRASH_DBG_INT           0x1805C050
#define MEM_TRASH_DBG_ADDR          0x1805C054
#define MEM_TRASH_DBG_RW            0x1805C058

#define WRAPPER_NUM 4

int set_addr;
int set_rw;
int flag_data_abort=-1;
int trigger_total=0;
//test 1 : read protect 0x1000-0x1100 in set 2, try to read 0x1000 will ...  data abort, check \e[1;32mOK\e[0m

char* print_str []=
{
    "data abort, check \e[1;32mOK\e[0m",
    "data abort, check \e[1;31mFAIL\e[0m",

    "\e[1;34mNO\e[0m data abort",
};

#define READ 0
#define WRITE 1
#define READ_WRITE 2
#define WRITE_READ 3

void ArmWrapper_Reset (void)
{
    int i;
    rtd_outl(MEM_TRASH_DBG_INT, 0x3E);

    for(i=0; i<WRAPPER_NUM; i++)
    {
        rtd_outl(MEM_TRASH_DBG_CTRL+i*4, 0x2);
        rtd_outl(MEM_TRASH_DBG_START0+i*4, 0);
        rtd_outl(MEM_TRASH_DBG_END0+i*4, 0);
    }
}

static int negative_flag=0;
//static unsigned int negative_addr_start,negative_addr_end;
void ArmWrapper_Set (unsigned int num, unsigned int addr_start,unsigned int addr_end,unsigned int rw)
{
    unsigned int rw_value[]= {0x17,0x1b,0x13,0x1f};
    ArmWrapper_Reset();

    rtd_outl(MEM_TRASH_DBG_START0+num*4, addr_start);
    rtd_outl(MEM_TRASH_DBG_END0+num*4,   addr_end);
    rtd_outl(MEM_TRASH_DBG_CTRL+num*4,   rw_value[rw]);

    if (negative_flag)
    {
        rtd_outl(MEM_TRASH_DBG_INT, 0x3);
    }
    else
    {
        rtd_outl(MEM_TRASH_DBG_INT, 0x9);
    }
}

void pre_test (unsigned int addr_start, unsigned int addr_end, unsigned int addr_test, unsigned int rw_set,    unsigned int rw_test)
{
    set_rw=rw_test;
    set_addr=addr_test;

    //negative is DBG_START[k]-DBG_END[k]-1
    if (negative_flag && ( (addr_test< addr_start )||(addr_test> (addr_end-1) ) ) && 
         (rw_set==2||rw_set==3||rw_set == rw_test) )
    {
        flag_data_abort=1;
        //printf("%s %d\n",__func__,__LINE__);
        return;
    } 
    else if (negative_flag)
    {
        flag_data_abort=0;
        return;
    }

    if ( addr_test>=addr_start && addr_test< (addr_end+4) && 
        (rw_set==2||rw_set==3||rw_set == rw_test))
    {
        flag_data_abort=1;
    }
    else
    {
        flag_data_abort=0;
    }
}

void post_test (unsigned int addr_set, unsigned int addr_test, unsigned int rw_set,    unsigned int rw_test)
{
    if (!flag_data_abort)
    {
        printf(print_str[2]);
        printf("\n");
    }
    else
    {
        printf(print_str[1]);
        printf("\n");
        while(1);
    }
}

char* str[]= {"read","write","read&write","read&write"};
void ArmWrapper_DataAbort_Trigger (
    unsigned int num,
    unsigned int addr_start, unsigned int addr_end,
    unsigned int addr_test,
    unsigned int rw_set,    unsigned int rw_test,
    unsigned int negative )
{
    negative_flag=negative;

    if (negative)
    {
        printf("test %5d: %10s \e[1;31mnegative\e[0m protect 0x%08x-0x%08x in set %d, try to %5s 0x%08x  will ...      ",
            ++trigger_total, str[rw_set],addr_start,addr_end,num,str[rw_test],addr_test);
    }
    else
    {
        printf("test %5d: %10s protect 0x%08x-0x%08x in set %d, try to %5s 0x%08x  will ...      ",
            ++trigger_total, str[rw_set],addr_start,addr_end,num,str[rw_test],addr_test);
    }

    ArmWrapper_Set(num,addr_start,addr_end,rw_set);

    pre_test(addr_start, addr_end, addr_test, rw_set, rw_test);

    if(rw_test)
    {
        rtd_outl(addr_test, 0x12345678);
    }
    else
    {
        rtd_inl(addr_test);
    }

    post_test(addr_start,addr_test, rw_set, rw_test);
}

static int test_loop;
struct test_set
{
    unsigned int start;
    unsigned int end;
    unsigned int test_addr;
    unsigned int read_write;
    unsigned int test_read_write;
    unsigned int negative;
};
static struct test_set array[]=
{
    {0x10000000,0x10000100,0x10000000,READ, READ,0},
    {0x10000000,0x10000100,0x10000000,READ, WRITE,0},
    {0x10000000,0x10000100,0x10000000,WRITE,READ,0},
    {0x10000000,0x10000100,0x10000000,WRITE,WRITE,0},

    {0x10000000,0x10000100,0x0ffffffc,READ, READ,0},
    {0x10000000,0x10000100,0x0ffffffc,READ, WRITE,0},
    {0x10000000,0x10000100,0x0ffffffc,WRITE,READ,0},
    {0x10000000,0x10000100,0x0ffffffc,WRITE,WRITE,0},

    {0x10000000,0x10000100,0x10000100,READ, READ,0},
    {0x10000000,0x10000100,0x10000100,READ, WRITE,0},
    {0x10000000,0x10000100,0x10000100,WRITE,READ,0},
    {0x10000000,0x10000100,0x10000100,WRITE,WRITE,0},

    {0x10000000,0x10000100,0x10000104,READ, READ,0},
    {0x10000000,0x10000100,0x10000104,READ, WRITE,0},
    {0x10000000,0x10000100,0x10000104,WRITE,READ,0},
    {0x10000000,0x10000100,0x10000104,WRITE,WRITE,0},

    {0x10000000,0x10000100,0x10000000,READ_WRITE, READ,0},
    {0x10000000,0x10000100,0x10000000,WRITE_READ, READ,0},
    {0x10000000,0x10000100,0x10000100,READ_WRITE, WRITE,0},
    {0x10000000,0x10000100,0x10000100,WRITE_READ, WRITE,0},
    {0x10000000,0x10000100,0x0ffffffc,READ_WRITE, READ,0},
    {0x10000000,0x10000100,0x0ffffffc,READ_WRITE, WRITE,0},
    {0x10000000,0x10000100,0x0ffffffc,WRITE_READ, READ,0},
    {0x10000000,0x10000100,0x0ffffffc,WRITE_READ, WRITE,0},
    {0x10000000,0x10000100,0x10000104,READ_WRITE, READ,0},
    {0x10000000,0x10000100,0x10000104,READ_WRITE, WRITE,0},
    {0x10000000,0x10000100,0x10000104,WRITE_READ, READ,0},
    {0x10000000,0x10000100,0x10000104,WRITE_READ, WRITE,0},

    {0x00000100,0x30000100,0x000000fc,READ, READ,1},
    {0x00000100,0x30000100,0x00000100,READ, READ,1},
    {0x00000100,0x30000100,0x300000fc,READ, READ,1},
    {0x00000100,0x30000100,0x30000100,READ, READ,1},
    {0x00000100,0x30000100,0x30000104,READ, READ,1},

    {0x00000100,0x30000100,0x000000fc,WRITE,WRITE,1},
    {0x00000100,0x30000100,0x00000100,WRITE,WRITE,1},
    {0x00000100,0x30000100,0x300000fc,WRITE,WRITE,1},
    {0x00000100,0x30000100,0x30000100,WRITE,WRITE,1},
    {0x00000100,0x30000100,0x30000104,WRITE,WRITE,1},

    {0x00000100,0x30000100,0x000000fc,READ, WRITE,1},
    {0x00000100,0x30000100,0x00000100,READ, WRITE,1},
    {0x00000100,0x30000100,0x300000fc,READ, WRITE,1},
    {0x00000100,0x30000100,0x30000100,READ, WRITE,1},
    {0x00000100,0x30000100,0x30000104,READ, WRITE,1},

    {0x00000100,0x30000100,0x000000fc,WRITE,READ,1},
    {0x00000100,0x30000100,0x00000100,WRITE,READ,1},
    {0x00000100,0x30000100,0x300000fc,WRITE,READ,1},
    {0x00000100,0x30000100,0x30000100,WRITE,READ,1},
    {0x00000100,0x30000100,0x30000104,WRITE,READ,1},

    {0x00000100,0x30000100,0x000000fc,READ_WRITE,READ,1},
    {0x00000100,0x30000100,0x00000100,READ_WRITE,READ,1},
    {0x00000100,0x30000100,0x300000fc,READ_WRITE,READ,1},
    {0x00000100,0x30000100,0x30000100,READ_WRITE,READ,1},
    {0x00000100,0x30000100,0x30000104,READ_WRITE,READ,1},

    {0x00000100,0x30000100,0x000000fc,WRITE_READ, WRITE,1},
    {0x00000100,0x30000100,0x00000100,WRITE_READ, WRITE,1},
    {0x00000100,0x30000100,0x300000fc,WRITE_READ, WRITE,1},
    {0x00000100,0x30000100,0x30000100,WRITE_READ, WRITE,1},
    {0x00000100,0x30000100,0x30000104,WRITE_READ, WRITE,1},

};

void ArmWrapper_DataAbort_Test (unsigned int index);
void ArmWrapper_DataAbort_Test_ (unsigned int index)
{
    static int i=-1;
    //for (i=0; i<WRAPPER_NUM;i++)
        ArmWrapper_DataAbort_Trigger(++i%WRAPPER_NUM,
                        array[index/WRAPPER_NUM].start,array[index/WRAPPER_NUM].end,
                        array[index/WRAPPER_NUM].test_addr,
                        array[index/WRAPPER_NUM].read_write,array[index/WRAPPER_NUM].test_read_write,
                        array[index/WRAPPER_NUM].negative);

    ArmWrapper_DataAbort_Test(test_loop++);
}

void ArmWrapper_DataAbort_Test (unsigned int index)
{
    if (index/WRAPPER_NUM >= (sizeof(array)/sizeof(struct test_set)) )
    {
        printf("\ntest successfully finished\n");
        while(1);
    }

    ArmWrapper_DataAbort_Test_(index);
}

void bad_mode (void);

void ArmWrapper_DataAbort_Check (struct pt_regs *pt_regs)
{
    unsigned int negative_int_status=rtd_inl(MEM_TRASH_DBG_INT)&0x4;
    unsigned int fsr,lpae=0,status,rw;
    asm volatile("mrc p15, 0, %0, c5, c0, 0" : "=r" (fsr));
    lpae=(fsr&0x200) >>9;
    status=lpae?(fsr&0x3F):( (fsr&0xF) | (fsr&0x400)>>6);
    //printf("fsr=%x  lpae=%x status=%x       INT=%x\n",fsr,lpae,status,rtd_inl(MEM_TRASH_DBG_INT));
    
    if (negative_flag && (!negative_int_status))
    {
        return;
    }
    else if ( !negative_flag && !(rtd_inl(MEM_TRASH_DBG_INT)>>4))
    {
        return;
    }

    if(lpae && status!=0x10 && status !=0x11 )
    {
        return;
    }
    else if(status!=0x8 && status!=0x16)
    {
        return;
    }

    if (lpae)
    {
        rw=(status==0x10)?0:1;
    }
    else
    {
        rw=(status==0x8)?0:1;
    }
    
    if (flag_data_abort== -1)
    {
        printf ("arm wrapper data abort:    trash addr=0x%08x,  trash type=%s\n",rtd_inl(MEM_TRASH_DBG_ADDR),rtd_inl(MEM_TRASH_DBG_RW)?"write":"read");
        show_regs (pt_regs);
        bad_mode();
        return;
    }

    if(flag_data_abort==0)
    {
        printf(print_str[1]);
        printf("        trash addr=0x%x,type=%x\n",rtd_inl(MEM_TRASH_DBG_ADDR),rtd_inl(MEM_TRASH_DBG_RW));
        while(1);
    }

    if (  (set_rw    == rtd_inl(MEM_TRASH_DBG_RW)) && (set_rw==rw) &&
            (set_addr == rtd_inl(MEM_TRASH_DBG_ADDR)) )
    {
        char* str_trash[2]={"\e[1;31mread\e[0m  trash","\e[1;31mwrite\e[0m trash"};
        printf(print_str[0]);
        //printf("        trash addr=0x%x,type=%x\n",rtd_inl(MEM_TRASH_DBG_ADDR),rtd_inl(MEM_TRASH_DBG_RW));
        printf("        %s 0x%x\n",str_trash[rtd_inl(MEM_TRASH_DBG_RW)&0x1],rtd_inl(MEM_TRASH_DBG_ADDR));
        rtd_outl(MEM_TRASH_DBG_INT, 0x10);
        flag_data_abort=0;

        ArmWrapper_DataAbort_Test(test_loop++);
    }
}


int arm_wrapper_function_test ( cmd_tbl_t *cmdtp, int flag, int argc, char * const argv[])
{
    dcache_disable();
    ArmWrapper_DataAbort_Test(test_loop++);
    return 0;
}

U_BOOT_CMD(
    arm_wrapper,      2,      1,      arm_wrapper_function_test,
    "start arm_wrapper test",
    ""
);
