#include "cmd_hwmonitor.h"

#define HWM_CMD_BUF_LEN (256)
#define HWM_ENV_RANGE_DFT        "hwm_range"
char hwm_cmd_buf[HWM_CMD_BUF_LEN];
menu_state state;
extern setmenu top_menu[];

/****COMMON FUNC AREA****/
void prepare_go_ahead(void)
{
        if(state.level >= MENU_LEVEL_MAX)
        {
                return;
        }

        snprintf(state.path[state.level],MENU_NAME_LEN,state.menu[0].name);
        state.level++;
}

void prepare_go_back(void)
{
        if(state.level > 0)
        {
                state.level--;
        }
}

int go_up_menu(void)
{
        prepare_go_back();
        state.menu = state.menu[0].father;
        return 0;
}

int go_quick_exit(void)
{
        state.level = 0;
        state.menu = NULL;
        return 0;
}

void show_now_path(void)
{
        int i = 0;
        printf(FBLUE_START);
        for(i = 0;i < state.level;++i)
        {
                if(i != 0)
                {
                        printf(" -> ");
                }
                else
                {
                        printf("menu path: ");
                }
                printf("%s",state.path[i]);
        }
        printf("\n" FCOLOR_END);
}

int go_exit(void)
{
        printf("******** bye-bye! ********\n");
        return 0;
}

void show_menu(setmenu * p)
{
        int i = 0;

        printf("%s\n", p[0].desc);//print title

        i = 1;
        while(p[i].name != NULL)//print items
        {
                printf("[%d] %-16s : %s\n", i, p[i].name, p[i].desc);
                i++;
        }
}

int get_menu_len(setmenu * p)
{
        int i = 0;
        while(p[i].name != NULL)//print items
        {
                i++;
        }

        return i-1;//don't count p[0](p[0] is a title item)
}

void dump_buffer_data(char * buf,int len)
{
        int i = 0;
        for(i = 0;i < len;++i)
        {
                if(i % 8 == 0)
                {
                        printf("\n%4d:",i);
                }
                printf(" %02x(%c) ",buf[i],buf[i]); //[CID_30456] too_few_printf_args: the format string requires additional arguments
        }
}

void menu_wait(setmenu * p)
{
        int index = 0;
        char buf[BOOT_PARAM_LEN_MAX] = {0};
        int len;

        printf(
"\n");
        show_now_path();
        show_menu(p);
        len = get_menu_len(p);

        index = 0;
        while((index <= 0) || (index > len))
        {
                printf("\n'q'+'ENTER' to go back\n");
                printf("'Q'+'ENTER' to go exit\n");
                printf("\ninput chooice: ");
                fgets(buf,sizeof(buf));
                //dump_buffer_data(buf,4);
                if(buf[0] == 'q') //if use 'q', go back
                {
                        printf("\n");
                        go_up_menu();
                        break;
                }
                else if(buf[0] == 'Q') //if use 'Q', go back
                {
                        printf("\n");
                        go_quick_exit();
                        return;
                }
                index = simple_strtoul(buf, NULL, 16);
                printf("\n");
                //printf("\nget chooice:%d\n", index);
        }

        if(p[index].func != NULL)
        {
                (*(p[index].func))();
        }
}

int append_bootargs(char * str)
{
        char *commandline = getenv("bootargs");
        char *tmp_cmdline = NULL;
        char *value;
        int  len          = 0;

        if(str == NULL)
        {
                return -1;
        }

        value = getenv(str);

        if (value == NULL)
        {
                return -1;
        }

        len = strlen(commandline) + strlen(str) + strlen(value) + 3;

        tmp_cmdline = (char *)malloc(len);
        if (!tmp_cmdline)
        {
                printf("[ERR] %s: Malloc failed\n", __func__);
                return -1;
        }

        memset(tmp_cmdline, 0, len);

        snprintf(tmp_cmdline, len, "%s %s=%s", commandline, str,value);

        setenv("bootargs", tmp_cmdline);

        free(tmp_cmdline);

        return 0;
}

void show_boot_param(char * key)
{
        char * cmdline = NULL;

        cmdline = getenv(key);
        if(cmdline != NULL)
        {
                printf("[%s]:%s\n",key,cmdline);
        }
        else
        {
                printf("[%s] is NULL!\n",key);
        }
}

int update_boot_param(char * key,char * buf)
{
        char cmd[BOOT_PARAM_LEN_MAX];

        memset(cmd,0,sizeof(cmd));

        snprintf(cmd,sizeof(cmd),"env set %s %s",key,buf);

        if (run_command(cmd,0)!=0)
        {
                printf("%s failed.  env set failed.\n",cmd);
                return -1;
        }

        if (run_command("saveenv",0)!=0)
        {
                printf("%s failed.  saveenv failed.\n",cmd);
                return -1;
        }

        return 0;
}


int add_boot_param(char * key,char * buf)
{
        char * cmdline = NULL;
        char cmd[BOOT_PARAM_LEN_MAX];

        memset(cmd,0,sizeof(cmd));

        cmdline = getenv(key);

        if(!cmdline)
        {
                snprintf(cmd,sizeof(cmd),"env set %s %s",key,buf);
        }
        else
        {
                snprintf(cmd,sizeof(cmd),"env set %s %s,%s",key,cmdline,buf);
        }

        if (run_command(cmd,0)!=0)
        {
                printf("%s failed.  env set failed.\n",cmd);
                return -1;
        }

        if (run_command("saveenv",0)!=0)
        {
                printf("%s failed.  saveenv failed.\n",cmd);
                return -1;
        }

        return 0;
}

int clear_boot_param(char * key)
{
        char cmd[128];

        memset(cmd,0,sizeof(cmd));

        snprintf(cmd,sizeof(cmd),"env set %s ",key);

        if (run_command(cmd,0)!=0)
        {
                printf("%s failed.  env set failed.\n",cmd);
                return -1;
        }

        if (run_command("saveenv",0)!=0)
        {
                printf("%s failed.  saveenv failed.\n",cmd);
                return -1;
        }

        return 0;
}

/****DCMT LOGIC AREA****/
int go_dcmt_top(void)
{
        state.menu = dcmt_root_menu;
        prepare_go_ahead();
        return 0;
}

/****SB2 LOGIC AREA****/
int go_sb2_top(void)
{
        prepare_go_ahead();
        return 0;
}

/****ARM_WRAPPER LOGIC AREA****/
int go_armwrapper_top(void)
{
        prepare_go_ahead();
        return 0;
}

/****HW MONITOR LOGIC AREA****/
int go_hwm_range_dft(void)
{
        char* buf = hwm_cmd_buf;
        char * cmdline = NULL;

        printf("\n******** HW monitor default range monitor ********\n");
        memset(buf,0,HWM_CMD_BUF_LEN);
        cmdline = getenv(HWM_ENV_RANGE_DFT);
        DBG_YELLOW("HW monitor default range param is : %s\n",cmdline);
        printf("\ndo you want HW monitor to monitor default range?(Yes/No/Default):");
        fgets(buf,HWM_CMD_BUF_LEN);
        DBG_YELLOW("\nyour chooice is:%s\n",buf);

        if((strcmp(buf,"y") == 0)||(strcmp(buf,"Y") == 0)||(strcmp(buf,"yes") == 0)||(strcmp(buf,"YES") == 0)||(strcmp(buf,"Yes") == 0))
        {
                update_boot_param(HWM_ENV_RANGE_DFT,"y");
                DBG_YELLOW("\nupdate HW monitor default range param as below:\n");
                show_boot_param(HWM_ENV_RANGE_DFT);
        }
        else if((strcmp(buf,"n") == 0)||(strcmp(buf,"N") == 0)||(strcmp(buf,"no") == 0)||(strcmp(buf,"NO") == 0)||(strcmp(buf,"No") == 0))
        {
                update_boot_param(HWM_ENV_RANGE_DFT,"n");
        }
        else if((strcmp(buf,"d") == 0)||(strcmp(buf,"D") == 0)||(strcmp(buf,"default") == 0)||(strcmp(buf,"DEFAULT") == 0)||(strcmp(buf,"Default") == 0))
        {
                clear_boot_param(HWM_ENV_RANGE_DFT);
        }
        else
        {
                printf("param not changed...\n");
        }

        cmdline = getenv(HWM_ENV_RANGE_DFT);
        DBG_YELLOW("\nupdate disable DCMT driver param to : %s\n",cmdline);

        return 0;
}

int go_hwm_param_default(void)
{
        char* buf = hwm_cmd_buf;

        printf("\n******** hwm default param flow ********\n");
        memset(buf,0,HWM_CMD_BUF_LEN);
        DBG_YELLOW("hwm boot param is as below:\n");
        show_boot_param(HWM_ENV_RANGE_DFT);
        printf("\ndo you want to clear them all?(Yes/No):");
        fgets(buf,HWM_CMD_BUF_LEN);
        DBG_YELLOW("\nyour chooice is:%s\n",buf);

        if((strcmp(buf,"y") == 0)||(strcmp(buf,"Y") == 0)||(strcmp(buf,"yes") == 0)||(strcmp(buf,"YES") == 0)||(strcmp(buf,"Yes") == 0))
        {
                clear_boot_param(HWM_ENV_RANGE_DFT);

                DBG_YELLOW("\nupdate dcmt boot param is as below:\n");
                show_boot_param(HWM_ENV_RANGE_DFT);
        }
        else
        {
                printf("param not changed...\n");
        }

        return 0;
}

int go_hwm_param_show(void)
{
        char* buf = hwm_cmd_buf;

        //cmdline=getenv(str);
        memset(buf,0,HWM_CMD_BUF_LEN);
        show_boot_param(HWM_ENV_RANGE_DFT);

        //run_command(cmd,0); //cmd = "env set %s %x"
        printf("\npress 'ENTER' to continue...\n");
        fgets(buf,HWM_CMD_BUF_LEN);

        return 0;
}

setmenu hwm_root_menu[] =
{
        {"hwm", "******** Welcome to HW monitor set menu ********", NULL, top_menu},
        {"default range", "set DCMT/ARM_WRAPPER/SB2_DBG to monitor default region", go_hwm_range_dft, hwm_root_menu},
        {"show boot param", "show bootcode param for HW monitor", go_hwm_param_show, hwm_root_menu},
        {"default", "restore bootcode param as default setting", go_hwm_param_default, hwm_root_menu},
        {"go back", "go to up menu", go_up_menu, hwm_root_menu},
        {NULL, NULL, NULL, NULL},
};

int go_hwm_top(void)
{
        state.menu = hwm_root_menu;
        prepare_go_ahead();
        return 0;
}

int rtk_plat_boot_prep_hwm(void)
{
        append_bootargs(HWM_ENV_RANGE_DFT);

        return 0;
}

/****MAIN FRAME AREA****/
setmenu top_menu[] =
{
        {"root", "******** Welcome to hw-monitor set menu ********", NULL, NULL}, //title
        {"DCMT", "DCMT setting", go_dcmt_top, NULL},
        {"SB2", "SB2 setting", go_sb2_top, NULL},
        {"ARM WRAPPER", "ARM WRAPPER setting", go_armwrapper_top, NULL},
        {"HW MONITOR", "HW MONITOR setting", go_hwm_top, NULL},
        {"exit", "go exit", go_up_menu, NULL},
        {NULL, NULL, NULL, NULL},
};

void hwmonitor_help(void)
{
        printf("\n******** hwmonitor help information ********\n");
        printf(FYELLOW_START"1) hwmonitor manual"FCOLOR_END
                  "\t -->enter hwmonitor manual menu\n");
        printf(FYELLOW_START"2) hwmonitor dcmt"FCOLOR_END
                  "\t -->quick set dcmt cmd to env\n");
        printf(FYELLOW_START"3) hwmonitor sb2"FCOLOR_END
                  "\t -->quick set sb2 cmd to env\n\n");
}

int hwmonitor_set (cmd_tbl_t *cmdtp, int flag, int argc, char * const argv[])
{
        if(argc<2)
        {
                hwmonitor_help();
                return 0;
        }
        else if(argc>=2)
        {
               if(argv[1] == NULL)
               {
                    hwmonitor_help();
                    return 0;
               }
               
               if(!strcmp("manual", argv[1]))//enter manual menu
               {
                        state.menu = top_menu;
                        prepare_go_ahead();
                        while(state.menu != NULL)
                        {
                                menu_wait(state.menu);
                        }
                        go_exit();
                        
               }else if(!strcmp("dcmt", argv[1])){//action dcmt quick setting
                        if(hwmonitor_dcmt_quick_set(argv[2], argv[3]) < 0)
                            DBG_YELLOW("Please re-input dcmt cmd\n");
               }
               else if(strcmp("sb2", argv[1])==0)
               {
               //action sb2 quick setting
                    if(hwmonitor_sb2_quick_set(argv[2], argv[3]) < 0)
                    DBG_YELLOW("Please re-input sb2 cmd\n");

               }
               else{
                    hwmonitor_help();
               }
        }
        
        return 0;
}

U_BOOT_CMD(
        hwmonitor,   CONFIG_SYS_MAXARGS,      0,      hwmonitor_set,
        "hw monitor setting",
        ""
);

