#include <linux/compiler.h>
#include "../../src/include/project_config.h"
#include <command.h>
#include <common.h>
#include <malloc.h>
#include <factorylib.h>

// turn it on to init rtd logbuf param
#ifdef Config_RTD_LOGBUF_TRUE

#define RTD_LOGBUF_VERSION 6
#define RTD_LOGBUF_MAGIC_1 0xBFBFBFBF
#define RTD_LOGBUF_MAGIC_2 0x20160724
#define RTD_LOGBUF_MAGIC_3 0x1234ABCD
#define RTD_LOGBUF_MAGIC_4 0xBFBFBFBF
#define RTD_MAGIC_SAVELEVEL 0x4321DCBA

#define RTD_LOGBUF_START_ADDR 0x1CA00000
#define RTD_LOGBUF_LIMIT 0x80000
#define RTD_LOGBUF_RBCB_OFFSET 0x100
#define RTD_LOGBUF_RBCB_OFFSET_64 0x50
#define RTD_LOGBUF_CMDLINE_OFFSET 0xe00
#define RTD_LOGBUF_CMDLINE_SIZE 0x200

#define RTD_LOGBUF_ARG_NUM 5
#define RTD_LOGBUF_ARG_EX_NUM 7
#define RTD_LOGBUF_CPU_TYPE_NUM 7
#define RTD_LOGBUF_LOGFILE_NUM 7
#define RTD_LOGBUF_BUFSIZE_DEFAULT_NUM 4

/*
LOGSWI:
bit31: logswi setting valid (1:valid 0:invalid)
bit[30~15]: reserved
bit[14~8]: HAL(bit14),KERNEL(bit13),KCPU/OPTEE(bit12),V2(bit11),
                V1(bit10),A2(bit9),A1(bit8) logbuffer enable (1:enable 0:disable)
bit7: rtd_prelogbuf.bin (1:enable 0:disable)
bit6: rtd_logfile.log (1:enable 0:disable)
bit[5~0]: show log on console for HAL(bit5),KCPU/OPTEE(bit4),
              V2(bit3),V1(bit2),A2(bit1),A1(bit0) (1:enable 0:disable)

the meaning of "logswi = 0x8000ff60" is
1.enable logbuf setting
2.enable A1,A2,V1,V2,KCPU/OPTEE,KERNEL,HAL logbuf
3.enalbe rtd_logfile.log
4.enable HAL log to console by logbuf read thread

the priority of logswi define: 
Config_RTD_LOGBUF_LOGSWI_Value(inc file) > RTD_LOGBUF_LOGSWI_CUSTOMER(customer header file) > RTD_LOGBUF_LOGSWI_DEFAULT(default value)

*/
#ifdef Config_RTD_LOGBUF_LOGSWI_Value

#define RTD_LOGBUF_LOGSWI_DEFAULT Config_RTD_LOGBUF_LOGSWI_Value

#else   // Config_RTD_LOGBUF_LOGSWI_Value

#ifdef RTD_LOGBUF_LOGSWI_CUSTOMER
#define RTD_LOGBUF_LOGSWI_DEFAULT RTD_LOGBUF_LOGSWI_CUSTOMER            // logswi. Transfer parameters defaut value
#else // RTD_LOGBUF_LOGSWI_CUSTOMER
#define RTD_LOGBUF_LOGSWI_DEFAULT "8000ff60"            // logswi. Transfer parameters defaut value
#endif  // RTD_LOGBUF_LOGSWI_CUSTOMER

#endif // Config_RTD_LOGBUF_LOGSWI_Value

#define RTD_LOGBUF_LOGFILE_DEFAULT "9000aaaa"           // logfile. Transfer parameters defaut value
#define RTD_LOGBUF_LOGDIR_DEFAULT "/tmp/var/log/"    // logdir. Transfer parameters defaut value
#define RTD_LOGBUF_BUFSIZE_DEFAULT "512K"                // bufsize. Transfer parameters defaut value

#define ENABLE_HALLOG 0x00002020
#define ENABLE_TOTAL_FILE 0x00000040
#define ENABLE_PRELOG 0x00000080
#define LOGSWI_BUFFER_MASK 0xFFFFFF00   //console log off, buffer still on
#define RTD_FILESIZE_INIFINITE 0xFFFFFFFF
#define RTD_LOGBUF_HAL_MODULE 0xFFFFFFFF
#define RTD_DEFAULT_SAVELEVEL 7    //default savelevel 

#define K 1024
#define M (1024*1024)

#define CMD_BUFSIZE 128
#define SYS_SOC_284x "rtd284x"
#define LOGBUF_ENV_NAME "logbuf"

typedef enum EnumLogbufType {
        ACPU1_BUF = 0,
        ACPU2_BUF,
        VCPU1_BUF,
        VCPU2_BUF,
        KCPU_BUF,
        KERNEL_BUF,
        HAL_BUF,
        MAX_LOGBUF_NUM
} EnumLogbufType;

const u32 logbuf_phy_offset[MAX_LOGBUF_NUM+1] = {
        0x0000e000, //acpu1
        0x0001e000, //acpu2
        0x00020000, //vcpu1
        0x00030000, //vcpu2
        0x00040000, //kcpu
        0x00050000, //kernel
        0x00070000, //hal
        RTD_LOGBUF_LIMIT //logbuf limit
};

typedef struct logbuf_global_status {
        volatile u64 nsec;      //time stamp
        volatile u32 magic_seg1;   //magic number Seg1 for logbuf
        volatile u32 magic_seg2;   //magic number Seg2 for logbuf
        volatile u32 magic_seg3;   //magic number Seg3 for logbuf
        volatile u32 magic_seg4;   //magic number Seg4 for logbuf
        volatile u32 rbcb_addr[MAX_LOGBUF_NUM];//logbuf ring_buffer_control_block phy address
        volatile u8 version;  //rtd logbuf sw version
        volatile u8 reserve_1;
        volatile u8 reserve_2;
        volatile u8 reserve_3;
        volatile u32 magic_savelevel;   //magic number for savelevel
} logbuf_global_status; //rtd logbuf global status

typedef struct kfifo
{
        unsigned char *buffer;  /* the buffer holding the data */
        unsigned int size;  /* the size of the allocated buffer */
        unsigned int in;    /* data is added at offset (in % size) */
        unsigned int out;   /* data is extracted from off. (out % size) */
        unsigned int out_cat;   /* data out pos for logcat */
}rtdlog_kfifo;

typedef struct logbuf_rbcb {
        u32 reserve_a;
        u32 reserve_b;
        u32 reserve_c;
        u32 drop_cnt; //counter of dropped logs
        u32 in_reset; //keep old value of kfifo->in to record the old log end offset
        rtdlog_kfifo fifo ; // logbuf size
        volatile u8 savelevel;
} logbuf_rbcb;
// for 64bit
typedef struct kfifo_64
{
        u64 size;  /* the size of the allocated buffer */
        u64 in;    /* data is added at offset (in % size) */
        u64 out;   /* data is extracted from off. (out % size) */
        u64 out_cat;   /* data out pos for logcat */
        unsigned char *buffer;  /* the buffer holding the data */
}rtdlog_kfifo_64;

typedef struct logbuf_rbcb_64 {
        u64 reserve_a;
        u64 reserve_b;
        u64 reserve_c;
        u64 drop_cnt; //counter of dropped logs
        u64 in_reset; //keep old value of kfifo->in to record the old log end offset
        rtdlog_kfifo_64 fifo ; // logbuf size
} logbuf_rbcb_64;

typedef struct logbuf_filesize{
        unsigned int total_file;
        unsigned int acpu1_file;
        unsigned int acpu2_file;
        unsigned int vcpu1_file;
        unsigned int vcpu2_file;
        unsigned int reserve1;
        unsigned int reserve2;
        unsigned int reserve3;
}logbuf_filesize;

typedef struct filter_cpu {
        unsigned int module_mask;    //module bit mask: 0-mask 1-nonmask
        unsigned char level;           //level: log level <= this level will put into file
        unsigned char core_mask;       //core bit mask: 1-mask 0-nonmask
        unsigned char reserve1;
        unsigned char reserve2;
} filter_cpu;   //filter for each logbuf type

typedef struct rtdlog_filter {
        filter_cpu sub_mask[MAX_LOGBUF_NUM];    //filter for each logbuf type
        unsigned char buf_cat;        //a1,a2,v1,v2,ker,hal bit-value: 1-read thread skip 0-normal
        unsigned char buf_mask;        //a1,a2,v1,v2,ker,hal bit-value : 1-mask 0-nonmask
        unsigned char level_mask;   //level_warning: log level <= this level,ignore buf_mask ,put into file
        unsigned char reserve2;     // control hal log show on console (log level <= level_hal), 
} rtdlog_filter;
#define LOGBUF_DIR_LEN 64
typedef struct logbuf_cmdline{
        volatile unsigned int magic_seg1;   //magic number Seg1 for logbuf
        volatile unsigned int magic_seg2;   //magic number Seg2 for logbuf
        volatile unsigned int magic_seg3;   //magic number Seg3 for logbuf
        volatile unsigned int magic_seg4;   //magic number Seg4 for logbuf
        unsigned int logswi;
        unsigned int logfile;
        unsigned int reserve1;
        unsigned int reserve2;
        unsigned char logdir[LOGBUF_DIR_LEN];
        logbuf_filesize filesize;
        rtdlog_filter log_filter;
}logbuf_cmdline;

const char * logbuf_savelevel_name[MAX_LOGBUF_NUM]={
        "acpu1_savelevel",
        "acpu2_savelevel",
        "vcpu1_savelevel",
        "atf_savelevel",
        "optee_savelevel",
        "kernel_savelevel",
        "hal_savelevel"
};

int rtd_get_bufsize_power(void);
int logbuf_get_enable_status(void);

/*
ret 0,magic OK
ret 1,invalid magic
*/
int check_version_and_magic(void)
{
        int ret = 1;
        volatile logbuf_global_status * pStatus = (logbuf_global_status *)(RTD_LOGBUF_START_ADDR);
        if( (pStatus->version == RTD_LOGBUF_VERSION)&&
            (pStatus->magic_seg1 == RTD_LOGBUF_MAGIC_1)&&
            (pStatus->magic_seg2 == RTD_LOGBUF_MAGIC_2)&&
            (pStatus->magic_seg3 == RTD_LOGBUF_MAGIC_3)&&
            (pStatus->magic_seg4 == RTD_LOGBUF_MAGIC_4) )
        {
                ret = 0;
        }
        return ret;
}

/* 0: not set filesize, kernel use default set; not 0: set filesize*/
static unsigned int logbuf_get_env_value(char * str)
{
        char *env_value = NULL;
        unsigned int filesize =0;

        env_value=getenv(str);
        if(env_value != NULL){
                filesize = simple_strtoul(env_value, NULL, 16);
        }

        return filesize;
}

static unsigned char logbuf_get_save_level(const char *str)
{
        char *env_value = NULL;
        unsigned char ret_value = RTD_DEFAULT_SAVELEVEL;

        env_value = getenv(str);

        if(env_value != NULL){
                ret_value = simple_strtoul(env_value, NULL, 16);
        }

        return ret_value;
}

static unsigned char logbuf_get_hal_level(char * str)
{
        char *env_value = NULL;
        unsigned char ret_value = 5;  // hal default loglevel

        env_value=getenv(str);
        if(env_value != NULL){
                ret_value = simple_strtoul(env_value, NULL, 16);
        }

        return ret_value;
}

static unsigned int logbuf_get_hal_module(char * str)
{
        char *env_value = NULL;
        unsigned int ret_value = RTD_LOGBUF_HAL_MODULE;

        env_value=getenv(str);
        if(env_value != NULL){
                ret_value = simple_strtoul(env_value, NULL, 16);
        }

        return ret_value;
}

int init_cmdline_magic_number(logbuf_cmdline * cmdline)
{
        cmdline->magic_seg1 = RTD_LOGBUF_MAGIC_1;
        cmdline->magic_seg2 = RTD_LOGBUF_MAGIC_2;
        cmdline->magic_seg3 = RTD_LOGBUF_MAGIC_3;
        cmdline->magic_seg4 = RTD_LOGBUF_MAGIC_4;

        return 0;
}
unsigned long rtd_get_current_value(char * str);
int init_cmdline_logswi(logbuf_cmdline * cmdline)
{
        cmdline->logswi = (unsigned int)rtd_get_current_value("logswi");
        return 0;
}

int init_cmdline_logfile(logbuf_cmdline * cmdline)
{
        cmdline->logfile= (unsigned int)rtd_get_current_value("logfile");
        return 0;
}
#if 0
u32 rtd_get_bufsize_value(void);
int init_cmdline_bufsize(logbuf_cmdline * cmdline)
{
        cmdline->bufsize= rtd_get_bufsize_value();
        return 0;
}
#endif
int init_cmdline_logdir(logbuf_cmdline * cmdline)
{
        char * logdir = NULL;

        logdir = getenv("logdir");
        if(logdir == NULL){
                snprintf((char*)(cmdline->logdir), LOGBUF_DIR_LEN-1, "%s", RTD_LOGBUF_LOGDIR_DEFAULT);
        }else{
                snprintf((char*)(cmdline->logdir), LOGBUF_DIR_LEN-1, "%s", logdir);
        }
       
        return 0;
}
int init_cmdline_filesize(logbuf_cmdline * cmdline)
{
        cmdline->filesize.total_file = logbuf_get_env_value("total_filesize");
        cmdline->filesize.acpu1_file= logbuf_get_env_value("a1_filesize");
        cmdline->filesize.acpu2_file = logbuf_get_env_value("a2_filesize");
        cmdline->filesize.vcpu1_file= logbuf_get_env_value("v1_filesize");
        cmdline->filesize.vcpu2_file= logbuf_get_env_value("v2_filesize");
        return 0;
}
void init_cmdline_logfilter_submask_hal(logbuf_cmdline * cmdline)
{
        cmdline->log_filter.sub_mask[HAL_BUF].core_mask = 0x0;
        cmdline->log_filter.sub_mask[HAL_BUF].level = logbuf_get_hal_level("hal_level");
        cmdline->log_filter.sub_mask[HAL_BUF].module_mask = (unsigned int)logbuf_get_hal_module("hal_module");
}

int init_cmdline_logfilter_submask(logbuf_cmdline * cmdline)
{
        //ACPU1 BUF
        cmdline->log_filter.sub_mask[ACPU1_BUF].core_mask = 0x0;
        cmdline->log_filter.sub_mask[ACPU1_BUF].level = 7;
        cmdline->log_filter.sub_mask[ACPU1_BUF].module_mask = 0xFFFFFFFF;
        //ACPU2 BUF
        cmdline->log_filter.sub_mask[ACPU2_BUF].core_mask = 0x0;
        cmdline->log_filter.sub_mask[ACPU2_BUF].level = 7;
        cmdline->log_filter.sub_mask[ACPU2_BUF].module_mask = 0xFFFFFFFF;
        //VCPU BUF
        cmdline->log_filter.sub_mask[VCPU1_BUF].core_mask = 0x0;
        cmdline->log_filter.sub_mask[VCPU1_BUF].level = 4;
        cmdline->log_filter.sub_mask[VCPU1_BUF].module_mask = 0xFFFFFFFF;
        //VCPU2 BUF
        cmdline->log_filter.sub_mask[VCPU2_BUF].core_mask = 0x0;
        cmdline->log_filter.sub_mask[VCPU2_BUF].level = 4;
        cmdline->log_filter.sub_mask[VCPU2_BUF].module_mask = 0xFFFFFFFF;
        //KCPU BUF
        cmdline->log_filter.sub_mask[KCPU_BUF].core_mask = 0x0;
        cmdline->log_filter.sub_mask[KCPU_BUF].level = 7;
        cmdline->log_filter.sub_mask[KCPU_BUF].module_mask = 0xFFFFFFFF;
        //KERNEL BUF
        cmdline->log_filter.sub_mask[KERNEL_BUF].core_mask = 0x0;
        cmdline->log_filter.sub_mask[KERNEL_BUF].level = 7;
        cmdline->log_filter.sub_mask[KERNEL_BUF].module_mask = 0xFFFFFFFF;
        //HAL BUF
        init_cmdline_logfilter_submask_hal(cmdline);

        return 0;
}

void init_cmdline_logfilter_others(logbuf_cmdline * cmdline)
{
        cmdline->log_filter.buf_cat = 0;
        cmdline->log_filter.buf_mask = 0;
        cmdline->log_filter.level_mask = 3;
        //cmdline->log_filter.level_hal = logbuf_get_hal_level("hal_level");
}

int init_cmdline_logfilter(logbuf_cmdline * cmdline)
{
        init_cmdline_logfilter_submask(cmdline);
        init_cmdline_logfilter_others(cmdline);
        return 0;
}

int init_logbuf_savelevel(void)
{
        volatile logbuf_global_status * pStatus = (logbuf_global_status *)(RTD_LOGBUF_START_ADDR);
        volatile logbuf_rbcb * pRbcb;
        int cpu_type;

        for(cpu_type = 0; cpu_type < MAX_LOGBUF_NUM; ++cpu_type){
                pRbcb = (logbuf_rbcb *)(pStatus->rbcb_addr[cpu_type]);
                pRbcb->savelevel = logbuf_get_save_level(logbuf_savelevel_name[cpu_type]);
        }

        return 0;
}

int init_logbuf_cmdline_para(void)
{
        logbuf_cmdline * cmdline_p = (logbuf_cmdline *)(RTD_LOGBUF_START_ADDR + RTD_LOGBUF_CMDLINE_OFFSET);  

        memset(cmdline_p, 0, sizeof(logbuf_cmdline));

        /* set magic number */
        init_cmdline_magic_number(cmdline_p);
        /* set bufsize. transfer bufsize by kernel cmdline*/
        //init_cmdline_bufsize(cmdline_p);
        /* set logswi */
        init_cmdline_logswi(cmdline_p);
        /* set logfile */
        init_cmdline_logfile(cmdline_p);
        /* set logdir */
        init_cmdline_logdir(cmdline_p);
        /* set filesize */
        init_cmdline_filesize(cmdline_p);
        /* set logfilter */
        init_cmdline_logfilter(cmdline_p);
        return 0;

}

void reset_logbuf_para(int check_result)
{
        int i;
        //init global status
        volatile logbuf_global_status * pStatus = (logbuf_global_status *)(RTD_LOGBUF_START_ADDR);
        volatile logbuf_rbcb * pRbcb;
        volatile logbuf_rbcb_64 * pRbcb_64;
        int  bufsize_power = rtd_get_bufsize_power();

        if(logbuf_get_enable_status() <= 0){  // logbuf disable
                memset((void*)pStatus, 0, (RTD_LOGBUF_LIMIT<<bufsize_power) -1);
                return;
        }

        pStatus->nsec = 0;
        pStatus->magic_savelevel = RTD_MAGIC_SAVELEVEL;

        //rtd_reset_logbuf_phy_offset(bufsize_power);     //reset logbuf_phy_offset;
        if(check_result){
                pStatus->version = RTD_LOGBUF_VERSION;
                pStatus->magic_seg1 = RTD_LOGBUF_MAGIC_1;
                pStatus->magic_seg2 = RTD_LOGBUF_MAGIC_2;
                pStatus->magic_seg3 = RTD_LOGBUF_MAGIC_3;
                pStatus->magic_seg4 = RTD_LOGBUF_MAGIC_4;
        }

        for(i = 0; i < MAX_LOGBUF_NUM; ++i) {
                pStatus->rbcb_addr[i] = RTD_LOGBUF_START_ADDR + RTD_LOGBUF_RBCB_OFFSET*(i+1);
        }
        //init rbcb
        for(i = 0; i < MAX_LOGBUF_NUM; ++i) {
                pRbcb = (logbuf_rbcb *)(pStatus->rbcb_addr[i]);
                pRbcb_64 = (logbuf_rbcb_64 *)(pStatus->rbcb_addr[i] + RTD_LOGBUF_RBCB_OFFSET_64);

                if(check_result){
                        memset((void*)pRbcb, 0, RTD_LOGBUF_RBCB_OFFSET);
                }else{
                        pRbcb->drop_cnt = 0;
                        pRbcb->in_reset = pRbcb->fifo.in;
                        pRbcb->fifo.out = pRbcb->fifo.in;
                        pRbcb->fifo.out_cat = pRbcb->fifo.in;
                        // 64bit region
                        pRbcb_64->drop_cnt = 0;
                        pRbcb_64->in_reset = pRbcb_64->fifo.in;
                        pRbcb_64->fifo.out = pRbcb_64->fifo.in;
                        pRbcb_64->fifo.out_cat= pRbcb_64->fifo.in;
                }
                pRbcb->fifo.buffer = (unsigned char*)(RTD_LOGBUF_START_ADDR + (logbuf_phy_offset[i]<<bufsize_power));
                pRbcb->fifo.size = (logbuf_phy_offset[i+1]<<bufsize_power) - (logbuf_phy_offset[i]<<bufsize_power);
                // 64bit region
                pRbcb_64->fifo.buffer = pRbcb->fifo.buffer;
                pRbcb_64->fifo.size = (u64)(pRbcb->fifo.size);
#ifndef Config_Jump_64Bit_Kernel_TRUE
                printf("logbuf fifo[%d] [buffer=%x], [size=%x]\n", i,(unsigned int)(pRbcb->fifo.buffer), pRbcb->fifo.size);
#else
                printf("[64bit] logbuf fifo[%d] [buffer=%lx], [size=%llx]\n", i,(unsigned long)(pRbcb_64->fifo.buffer), pRbcb_64->fifo.size);
#endif
        }
        /* init logbuf cmdline, transfer boot args by DDR*/
        init_logbuf_cmdline_para();
        init_logbuf_savelevel();
        flush_dcache_all();
}

int init_rtd_logbuf(void)
{
        int check_result = check_version_and_magic();

        reset_logbuf_para(check_result);
        return 0;
}


///***************************LOGBUF CMD*********************///
typedef enum LogbufArgType {
        BUFSIZE = 0,
        LOGSWI,
        LOGDIR,
        LOGFILE,
        SAVELEVEL
} LogbufArgType;

typedef enum LogbufArgTypeEX {
        ACPU_HEADER = 0,
        ACPU_WRAP,
        AVLOG_IN_TOTAL,
        //TOTAL_FILESIZE,
        //TOTAL_FILE,
        PRELOG,
        HAL,
        HAL_MODULE,
        HAL_LEVEL,
} LogbufArgTypeEX;

typedef enum LogbufParamType {
        ARG_TYPE = 0,
        ARG_TYPE_EX,
        CPU_TYPE,
        LOGBUF_EN
} LogbufParamType;




typedef enum LogbufParam {
        RTD_BUFSIZE = 0,
        RTD_LOGSWI,
        RTD_LOGDIR,
        RTD_LOGFILE,
        RTD_FILESIZE,
        RTD_LOGFILTER,
        RTD_SAVELEVEL,
        PARAM_NUM
} LogbufParam;

typedef struct lobuf_param_name{
        LogbufParam type;
        const char name[32];
}lobuf_param_name;

static const lobuf_param_name logbuf_param_type[]={
        // bufsize
        {RTD_BUFSIZE, "bufsize"},

        // logswi
        {RTD_LOGSWI, "logswi"},
        {RTD_LOGSWI, "prelog"},
        {RTD_LOGSWI, "hal"},

        // logdir
        {RTD_LOGDIR, "logdir"},

        // logfile
        {RTD_LOGFILE, "logfile"},
        {RTD_LOGFILE, "acpu_header"},
        {RTD_LOGFILE, "acpu_wrap"},
        {RTD_LOGFILE, "avlog_in_total"},

        // filesize
        {RTD_FILESIZE, "a1_filesize"},
        {RTD_FILESIZE, "a2_filesize"},
        {RTD_FILESIZE, "v1_filesize"},
        {RTD_FILESIZE, "v2_filesize"},
        {RTD_FILESIZE, "total_filesize"},

        // logfilter

        // savelevel
        {RTD_SAVELEVEL, "savelevel"},
};


const char * logbuf_arg_type[RTD_LOGBUF_ARG_NUM]={
        "bufsize",
        "logswi",
        "logdir",
        "logfile",
        "savelevel"
};
const char * logbuf_arg_type_ex[RTD_LOGBUF_ARG_EX_NUM]={
        "acpu_header",
        "acpu_wrap",
        "avlog_in_total",
        "prelog",        //rtd_prelogbuf.bin
        "hal",
        "hal_module",
        "hal_level"
};
const char * logbuf_cpu_type[RTD_LOGBUF_CPU_TYPE_NUM]={        
        "acpu1",
        "acpu2",
        "acpu",
        "vcpu1",        
        "vcpu2",
        "vcpu",
        "avcpu"
};

const char * logbuf_buf_type[MAX_LOGBUF_NUM+1]={
        "acpu1",
        "acpu2",
        "vcpu1",
        "vcpu2",
        "kcpu1",
        "s-ker",
        "s-hal",
        "all"
};

const char * logbuf_filesize_name[RTD_LOGBUF_LOGFILE_NUM]={        
        "a1_filesize",
        "a2_filesize",
        NULL,
        "v1_filesize",        
        "v2_filesize",
        NULL,
        NULL       //rtd_logfile.log
};

unsigned long av_on_value[RTD_LOGBUF_CPU_TYPE_NUM] = {
        0x00000101,     //acpu1 on 
        0x00000202,     //acpu2 on
        0x00000303,     //acpu1 acpu2 on
        0x00000404,     //vcpu1 on
        0x00000808,     //vcpu2 on
        0x00000c0c,     //vcpu1 vcpu2 on
        0x00000f0f      //acpu1,acpu2,vcpu1,vcpu2 on
};
unsigned long wrap_on_value[RTD_LOGBUF_CPU_TYPE_NUM] = {
        0x00000002,     //acpu1 on 
        0x00000020,     //acpu2 on
        0x00000022,     //acpu1 acpu2 on
        0x00000200,     //vcpu1 on
        0x00002000,     //vcpu2 on
        0x00002200,     //vcpu1 vcpu2 on
        0x00002222      //acpu1,acpu2,vcpu1,vcpu2 on
};
unsigned long header_on_value[RTD_LOGBUF_CPU_TYPE_NUM] = {
        0x00000001,     //acpu1 on 
        0x00000010,     //acpu2 on
        0x00000011,     //acpu1 acpu2 on
        0x00000100,     //vcpu1 on
        0x00001000,     //vcpu2 on
        0x00001100,     //vcpu1 vcpu2 on
        0x00001111      //acpu1,acpu2,vcpu1,vcpu2 on
};

const char * bufsize_default[RTD_LOGBUF_BUFSIZE_DEFAULT_NUM]={        
        "512K",
        "1M",
        "2M",
        "4M"
};

static int get_param_type(char *str)
{
        int i;
        int type_num = sizeof(logbuf_param_type)/sizeof(lobuf_param_name);

        for(i=0; i<type_num; i++){
                if(strcmp(str, logbuf_param_type[i].name) == 0){
                        return logbuf_param_type[i].type;
                }

        }
        return -1;
}

static int update_ddr_param(LogbufParam type)
{
        logbuf_cmdline * cmdline_p = (logbuf_cmdline *)(RTD_LOGBUF_START_ADDR + RTD_LOGBUF_CMDLINE_OFFSET);  
        switch(type)
        {
                case RTD_BUFSIZE:       // transfer bufsize by kernel cmdline
                        //init_cmdline_bufsize(cmdline_p);
                        break;
                case RTD_LOGSWI:
                        init_cmdline_logswi(cmdline_p);
                        break;
                case RTD_LOGDIR:
                        init_cmdline_logdir(cmdline_p);
                        break;
                case RTD_LOGFILE:
                        init_cmdline_logfile(cmdline_p);
                        break;
                case RTD_FILESIZE:
                        init_cmdline_filesize(cmdline_p);
                        break;
                case RTD_LOGFILTER:
                        init_cmdline_logfilter_submask_hal(cmdline_p);
                        init_cmdline_logfilter_others(cmdline_p);
                        break;
                case RTD_SAVELEVEL:
                        init_logbuf_savelevel();
                        break;
                default:
                        printf("[%s]unknown type=%d\n",__FUNCTION__, type);
                        break;
        }
               
        return 0;
}


static int str_is_alldigit(char *str)
{
        int i;
        char c;

        for(i=0; i<strlen(str);i++){
                c = *str;
                if( c >= '0' && c <= '9'){
                        str++;
                        continue;
                }else{
                        return -1;
                }

        }
        return 0;
}

static int logbuf_set_env (char * cmd, int re_init )
{
        if (run_command(cmd,0)!=0)
        {
                printf("%s failed.  env set failed.\n",cmd);
                return -1;
        }

        if (run_command("saveenv",0)!=0)
        {
                printf("%s failed.  saveenv failed.\n",cmd);
                return -1;
        }

        //set bufsize , init_rtdlog again
        if(re_init == 1){
                init_rtd_logbuf();
        }
        return 0;
}

//return: -1:not found, other:foud
static int logbuf_compare_type_str(char *str, const char *type[], int num)
{
        int i;
        
        for(i=0; i<num; i++){
                if(strcmp(str, type[i]) == 0){
                        return i;
                }
        }
        return -1;
}

static int logbuf_get_param_type(char *str)
{

        if(logbuf_compare_type_str(str, logbuf_arg_type, RTD_LOGBUF_ARG_NUM) != -1)
        {
                return ARG_TYPE;
        }

        if(logbuf_compare_type_str(str, logbuf_arg_type_ex, RTD_LOGBUF_ARG_EX_NUM) != -1)
        {
                return ARG_TYPE_EX;
        }

        if(logbuf_compare_type_str(str, logbuf_cpu_type, RTD_LOGBUF_CPU_TYPE_NUM) != -1)
        {
                return CPU_TYPE;
        }

        if((strcmp(str, "on") == 0) || (strcmp(str, "ON") == 0) || (strcmp(str, "off") == 0) || (strcmp(str, "OFF") == 0))
        {
                return LOGBUF_EN;
        }
        return -1;

}


/*******arg_type:  bufsize,logfile,logdir,logswi************/

unsigned long rtd_get_current_value(char * str)
{
        char *cmdline = NULL;
        char *default_value = NULL;
        unsigned long value = 0;


        if(strcmp(str, "logfile") == 0){
                default_value= RTD_LOGBUF_LOGFILE_DEFAULT;
        }else if(strcmp(str, "logswi")==0){
                default_value= RTD_LOGBUF_LOGSWI_DEFAULT;
        }else{
                printf("[%s]input str '%s' error!!!\n",__FUNCTION__,str);
                return 0;
        }

        cmdline=getenv(str);
        if(cmdline != NULL){
                value = simple_strtoul(cmdline, NULL, 16);
        }else{
                value = simple_strtoul(default_value, NULL, 16);
        }

        return value;
}

int rtd_get_bufsize_power(void)
{
        int i;
        int size_power = 0;
        char * cmdline =getenv("bufsize");

        if(cmdline == NULL){
                cmdline = (char *)RTD_LOGBUF_BUFSIZE_DEFAULT;
        }

        for(i=0; i<RTD_LOGBUF_BUFSIZE_DEFAULT_NUM; i++){
                if(strcmp(cmdline, bufsize_default[i]) == 0){
                        size_power = i;
                        break;
                }
        }
        return size_power;      //default size_mult=0, size use RTD_LOGBUF_LIMIT
}

u32 rtd_get_bufsize_value(void)
{
        u32 ret;
        int size_power = rtd_get_bufsize_power();
        
        ret = RTD_LOGBUF_LIMIT << size_power;      //default size_mult=0, size use RTD_LOGBUF_LIMIT
        return ret;
}

static int rtd_check_arg_bufsize(char * arg_value)
{
        int i;

        for(i=0; i<RTD_LOGBUF_BUFSIZE_DEFAULT_NUM; i++){
                if(strcmp(arg_value, bufsize_default[i]) == 0)
                        return 0;
        }
        printf("Import value '%s' error!!!\n", arg_value);
        return -1;
}

static int rtd_check_arg_logswi_logfile(char * arg_value)
{
        if(strlen(arg_value)!=8){
                printf("Import value '%s' error!!!\n", arg_value);
                //printf("Reference: logbuf logswi %s\n", RTD_LOGBUF_LOGSWI_DEFAULT);
                //printf("Reference: logbuf logfile %s\n", RTD_LOGBUF_LOGFILE_DEFAULT);
                return -1;
        }
        return 0;
}

static int rtd_check_arg_logdir(char * arg_value)
{
        int len = strlen(arg_value);

        if(strcmp(arg_value,"USB")==0){
                return 0;
        }
        if((arg_value[0]!='/') ||(arg_value[len-1]!='/') ){
                printf("Import value '%s' error, miss '/' !!!\n", arg_value);
                //printf("Reference: logbuf logdir /tmp/var/log/\n");
                //printf("                logbuf logdir USB\n");
                return -1;
        }
        return 0;
}

static int logbuf_print_arg_type(int type)
{
        char *cmdline = NULL;
        const char * arg_default_value[RTD_LOGBUF_ARG_NUM]={
                RTD_LOGBUF_BUFSIZE_DEFAULT,
                RTD_LOGBUF_LOGSWI_DEFAULT,
                RTD_LOGBUF_LOGDIR_DEFAULT,
                RTD_LOGBUF_LOGFILE_DEFAULT
        };

        if(type >= RTD_LOGBUF_ARG_NUM){
                printf("[%s]input type=%d error\n",__FUNCTION__, type);
                return -1;
        }

        cmdline = getenv(logbuf_arg_type[type]);
        if(cmdline != NULL){
                printf("%s is \e[33m%s\e[0m\n",logbuf_arg_type[type], cmdline);
        }else{
                printf("%s is \e[33m%s\e[0m\n", logbuf_arg_type[type], arg_default_value[type]);
        }
        return 0;
}

static int logbuf_check_arg_type_value(int type, char * value)
{
        
        if(type==BUFSIZE){
                if(rtd_check_arg_bufsize(value) == -1)
                        return -1;      //arg_value error
        }
        if((type==LOGSWI) || (type==LOGFILE)){                        
                if(rtd_check_arg_logswi_logfile(value) == -1)
                        return -1;
        }
        if(type==LOGDIR){
                if(rtd_check_arg_logdir(value) == -1)
                        return -1;
        }

        return 0;

}
static int logbuf_set_arg_type(int type, char * name, char * value)
{
        char cmd[CMD_BUFSIZE]={0};
        int param_type = -1;

        if(logbuf_check_arg_type_value(type, value) == 0){
                sprintf(cmd,"env set %s %s",name, value);
                if(type == 0){   //bufsize
                        logbuf_set_env(cmd, 1);
                }else{
                        logbuf_set_env(cmd, 0);
                }
                param_type = get_param_type((char *)logbuf_arg_type[type]);
                update_ddr_param(param_type);
                return 0;
        }else{
                return -1;
        }
        
}

static int logbuf_print_arg_type_usage(LogbufArgType type)
{
        switch(type)
        {
                case BUFSIZE:
                        printf("\nbufsize - set logbuf ring buffer size\n");
                        printf("\nbufsize Usage:\n");
                        printf("logbuf bufsize value  -  selectable value 512K, 1M, 2M, 4M .\n\n");
                        break;
                case LOGSWI:
                        printf("\nlogswi - conctrol fw write to file and console log.\n");
                        printf("\nlogswi Usage:\n");
                        printf("logbuf logswi value  -  hex value must 8bit and except 0x, default value is %s.\n\n", RTD_LOGBUF_LOGSWI_DEFAULT);
                        break;
                case LOGDIR:
                        printf("\nlogdir - set logfile path\n");
                        printf("\nlogdir Usage:\n");
                        printf("logbuf logdir value  -  path must contain '/' except for 'USB', default value is %s.\n\n", RTD_LOGBUF_LOGDIR_DEFAULT);
                        break;
                case LOGFILE:
                        printf("\nlogfile Usage:\n");
                        printf("logbuf logfile value  -  hex value must 8bit and except 0x, default value is %s.\n\n", RTD_LOGBUF_LOGFILE_DEFAULT);
                        //printf("  value: except 0x \n");
                        break;
                default:
                        printf("[%s]unknown type=%d\n",__FUNCTION__, type);
                        break;

        }
        return 0;
}

static int logbuf_handle_bufsize_logswi_logdir(int type, int argc, char * const argv[])
{
        if(argc == 2){
                //printf
                printf("current ");
                logbuf_print_arg_type(type);                
                logbuf_print_arg_type_usage(type);
                return 0;
                
        }else if(argc ==3){
                //set
                if(logbuf_set_arg_type(type, argv[1], argv[2])==0){
                        printf("current ");
                        logbuf_print_arg_type(type);        //print set value
                }else{
                        logbuf_print_arg_type_usage(type);
                }
                return 0;                
        }else{
                printf("ERR: Too much argument!!!\n");
                logbuf_print_arg_type_usage(type);
                return -1;

        }
}

static void logbuf_logfile_print_file(void)
{
        //rtd_show_total_file();
        unsigned long value;
        unsigned long logswi_value = rtd_get_current_value("logswi"); 
        
        if(logswi_value ==0)
                return;
        
        value = logswi_value & ENABLE_TOTAL_FILE;

        if(value == ENABLE_TOTAL_FILE){
                printf("rtd_logfile.log file is \e[33mon\e[0m\n");
        }else {
                printf("rtd_logfile.log file is \e[33moff\e[0m\n");
        }
}

extern char *getenv1(const char *name);

static void logbuf_logfile_print_all_filesize(void)
{
        const char *cmdline = NULL; 
        unsigned long filesize = 0;

        cmdline = getenv1("total_filesize");  //total filesize

        if(cmdline == NULL){
                printf("rtd_logfile.log filesize is \e[33mnot set\e[0m\n");
        }else{
                filesize = simple_strtoul(cmdline, NULL, 16);
                 if(filesize == RTD_FILESIZE_INIFINITE)
                {
                        printf("rtd_logfile.log filesize is \e[33minfinite\e[0m\n");
                }else if((filesize/K) < K){
                        printf("rtd_logfile.log filesize is \e[33m%luK\e[0m\n", (filesize/K));
                }else{
                        printf("rtd_logfile.log filesize is \e[33m%luM\e[0m\n", (filesize/M));
                }
        }   
}

static int _set_filesize(const char * name, unsigned long value)
{        
        char cmd[CMD_BUFSIZE]={0};

        if(name == NULL){
                printf("[%s]cpu type error",__FUNCTION__);
                return -1;
        }
        
        if((strcmp(CONFIG_SYS_SOC, SYS_SOC_284x) == 0) && ((strcmp(name, "a2_filesize") == 0) || (strcmp(name, "v2_filesize") == 0))){
                printf("Note: %s not support acpu2 & vcpu2\n",SYS_SOC_284x);
                return -1;
        }
        sprintf(cmd,"env set %s %lx",name, value);
        logbuf_set_env(cmd, -1);

        update_ddr_param(RTD_LOGFILE);
        return 0;
}
static unsigned long _get_filesize_setvalue(char * value_str)
{
        int unit = 0;
        unsigned long value = 0;
        int len = strlen(value_str);
        char value_str2[20] = {0};

        if((value_str[len-1] == 'k') || (value_str[len-1] == 'K')){
                unit = K;    // 1k
        }else if((value_str[len-1] == 'm') || (value_str[len-1] == 'M')){                
                unit = M;       // 1M
        //}else if(strcmp(value_str, "infinite") == 0){
        }else if(strcmp(value_str, "0") == 0){
                return RTD_FILESIZE_INIFINITE;
        }else{
                printf("input filesize(%s) error!\n", value_str);
                return 0;
        }
        
        strncpy(value_str2,value_str,len-1);
        if(str_is_alldigit(value_str2) == -1){  //all digital number
                printf("input value '%s' error!\n", value_str);
                return 0;
        }
        value = simple_strtoul(value_str2, NULL, 10);
        value = value*unit;     
        return value;
}
static int logbuf_logfile_set_all_filesize(char *value)
{
        unsigned long set_value = _get_filesize_setvalue(value);

        printf("set_value=%lx\n",set_value);
        if(set_value == 0){
                return -1;
        }     
        _set_filesize("total_filesize", set_value);

        return 0;
}

static void logbuf_logfile_print_av_inside(void)
{
        unsigned long value = rtd_get_current_value("logfile"); 
        
        if(value & (0x01 << 29))   //avlog in total 29:bit[29]
                printf("rtd_logfile.log contain av log is \e[33mon\e[0m\n");
        else
                printf("rtd_logfile.log contain av log is \e[33moff\e[0m\n");
}
static int logbuf_logfile_set_av_inside(char *str)
{
        char cmd[CMD_BUFSIZE]={0};
        unsigned long value = rtd_get_current_value("logfile"); 

        if(strcmp(str, "on") == 0){
                value = value | (0x1 << 29);    //bit[29]  = 1
        }else if(strcmp(str, "off") == 0){
                value = value & 0xDFFFFFFF;     //bit[29]  = 0
        }else{
                printf("Import value '%s' error!!!\n", str);
                printf("Selectable value: in  out\n");
                return -1;
        }

        memset(cmd,0,CMD_BUFSIZE);
        sprintf(cmd,"env set logfile %lx", value);
        logbuf_set_env(cmd, 0);

        /*update ddr*/
        update_ddr_param(RTD_LOGFILE);
        return 0;
}

void logbuf_print_filesize_set_warning(void)
{
        printf("\e[34mNote:\e[0m\e[33m 1.the sum of files filesize should smaller than directory.\e[0m\n");
        printf("\e[33m      2.filesize beyond dirctory size may lead side effect\e[0m\n");

}
static int logbuf_set_logfile_all(int argc, char * const argv[])
{
        int ret = -1;
        if((argc < 3) || (argc >=6)) 
                return -1;
        
        if(argc == 3){
                logbuf_logfile_print_all_filesize();    //rtdlogfile.log filesize
                logbuf_logfile_print_av_inside();      //av log in rtdlogfile.log  

                ret =0;

        }else{
                if(strcmp(argv[3], "filesize") == 0){
                        if(argc == 4){  //print filesize
                                logbuf_logfile_print_all_filesize();
                                printf("\nlogfile Usage:\n");
                                printf("logbuf logfile all filesize value   - set rtd_logfile.log file size, selectable value *k , *M, 0 (infinite filesize).\n\n");
                                return 0;

                        }else{          //set filesize
                                if(logbuf_logfile_set_all_filesize(argv[4]) == 0){
                                        logbuf_logfile_print_all_filesize();
                                        logbuf_print_filesize_set_warning();
                                        update_ddr_param(RTD_FILESIZE);
                                        return 0;
                                }
                                printf("\nlogfile Usage:\n");
                                printf("logbuf logfile all filesize value   - set rtd_logfile.log file size, selectable value *k , *M ,0 (infinite filesize).\n\n");
                                return -1;
                        }                               

                }
                else if(strcmp(argv[3], "av_inside") == 0){
                        if(argc == 4){
                                logbuf_logfile_print_av_inside();
                                printf("\nlogfile Usage:\n");
                                printf("logbuf logfile all av_inside value  - rtd_logfile.log contain av log or not, selectable value on/off.\n\n");
                                return 0;

                        }else{
                                if(logbuf_logfile_set_av_inside(argv[4]) == 0){
                                        logbuf_logfile_print_av_inside();
                                        update_ddr_param(RTD_LOGFILE);
                                        return 0;
                                }
                                printf("\nlogfile Usage:\n");
                                printf("logbuf logfile all filesize value   - set rtd_logfile.log file size, selectable value *k , *M ,0 (infinite filesize).\n\n");
                                return -1;
                        }  
                }else{
                        ret =-1;
                        printf("Import value '%s' error!!!\n", argv[3]);
                }

        }
        
        printf("\nlogfile Usage:\n");
        printf("logbuf logfile all filesize value   - set rtd_logfile.log file size, selectable value *k , *M ,0 (infinite filesize).\n");
        printf("logbuf logfile all av_inside value  - rtd_logfile.log contain av log or not, selectable value on/off.\n\n");
        return ret;
}

static int rtd_set_total_file (char *value)
{
        unsigned long set_value = 0;        
        char cmd[CMD_BUFSIZE]={0};
        unsigned long logswi_value = rtd_get_current_value("logswi"); 

        if(logswi_value ==0)
                return -1;

        if(strcmp(value, "on") == 0){
                set_value = logswi_value | ENABLE_TOTAL_FILE;
        }else if(strcmp(value, "off") == 0){
                set_value = logswi_value & (~ENABLE_TOTAL_FILE);
        }else{
                printf("Import value '%s' error!!!\n", value);
                printf("Selectable value: on  off\n");
                return -1;
        }

        sprintf(cmd,"env set logswi %lx", set_value);
        return logbuf_set_env(cmd, -1);
}

static void logbuf_print_savelevel_usage(void)
{
        printf("\nsavelevel Usage:\n");
        printf("logbuf savelevel buf_type value  - set savelevel, value range is between 1 and 7, default value is 7.\n");
        printf("buf_type:selectable value acpu1,acpu2,vcpu1,vcpu2,kcpu,kcpu1,s-ker,s-hal,all\n\n");
}

static void logbuf_print_logfile_usage(void)
{
        printf("\nlogfile Usage:\n");
        printf("logbuf logfile value                - hex value must 8bit and except 0x, default value is %s.\n", RTD_LOGBUF_LOGFILE_DEFAULT);
        printf("logbuf logfile on/off               - enable rtd_logfile.log or not.\n");
        printf("logbuf logfile all filesize value   - set rtd_logfile.log file size, selectable value *k , *M ,0 (infinite filesize).\n");
        printf("logbuf logfile all av_inside value  - rtd_logfile.log contain av log or not, selectable value on/off.\n\n");
}
static int logbuf_set_logfile(int argc, char * const argv[])
{
        if((argc <= 2) || (argc >= 6) ){
                return -1;
        }

        if(strcmp(argv[2], "all") == 0){
                return logbuf_set_logfile_all(argc, argv);

        }else if((strcmp(argv[2], "on") == 0) || (strcmp(argv[2], "off") == 0)){        //logbuf logfile on/off
                if(argc == 3){
                        if(rtd_set_total_file(argv[2]) == 0){
                                printf("\nlogfile current setting:\n");
                                logbuf_logfile_print_file();                //rtdlogfile.log on/off
                                update_ddr_param(RTD_LOGSWI);
                                return 0;
                        }
                }else{
                        printf("ERR: Too much argument!!!\n");
                }
        }else{                          //logbuf logfile value 
                if(argc == 3){
                        if(logbuf_set_arg_type(LOGFILE, argv[1], argv[2])==0){
                                printf("current ");
                                logbuf_print_arg_type(LOGFILE);        //print set value
                                return 0;
                        }         
                }else{
                        printf("ERR: Too much argument!!!\n");
                }
        }         
        logbuf_print_logfile_usage();          
        return -1;
        
}

static void logbuf_show_savelevel_all(const char *str)
{
        if(str != NULL){
                printf("%s",str);
        }
        printf("%-8s %-8s %-8s %-8s %-8s %-8s %-8s\n","acpu1","acpu2","vcpu1","vcpu2","kcpu1","s-ker","s-hal");
        printf("\e[33m%-8d %-8d %-8d %-8d %-8d %-8d %-8d\e[0m\n",logbuf_get_save_level(logbuf_savelevel_name[ACPU1_BUF]),
                logbuf_get_save_level(logbuf_savelevel_name[ACPU2_BUF]),logbuf_get_save_level(logbuf_savelevel_name[VCPU1_BUF]),
                logbuf_get_save_level(logbuf_savelevel_name[VCPU2_BUF]),logbuf_get_save_level(logbuf_savelevel_name[KCPU_BUF]),
                logbuf_get_save_level(logbuf_savelevel_name[KERNEL_BUF]),logbuf_get_save_level(logbuf_savelevel_name[HAL_BUF]));
}

static int logbuf_set_savelevel(int argc, char * const argv[])
{
    int buf_type = 0;
    int i = 0;

    if(argc != 4 || strcmp(argv[1], logbuf_arg_type[SAVELEVEL])){
            return -1;
    }

    buf_type = logbuf_compare_type_str(argv[2], logbuf_buf_type, MAX_LOGBUF_NUM+1);
    if(buf_type < 0 || buf_type > MAX_LOGBUF_NUM){
            return -1;
    }

    if(buf_type == MAX_LOGBUF_NUM){//logbuf savelevel all value
            for(i = 0; i < MAX_LOGBUF_NUM; i++){
                    logbuf_set_arg_type(SAVELEVEL, (char *)logbuf_savelevel_name[i], argv[3]);
            }
    }else{
            logbuf_set_arg_type(SAVELEVEL, (char *)logbuf_savelevel_name[buf_type], argv[3]);
    }

    logbuf_show_savelevel_all("\ncurrent setting:\n");

    return 0;
}

static int logbuf_handle_savelevel(int argc, char * const argv[])
{
        int ret = -1;

        if(argc == 4){
                ret = logbuf_set_savelevel(argc, argv);
        }

        if(ret){
                logbuf_print_savelevel_usage();
        }

        return ret;
}

static int logbuf_handle_logfile(int argc, char * const argv[])
{
        int ret = -1;
        //logbuf logfile
        if(argc == 2){
                //print
                printf("\nlogfile current setting:\n");
                logbuf_print_arg_type(LOGFILE);         //logfile value
                logbuf_logfile_print_file();                //rtdlogfile.log on/off
                logbuf_logfile_print_all_filesize();    //rtdlogfile.log filesize
                logbuf_logfile_print_av_inside();      //av log in rtdlogfile.log  

                ret =0;                
        }else if(argc >2 && argc <6){
                return logbuf_set_logfile(argc, argv);

        }else{
                printf("ERR: Too much argument!!!\n");
                ret =-1;
        }
        logbuf_print_logfile_usage();
        return ret;
}

static int logbuf_handle_arg_type(int argc, char * const argv[])
{
        int arg_type = logbuf_compare_type_str(argv[1], logbuf_arg_type, RTD_LOGBUF_ARG_NUM);

        switch(arg_type)
        {
                case BUFSIZE:
                case LOGSWI:
                case LOGDIR:
                        return logbuf_handle_bufsize_logswi_logdir(arg_type, argc, argv);
                        //break;
                case LOGFILE:
                        return logbuf_handle_logfile(argc,argv);
                case SAVELEVEL:
                        return logbuf_handle_savelevel(argc,argv);
                default:
                        return -1;
        }
}


/****arg_type_ex acpu_header,acpu_wrap,avlog_in_total,total_filesize,total_file,prelog********/
//show
static void rtd_show_acpu_header (unsigned long value)
{
        if(value & 0x11)   //a_header   0x11:bit[0] bit[4]
                printf("acpu_header is \e[33mlong\e[0m\n");
        else
                printf("acpu_header is \e[33mshort\e[0m\n");
}

static void rtd_show_acpu_wrap (unsigned long value)
{
        if(value & 0x22)   //a_wrap     0x22:bit[1] bit[5]
                printf("acpu_wrap is \e[33mon\e[0m\n");
        else
                printf("acpu_wrap is \e[33moff\e[0m\n");
}

static void rtd_show_prelog(void)
{
        unsigned long value;
        unsigned long logswi_value = rtd_get_current_value("logswi"); 

        if(logswi_value ==0)
                return;
        
        value = logswi_value & ENABLE_PRELOG;

        if(value == ENABLE_PRELOG){
                printf("rtd_prelogbuf.bin is \e[33mon\e[0m\n");
        }else {
                printf("rtd_prelogbuf.bin is \e[33moff\e[0m\n");
        }    
}

static void rtd_show_hallog(void)
{
        unsigned long value;
        unsigned long logswi_value = rtd_get_current_value("logswi"); 

        if(logswi_value ==0)
                return;
        
        value = logswi_value & ENABLE_HALLOG;

        if(value == ENABLE_HALLOG){
                printf("hal console log is \e[33mon\e[0m\n");
        }else {
                printf("hal console log is \e[33moff\e[0m\n");
        }    
}

static void rtd_show_hal_module(void)
{
        unsigned int value = logbuf_get_hal_module("hal_module");

        printf("hal module value is \e[33m%x\e[0m\n", value);
}

static void rtd_show_hal_level(void)
{
        unsigned char level = logbuf_get_hal_level("hal_level");

        printf("hal module level is \e[33m%d\e[0m\n", level);
}

//set
static int rtd_set_acpu_header (unsigned long value, char *str)
{
        char cmd[CMD_BUFSIZE]={0};

        if(strcmp(str, "long") == 0){
                value = value | 0x11;   //bit[0] bit[4]  = 1
        }else if(strcmp(str, "short") == 0){
                value = value & 0xFFFFFFEE;     ////bit[0] bit[4]  = 0
        }else{
                printf("Import value '%s' error!!!\n", str);
                //printf("Selectable value: long  short\n");
                return -1;
        }

        memset(cmd,0,CMD_BUFSIZE);
        sprintf(cmd,"env set logfile %lx", value);
        logbuf_set_env(cmd, 0);

        /*update ddr*/
        update_ddr_param(RTD_LOGFILE);
        return 0;
}

static int rtd_set_acpu_wrap (unsigned long value, char *str)
{
        char cmd[CMD_BUFSIZE]={0};

        if(strcmp(str, "on") == 0){
                value = value | 0x22;   //bit[1] bit[5]  = 1
        }else if(strcmp(str, "off") == 0){
                value = value & 0xFFFFFFDD;     //bit[1] bit[5]  = 0
        }else{
                printf("Import value '%s' error!!!\n", str);
                //printf("Selectable value: on off\n");
                return -1;
        }

        memset(cmd,0,CMD_BUFSIZE);
        sprintf(cmd,"env set logfile %lx", value);
        logbuf_set_env(cmd, 0);

        /*update ddr*/
        update_ddr_param(RTD_LOGFILE);
        return 0;
}

static int rtd_set_prelog (char *value)
{
        unsigned long set_value = 0;        
        char cmd[CMD_BUFSIZE]={0};
        unsigned long logswi_value = rtd_get_current_value("logswi"); 

        if(logswi_value == 0)
                return -1;
        
        if(strcmp(value, "on") == 0){
                set_value = logswi_value | ENABLE_PRELOG;
        }else if(strcmp(value, "off") == 0){
                set_value = logswi_value & (~ENABLE_PRELOG);
        }else{
                printf("Import value '%s' error!!!\n", value);
                //printf("Selectable value: on  off\n");
                return -1;
        }    
        
        sprintf(cmd,"env set logswi %lx", set_value);
        logbuf_set_env(cmd, -1);

        /*update ddr*/
        update_ddr_param(RTD_LOGSWI);
        return 0;
}

static int rtd_set_hallog (char *value)
{
        unsigned long set_value = 0;        
        char cmd[CMD_BUFSIZE]={0};
        unsigned long logswi_value = rtd_get_current_value("logswi"); 

        if(logswi_value == 0)
                return -1;
                
        if(strcmp(value, "on") == 0){
                set_value = logswi_value | ENABLE_HALLOG;
        }else if(strcmp(value, "off") == 0){
                set_value = logswi_value & ((~ENABLE_HALLOG) | LOGSWI_BUFFER_MASK);
        }else{
                printf("Import value '%s' error!!!\n", value);
                //printf("Selectable value: on  off\n");
                return -1;
        }    
        
        sprintf(cmd,"env set logswi %lx", set_value);
        logbuf_set_env(cmd, -1);

        /*update ddr*/
        update_ddr_param(RTD_LOGSWI);
        return 0;
}

static int rtd_set_hal_module(char *value)
{
        char cmd[CMD_BUFSIZE]={0};

        sprintf(cmd,"env set hal_module %s", value);
        logbuf_set_env(cmd, -1);

        /*update ddr*/
        update_ddr_param(RTD_LOGFILTER);
        return 0;
}

static int rtd_set_hal_level(char *value)
{
        char cmd[CMD_BUFSIZE]={0};
        sprintf(cmd,"env set hal_level %s", value);
        logbuf_set_env(cmd, -1);

        /*update ddr*/
        update_ddr_param(RTD_LOGFILTER);
        return 0;
}

static int logbuf_print_arg_type_ex(LogbufArgTypeEX type)
{
        unsigned long logfile_value = rtd_get_current_value("logfile"); 

        if(logfile_value == 0)
                return -1;
        
        printf("current ");
        switch(type)
        {
                case ACPU_HEADER:
                        rtd_show_acpu_header(logfile_value); 
                        return 0;
                case ACPU_WRAP:       
                        rtd_show_acpu_wrap(logfile_value);
                        return 0;
                case AVLOG_IN_TOTAL:       
                        logbuf_logfile_print_av_inside();
                        return 0;
                case PRELOG:
                        rtd_show_prelog();
                        return 0;
                case HAL:
                        rtd_show_hallog();
                        return 0;
                case HAL_MODULE:
                        rtd_show_hal_module();
                        return 0;
                case HAL_LEVEL:
                        rtd_show_hal_level();
                        return 0;        
                default:
                        return -1;
        }
}

static int logbuf_set_arg_type_ex(int type, char * name, char * value)
{
        unsigned long logfile_value = rtd_get_current_value("logfile"); 
        
        if(logfile_value == 0)
                return -1;

        switch(type)
        {
                case ACPU_HEADER:
                        return rtd_set_acpu_header(logfile_value, value);
                        //break;
                case ACPU_WRAP:
                        return rtd_set_acpu_wrap(logfile_value, value);
                        //break;
                case AVLOG_IN_TOTAL:
                        return logbuf_logfile_set_av_inside(value);
                        //break;
                #if 0        
                case TOTAL_FILESIZE:
                        return rtd_set_total_filesize(value);
                        //break;
                case TOTAL_FILE:
                        return rtd_set_total_file(value);
                        //break;
                #endif        
                case PRELOG:
                        return rtd_set_prelog(value);
                case HAL:
                        return rtd_set_hallog(value);
                case HAL_MODULE:
                        rtd_set_hal_module(value);
                        rtd_set_hallog("on");
                        return 0;
                case HAL_LEVEL:
                        rtd_set_hal_level(value);
                        rtd_set_hallog("on");
                        return 0;
                default:
                        printf("Unknown type '%d'\n", type);
                        return -1;
                        //break;       
        }
}

static void logbuf_print_arg_type_ex_usage(LogbufArgTypeEX type)
{
        switch(type)
        {
                case ACPU_HEADER:
                        printf("\nacpu_header Usage:\n");
                        printf("logbuf acpu_header value   - set acpu&acpu2 log header is long or short,  selectable value: short/long\n");
                        break;
                case ACPU_WRAP:       
                        printf("\nacpu_wrap Usage:\n");
                        printf("logbuf acpu_wrap value   -  set apu1&acpu2 log is wrap or not, selectable value: on/off\n");
                        break;
                case AVLOG_IN_TOTAL:       
                        printf("\navlog_in_total Usage: - enable rtd_logfile.log contain avcpu log.\n");
                        printf("logbuf avlog_in_total value  -  rtd_logfile.log contain avcpu log or not, selectable value: on/off\n");
                        break;
                case PRELOG:
                        printf("\npre_log Usage:\n");
                        printf("logbuf pre_log value   - create rtd_prelogbuf.bin or not, selectable value: on/off\n");
                        break;
                case HAL:
                        printf("\nhal consol log Usage:\n");
                        printf("logbuf hal value       - console show hal log or not, selectable value: on/off\n");
                        break;
                case HAL_MODULE:
                        printf("\nhal module mask Usage:\n");
                        printf("logbuf hal_module value- value:hex value. set hal module mask bit, 0: mask, 1: no-mask. default value:FFFFFFFF\n");
                        break;
                case HAL_LEVEL:
                        printf("\nhal module level Usage:\n");
                        printf("logbuf hal_level value - set hal loglevel, show level<=hal_level log, value range is between 1 and 7. default value is 7.\n");
                        break;
                default:
                        break;
        }

}

static int logbuf_handle_arg_type_ex(int argc, char * const argv[])
{
        int arg_type = logbuf_compare_type_str(argv[1], logbuf_arg_type_ex, RTD_LOGBUF_ARG_EX_NUM);
        
        if(argc == 2){
                logbuf_print_arg_type_ex(arg_type);
                logbuf_print_arg_type_ex_usage(arg_type);
                return 0;
        }else if(argc == 3){
                if(logbuf_set_arg_type_ex(arg_type, argv[1], argv[2]) == 0){
                        return logbuf_print_arg_type_ex(arg_type);
                }else{
                        logbuf_print_arg_type_ex_usage(arg_type);
                        return 0;                        
                }
        }else{
                printf("Too much argument\n");
                logbuf_print_arg_type_ex_usage(arg_type);
                return -1;
        }
}




/*********cpu-type: header,console log, wrap,filesize ***************/
#define LOGBUF_ACPU1 0
#define LOGBUF_ACPU2 1
#define LOGBUF_ACPU 2
#define LOGBUF_VCPU1 3
#define LOGBUF_VCPU2 4
#define LOGBUF_VCPU 5
#define LOGBUF_AVCPU 6
static int _print_cpu_log_state(int type)
{
        unsigned long value;
        unsigned long logswi_value = rtd_get_current_value("logswi"); 

        if(logswi_value == 0)
                return -1;
        if((type<0) || (type >=RTD_LOGBUF_CPU_TYPE_NUM))
                return -1;
        
        if((strcmp(CONFIG_SYS_SOC, SYS_SOC_284x) == 0) && ((type == LOGBUF_ACPU2) || (type == LOGBUF_VCPU2))){
               //printf("note: rtd284 not support acpu2 & vcpu2");
               return -1;
        }

                        
        value = logswi_value & av_on_value[type];

        if(value == av_on_value[type]){
                printf("%s console log is \e[33mon\e[0m", logbuf_cpu_type[type]);
        }else {
                printf("%s console log is \e[33moff\e[0m", logbuf_cpu_type[type]);
        }
        return 0;
}

static int logbuf_print_cpu_log_state(int type)
{

        if(type == LOGBUF_ACPU || type ==LOGBUF_VCPU){ // 2:acpu  5:vcpu
                _print_cpu_log_state(type-2);
                printf("\n");
                _print_cpu_log_state(type-1);
        }else if(type == LOGBUF_AVCPU){
                _print_cpu_log_state(0);// a1
                printf("\n");
                _print_cpu_log_state(1);//a2
                printf("\n");
                _print_cpu_log_state(3);//v1
                printf("\n");
                _print_cpu_log_state(4);//v2
        }else{
                _print_cpu_log_state(type);
        }   
        printf("\n");
        return 0;     
}

static int _print_cpu_wrap_state(int type)
{
        unsigned long value;
        unsigned long logfile_value = rtd_get_current_value("logfile"); 

        if(logfile_value == 0)
                return -1;
        
        if((type<0) || (type >=RTD_LOGBUF_CPU_TYPE_NUM))
                return -1;
        
        if((strcmp(CONFIG_SYS_SOC, SYS_SOC_284x) == 0) && ((type == LOGBUF_ACPU2) || (type == LOGBUF_VCPU2))){
               //printf("note: rtd284 not support acpu2 & vcpu2");
               return -1;
        }
        value = logfile_value & wrap_on_value[type];
        if(value == wrap_on_value[type]){
                printf("%s wrap is \e[33mon\e[0m", logbuf_cpu_type[type]);
        }else {
                printf("%s wrap is \e[33moff\e[0m", logbuf_cpu_type[type]);
        }
        return 0;
}
static int logbuf_print_cpu_wrap_state(int type)
{
        if(type == LOGBUF_ACPU || type ==LOGBUF_VCPU){ // 2:acpu  5:vcpu
                _print_cpu_wrap_state(type-2);
                printf("\n");
                _print_cpu_wrap_state(type-1);
        }else if(type == LOGBUF_AVCPU){
                _print_cpu_wrap_state(0);// a1
                printf("\n");
                _print_cpu_wrap_state(1);//a2
                printf("\n");
                _print_cpu_wrap_state(3);//v1
                printf("\n");
                _print_cpu_wrap_state(4);//v2
        }else{
                _print_cpu_wrap_state(type);
        }       
        printf("\n");
        return 0;    
        
}
static int _print_cpu_header_state(int type)
{
        unsigned long value;
        unsigned long logfile_value = rtd_get_current_value("logfile"); 

        if(logfile_value == 0)
                return -1;

        if((type<0) || (type >=RTD_LOGBUF_CPU_TYPE_NUM))
                return -1;
        
        if((strcmp(CONFIG_SYS_SOC, SYS_SOC_284x) == 0) && ((type == LOGBUF_ACPU2) || (type == LOGBUF_VCPU2))){
               //printf("note: rtd284 not support acpu2 & vcpu2");
               return -1;
        }
        
        value = logfile_value & header_on_value[type];
        if(value == header_on_value[type]){
                printf("%s header is \e[33mlong\e[0m", logbuf_cpu_type[type]);
        }else {
                printf("%s header is \e[33mshort\e[0m", logbuf_cpu_type[type]);
        }
        return 0;
}

static int logbuf_print_cpu_header_state(int type)
{
        if(type == LOGBUF_ACPU || type ==LOGBUF_VCPU){ // 2:acpu  5:vcpu
                _print_cpu_header_state(type-2);
                printf("\n");
                _print_cpu_header_state(type-1);
        }else if(type == LOGBUF_AVCPU){
                _print_cpu_header_state(0);// a1
                printf("\n");
                _print_cpu_header_state(1);//a2
                printf("\n");
                _print_cpu_header_state(3);//v1
                printf("\n");
                _print_cpu_header_state(4);//v2
        }else{
                _print_cpu_header_state(type);
        }         
        printf("\n");
        return 0;    
}
static int _print_cpu_filesize_state(const char *str)
{
        char *cmdline = NULL;
        unsigned long filesize =0;
        
        if(str == NULL){
                printf("cpu type error\n");
                return -1;
        }

        if((strcmp(CONFIG_SYS_SOC, SYS_SOC_284x) == 0) && ((strcmp(str, "a2_filesize") == 0) || (strcmp(str, "v2_filesize") == 0))){
               //printf("note: rtd284 not support acpu2 & vcpu2");
               return -1;
        }
        
        cmdline=getenv(str);
        if(cmdline == NULL){
                printf("%s \e[33mnot set\e[0m", str);
        }else{
                filesize = simple_strtoul(cmdline, NULL, 16);
                if(filesize == RTD_FILESIZE_INIFINITE)
                {
                        printf("%s filesize is \e[33minfinite\e[0m", str);
                }else if((filesize/K) < K){
                        printf("%s filesize is \e[33m%luK\e[0m", str, (filesize/K));
                }else{
                        printf("%s filesize is \e[33m%luM\e[0m", str, (filesize/M));
                }
        }
        return 0;
}

static int logbuf_print_cpu_filesize_state(int type)
{
        if(type == LOGBUF_ACPU || type ==LOGBUF_VCPU){ // 2:acpu  5:vcpu
                _print_cpu_filesize_state(logbuf_filesize_name[type-2]);
                printf("\n");
                _print_cpu_filesize_state(logbuf_filesize_name[type-1]);
        }else if(type == LOGBUF_AVCPU){
                _print_cpu_filesize_state(logbuf_filesize_name[0]);// a1
                printf("\n");
                _print_cpu_filesize_state(logbuf_filesize_name[1]);//a2
                printf("\n");
                _print_cpu_filesize_state(logbuf_filesize_name[3]);//v1
                printf("\n");
                _print_cpu_filesize_state(logbuf_filesize_name[4]);//v2
        }else{
                _print_cpu_filesize_state(logbuf_filesize_name[type]);
        }      
        printf("\n");
        return 0;    
}

static void logbuf_print_cpu_type_usage(char *name)
{
        printf("\n%s Usage:\n", name);
        printf("logbuf %s value  -  ", name);
        if(strcmp(name, "wrap") == 0){
                printf("set log is wrap or not, selectable value: on/off\n");
        }else if(strcmp(name, "header") == 0){
                printf("set log header is long or short,  selectable value: short/long\n");                //print header
        }else if(strcmp(name, "filesize") == 0){
                printf("set logfile filesize, value: *k,*K,*m,*M ,0 (infinite filesize).\n");                //print filesize
        }
}
static void logbuf_print_cpu_type_usage_all(int type)
{
        if((type<0) || (type >=RTD_LOGBUF_CPU_TYPE_NUM))
                return;
        printf("\nUsage:\n");
        printf("logbuf %s value          -  open console log or not, selectable value: on/off\n", logbuf_cpu_type[type]);
        printf("logbuf %s wrap value     -  set log is wrap or not, selectable value: on/off\n", logbuf_cpu_type[type]);
        printf("logbuf %s header value   -  set log header is long or short,  selectable value: short/long\n", logbuf_cpu_type[type]);
        printf("logbuf %s filesize value -  set logfile filesize, selectable value: *k,*K,*m,*M ,0 (infinite filesize).\n", logbuf_cpu_type[type]);

}

static int logbuf_set_cpu_log(int type, char * value)
{
        unsigned long set_value = 0;        
        char cmd[CMD_BUFSIZE]={0};
        unsigned long logswi_value = rtd_get_current_value("logswi"); 

        if(logswi_value == 0)
                return -1;
        if((strcmp(CONFIG_SYS_SOC, SYS_SOC_284x) == 0) && ((type == LOGBUF_ACPU2) || (type == LOGBUF_VCPU2))){
               printf("Note: %s not support acpu2 & vcpu2\n", SYS_SOC_284x);
               return -1;
        }
        if(strcmp(value, "on") == 0){
                set_value = logswi_value | av_on_value[type];
        }else if(strcmp(value, "off") == 0){
                set_value = logswi_value & ((~av_on_value[type]) | LOGSWI_BUFFER_MASK);
        }else{
                printf("Import value '%s' error!!!\n", value);
                return -1;
        }    

        sprintf(cmd,"env set logswi %lx", set_value);
        logbuf_set_env(cmd, -1);

        update_ddr_param(RTD_LOGSWI);
        return 0;

}
static int logbuf_set_cpu_wrap(int type, char * value)
{        
        unsigned long set_value = 0;        
        char cmd[CMD_BUFSIZE]={0};
        unsigned long logfile_value = rtd_get_current_value("logfile"); 

        if(logfile_value == 0)
                return -1;
        
        if((strcmp(CONFIG_SYS_SOC, SYS_SOC_284x) == 0) && ((type == LOGBUF_ACPU2) || (type == LOGBUF_VCPU2))){
               printf("Note: %s not support acpu2 & vcpu2\n", SYS_SOC_284x);
               return -1;
        }
        
        if(strcmp(value, "on") == 0){
                set_value = logfile_value | wrap_on_value[type];
        }else if(strcmp(value, "off") == 0){
                set_value = logfile_value & (~wrap_on_value[type]);
        }else{
                printf("Import value '%s' error!!!\n", value);
                return -1;
        }    

        sprintf(cmd,"env set logfile %lx", set_value);
        logbuf_set_env(cmd, -1);

        update_ddr_param(RTD_LOGFILE);
        return 0;

}

static int logbuf_set_cpu_header(int type, char * value)
{
        unsigned long set_value = 0;        
        char cmd[CMD_BUFSIZE]={0};
        unsigned long logfile_value = rtd_get_current_value("logfile"); 

        if(logfile_value == 0)
                return -1;
        if((strcmp(CONFIG_SYS_SOC, SYS_SOC_284x) == 0) && ((type == LOGBUF_ACPU2) || (type == LOGBUF_VCPU2))){
               printf("Note: %s not support acpu2 & vcpu2\n",SYS_SOC_284x);
               return -1;
        }        
        if(strcmp(value, "long") == 0){
                set_value = logfile_value | header_on_value[type];
        }else if(strcmp(value, "short") == 0){
                set_value = logfile_value & (~header_on_value[type]);
        }else{
                printf("Import value '%s' error!!!\n", value);
                return -1;
        }    

        sprintf(cmd,"env set logfile %lx", set_value);
        logbuf_set_env(cmd, -1);

        update_ddr_param(RTD_LOGFILE);
        return 0;
}

static int logbuf_set_cpu_filesize(int type, char * value)
{
        unsigned long set_value = _get_filesize_setvalue(value);

        if(set_value == 0){
                return -1;
        }
        if((strcmp(CONFIG_SYS_SOC, SYS_SOC_284x) == 0) && ((type == LOGBUF_ACPU2) || (type == LOGBUF_VCPU2))){
               printf("Note: %s not support acpu2 & vcpu2\n",SYS_SOC_284x);
               return -1;
        }
        if(type == LOGBUF_ACPU || type ==LOGBUF_VCPU){ // 2:acpu  5:vcpu
                _set_filesize(logbuf_filesize_name[type-2], set_value);
                _set_filesize(logbuf_filesize_name[type-1], set_value);
        }else if(type == LOGBUF_AVCPU){
                _set_filesize(logbuf_filesize_name[0], set_value);// a1
                _set_filesize(logbuf_filesize_name[1], set_value);//a2
                _set_filesize(logbuf_filesize_name[3], set_value);//v1
                _set_filesize(logbuf_filesize_name[4], set_value);//v2
        }else{
                _set_filesize(logbuf_filesize_name[type], set_value);
        }         
        return 0;
        
}

static int logbuf_handle_cpu_type(int argc, char * const argv[])
{
        int ret=-1;
        int cpu_type = logbuf_compare_type_str(argv[1], logbuf_cpu_type, RTD_LOGBUF_CPU_TYPE_NUM);

        if(cpu_type < 0 || cpu_type >=RTD_LOGBUF_CPU_TYPE_NUM)
                return -1;
        
        if((strcmp(CONFIG_SYS_SOC, SYS_SOC_284x) == 0) && ((cpu_type == LOGBUF_ACPU2) || (cpu_type == LOGBUF_VCPU2))){
               printf("Note: %s not support acpu2 & vcpu2\n",SYS_SOC_284x);
               return -1;
        }
        
        if(argc == 2){  //print console log        logbuf cpu_type on/off
                printf("\ncurrent setting:\n");
                logbuf_print_cpu_log_state(cpu_type);
                logbuf_print_cpu_wrap_state(cpu_type);
                logbuf_print_cpu_header_state(cpu_type); 
                logbuf_print_cpu_filesize_state(cpu_type);    
                
                logbuf_print_cpu_type_usage_all(cpu_type);
                return 0;                
        }
        else if(argc == 3){     //print & set       logbuf cpu_type arg_type   
        
                if((strcmp(argv[2], "off") == 0) || (strcmp(argv[2], "on") == 0)){      //set console log
                        ret = logbuf_set_cpu_log(cpu_type, argv[2]);            
                        if(ret == 0){//set success ,print set value
                                printf("%s Current setting:\n", logbuf_cpu_type[cpu_type]);
                                logbuf_print_cpu_log_state(cpu_type);
                        }
                        return ret;
                }       
                
                if(strcmp(argv[2], "wrap") == 0){
                        printf("%s Current setting:\n", logbuf_cpu_type[cpu_type]);
                        logbuf_print_cpu_wrap_state(cpu_type);          //print wrap
                }else if(strcmp(argv[2], "header") == 0){
                        printf("%s Current setting:\n", logbuf_cpu_type[cpu_type]);
                        logbuf_print_cpu_header_state(cpu_type);                //print header
                }else if(strcmp(argv[2], "filesize") == 0){
                        printf("%s Current setting:\n", logbuf_cpu_type[cpu_type]);
                        logbuf_print_cpu_filesize_state(cpu_type);      //print filesize
                }else{
                        printf("param '%s' error\n", argv[2]);
                        return -1;
                }  
                logbuf_print_cpu_type_usage(argv[2]);
                return 0;

        }
        else if(argc == 4){     //set
        
                if(strcmp(argv[2], "wrap") == 0){
                        ret = logbuf_set_cpu_wrap(cpu_type, argv[3]);
                        if(ret == 0){   //set success ,print set value
                                printf("\ncurrent setting:\n");
                                logbuf_print_cpu_wrap_state(cpu_type);
                        }
                }else if(strcmp(argv[2], "header") == 0){
                        ret = logbuf_set_cpu_header(cpu_type, argv[3]);
                        if(ret == 0){
                                printf("\ncurrent setting:\n");
                                logbuf_print_cpu_header_state(cpu_type);
                        }
                }else if(strcmp(argv[2], "filesize") == 0){
                        ret = logbuf_set_cpu_filesize(cpu_type, argv[3]);
                        if(ret == 0){
                                printf("\ncurrent setting:\n");
                                logbuf_print_cpu_filesize_state(cpu_type);
                                logbuf_print_filesize_set_warning();
                        }
                }else {
                        printf("param '%s' error\n", argv[2]);
                        ret = -1;
                }
                if(ret != 0){
                        logbuf_print_cpu_type_usage(argv[2]);
                }
                return ret;
        }
        else{
                printf("Too much argument\n");
                return -1;
        }

}

static void logbuf_print_function_usage(void)
{
        printf("\nlogbuf on/off Usage:\n");
        printf("logbuf on/off                       - enable/disable rtd logbuf.\n");
}

// return: 1: logbuf on, other value : logbuf off
int logbuf_get_enable_status(void)
{
        char * env_value = getenv(LOGBUF_ENV_NAME);
        int ret;

        if(env_value != NULL){
                if((strcmp(env_value,"on")==0) || (strcmp(env_value,"ON")==0)){
                        ret = 1;
                }else if((strcmp(env_value,"off")==0) || (strcmp(env_value,"OFF")==0)){
                        ret = 0;
                }else{
                        printf("%s value error, unknown value \e[33m%s\e[0m\n",LOGBUF_ENV_NAME, env_value);
                        ret = -1;
                }
        }else{
                ret = 1;
        }
        return ret;
}

static int logbuf_show_enable_status(void)
{
        int status = logbuf_get_enable_status();

        if(status == 1){
                printf("%s is \e[33m%s\e[0m\n",LOGBUF_ENV_NAME, "on");
        }else if(status == 0){
                printf("%s is \e[33m%s\e[0m\n",LOGBUF_ENV_NAME, "off");
        }

        return 0;
}

static void logbuf_func_enable(int argc, char * const argv[])
{
        char cmd[CMD_BUFSIZE]={0};

        if(argc != 2){
                printf("Too much argument\n");
                logbuf_print_function_usage();  //show usage
                return;
        }

        sprintf(cmd, "env set %s %s", LOGBUF_ENV_NAME, argv[1]);
        logbuf_set_env(cmd, -1);
        logbuf_show_enable_status();  // show current set

        init_rtd_logbuf();      // reinit logbuf

}

static void logbuf_show_arg_type_all(void)
{
        int i;

        for(i = 0; i<RTD_LOGBUF_ARG_NUM; i++){
                if((i == LOGSWI) || (i==LOGFILE) || (i==SAVELEVEL))
                        continue;
                logbuf_print_arg_type(i);
        }
        logbuf_logfile_print_file();                //rtdlogfile.log on/off
        
}
static void logbuf_show_arg_type_ex_all(void)
{
        unsigned long logfile_value =  rtd_get_current_value("logfile"); 
        
        if(logfile_value == 0)
                return;        
        
        //rtd_show_acpu_header(logfile_value);
        //rtd_show_acpu_wrap(logfile_value);
        //rtd_show_acpu_avlog_in_total(logfile_value);
        //rtd_show_total_filesize();
        //rtd_show_total_file();
        //printf("rtd_logfile.log setting:");
        logbuf_logfile_print_all_filesize();    //rtdlogfile.log filesize
        logbuf_logfile_print_av_inside();      //av log in rtdlogfile.log    
        rtd_show_prelog();
        rtd_show_hallog();
        rtd_show_hal_module();
        rtd_show_hal_level();
}
static  __maybe_unused void logbuf_show_acpu1_setting(void)
{
        unsigned long logfile_value =  rtd_get_current_value("logfile"); 
        
        if(logfile_value == 0)
                return;        
        
        //rtd_show_acpu_header(logfile_value);
        //rtd_show_acpu_wrap(logfile_value);
        //rtd_show_acpu_avlog_in_total(logfile_value);
        //rtd_show_total_filesize();
        //rtd_show_total_file();
        rtd_show_prelog();
}

static void logbuf_show_cpu_type_all(void)
{        
        int i=0;

        for(i=0; i<RTD_LOGBUF_CPU_TYPE_NUM;i++){
                if(i==LOGBUF_ACPU ||i ==LOGBUF_VCPU || i==LOGBUF_AVCPU){       
                        continue;       //just print acpu1,acpu2,vcpu1,vcpu2;  do not print acpu ,vcpu and avcpu
                }              

                if((strcmp(CONFIG_SYS_SOC, SYS_SOC_284x) == 0) && ((i == LOGBUF_ACPU2) || (i == LOGBUF_VCPU2))){
                       //printf("note: rtd284 not support acpu2 & vcpu2");
                       continue;
                }
                
                printf("\n%s setting: \n", logbuf_cpu_type[i]);
                _print_cpu_log_state(i); //show a/v cpu log display console
                printf("; ");
                _print_cpu_wrap_state(i);
                printf("; ");
                _print_cpu_header_state(i);
                printf("; ");
                _print_cpu_filesize_state(logbuf_filesize_name[i]);
                //logbuf_print_cpu_filesize_state(i);
                printf(";\n");
        }

}

static int rtd_print_logbuf_all (void)
{
        printf("\nCurrent logbuf setting:\n");

        logbuf_show_enable_status();

        logbuf_show_arg_type_all();       

        logbuf_show_arg_type_ex_all();

        logbuf_show_savelevel_all("\nsavelevel setting: \n");
        
        logbuf_show_cpu_type_all();

        printf("\n");

        return 0;
}
static int do_logbuf (cmd_tbl_t *cmdtp, int flag, int argc, char * const argv[])
{
        LogbufParamType param_type;
        
        if(argc == 1){     
                rtd_print_logbuf_all(); //print current value
                return CMD_RET_USAGE;

        }else if(argc > 1){

                param_type = logbuf_get_param_type(argv[1]);
                switch(param_type)
                {
                        case ARG_TYPE:
                                logbuf_handle_arg_type(argc,argv);
                                break;
                        case ARG_TYPE_EX:
                                logbuf_handle_arg_type_ex(argc,argv);
                                break;
                        case CPU_TYPE:
                                logbuf_handle_cpu_type(argc,argv);
                                break;
                        case LOGBUF_EN:
                                logbuf_func_enable(argc,argv);
                                break;
                        default:
                                printf("Unknown command '%s'\n", argv[1]);
                                return CMD_RET_USAGE;
                }

        }

        return CMD_RET_SUCCESS;
}


U_BOOT_CMD(
        logbuf, 5, 0, do_logbuf,
        "rtd logbuffer sub system",
        "\n"
        "logbuf on/off                       - enable/disable rtd logbuf.\n"
        "logbuf logswi value                 - hex value. must 8bit and except 0x, default value is "RTD_LOGBUF_LOGSWI_DEFAULT".\n"
        "logbuf bufsize value                - set logbuf ring buffer size, selectable value 512K, 1M, 2M, 4M, default value is "RTD_LOGBUF_BUFSIZE_DEFAULT"\n"
        "logbuf logdir value                 - set logfile path, value path must contain '/' except for 'USB', default value is "RTD_LOGBUF_LOGDIR_DEFAULT"\n"
        "logbuf logfile value                - hex value. must 8bit and except 0x, default value is "RTD_LOGBUF_LOGFILE_DEFAULT"\n"
        "logbuf logfile on/off               - enable rtd_logfile.log or not, selectable value on/off.\n"
        "logbuf logfile all filesize value   - set rtd_logfile.log file size, selectable value *k , *M ,0 (infinite filesize).\n"
        "logbuf logfile all av_inside value  - rtd_logfile.log contain av log or not, selectable value on/off.\n"
        "logbuf hal on/off                   - show hal console log or not, selectable value: on/off\n"
        "logbuf hal_module value             - value:hex value. set hal module mask bit, 0: mask, 1: no-mask. default value:FFFFFFFF\n"
        "logbuf hal_level value              - set hal loglevel, show level<=hal_level log, value range is between 1 and 7. default value is 7\n"
        "logbuf cpu_type value               - show a/v console log or not, selectable value: on/off\n"
        "logbuf cpu_type wrap value          - set log is wrap or not, selectable value: on/off\n"
        "logbuf cpu_type header value        - set log header is long or short,  selectable value: short/long\n"
        "logbuf cpu_type filesize value      - set logfile filesize, selectable value: *k,*K,*m,*M ,0 (infinite filesize).\n"
        "                                      cpu_type:selectable value acpu1,acpu2,acpu,vcpu1,vcpu2,vcpu avcpu.\n"
        "logbuf savelevel buf_type value     - set savelevel, value range is between 1 and 7. default value is 7.\n"
        "                                      buf_type:selectable value acpu1,acpu2,vcpu1,vcpu2,kcpu1,s-ker,s-hal,all.\n"
        );



/***************************** transfer param **************************/
int rtk_plat_boot_prep_logbuffer_bufsize(void)
{
        int len;
        char *tmp_cmd = NULL;
        char *commandline = getenv("bootargs");
        u32 bufsize = rtd_get_bufsize_value();

        len = strlen(commandline) + strlen(logbuf_arg_type[BUFSIZE])+ 8 + 4;  //  4:for "=" & " "

        tmp_cmd =  (char *)malloc(len);
        if(tmp_cmd == NULL){
                printf("[ERR] %s: Malloc failed\n", __func__);
                return -1;
        }

        memset(tmp_cmd, 0, len);
        sprintf(tmp_cmd, "%s bufsize=%x ", commandline, bufsize);
        printf("[muye]tmp_cmd=%s\n", tmp_cmd);
        setenv("bootargs", tmp_cmd);
        free(tmp_cmd);

        return 0;
}
#if 0
int rtk_plat_boot_prep_logbuffer_arg(void)
{
        char *value_logdir;
        char *value_logfile;
        char *commandline = getenv("bootargs");
        char *tmp_cmd = NULL;
        int len;
        u32 bufsize = rtd_get_bufsize_value();

        value_logdir = getenv(logbuf_arg_type[LOGDIR]);
        value_logfile = getenv(logbuf_arg_type[LOGFILE]);

        if(value_logdir == NULL){
                value_logdir = RTD_LOGBUF_LOGDIR_DEFAULT;
        }
        if(value_logfile == NULL){
                value_logfile = RTD_LOGBUF_LOGFILE_DEFAULT;
        }

        len = strlen(commandline) + \
                strlen(logbuf_arg_type[LOGDIR]) + strlen(value_logdir) + \
                strlen(logbuf_arg_type[LOGFILE]) + strlen(value_logfile) + \
                strlen(logbuf_arg_type[BUFSIZE])+ sizeof(u32)+ 10;  //  10:for "=" & " "

        tmp_cmd =  (char *)malloc(len);
        if(tmp_cmd == NULL){
                printf("[ERR] %s: Malloc failed\n", __func__);
                return -1;
        }

        memset(tmp_cmd, 0, len);
        snprintf(tmp_cmd, len," %s logdir=%s logfile=%s bufsize=%x ", commandline, value_logdir, value_logfile, bufsize);

        setenv("bootargs", tmp_cmd);
        free(tmp_cmd);

        return 0;
}
#define FILESIZE_NUM 5
int rtk_plat_boot_prep_logbuffer_filesize(void)
{
        const char * filesize_name[FILESIZE_NUM]={
                "a1_filesize",
                "a2_filesize",
                "v1_filesize",
                "v2_filesize",
                "total_filesize"
        };
        char *commandline = getenv("bootargs");
        char *tmp_cmd = NULL;
        char * get_value;
        int len=0, offset=0,i;

        len = strlen(commandline);
        for(i=0;i<FILESIZE_NUM;i++){
                len = len + strlen(filesize_name[i]) + 8 + 3;    // 8:value len, 3: for "="&" "
        }

        tmp_cmd =  (char *)malloc(len);
        if(tmp_cmd == NULL){
                printf("[ERR] %s: Malloc failed\n", __func__);
                return -1;
        }
        memset(tmp_cmd, 0, len);

        snprintf(tmp_cmd, strlen(commandline)+1,"%s ", commandline);

        offset =strlen(commandline) +1;
        for(i=0; i<FILESIZE_NUM; i++){
                get_value = getenv(filesize_name[i]);
                if(get_value == NULL){
                        continue;
                }

                snprintf(tmp_cmd+offset, strlen(filesize_name[i])+strlen(get_value)+2, "%s=%s ", filesize_name[i], get_value);
                offset = offset +strlen(filesize_name[i])+strlen(get_value)+2;

        }

        setenv("bootargs", tmp_cmd);
        free(tmp_cmd);

        return 0;

}

int rtk_plat_boot_prep_logbuffer_logswi(void)
{
        char *tmp_cmdline = NULL;
        int len;
        int logrc;
        char *logbuffer;
        int loglength;
        char * logbuf_close = "80000000";
        char *commandline = getenv("bootargs");
        char *value = getenv(logbuf_arg_type[LOGSWI]);

        len = strlen(commandline) + strlen(logbuf_arg_type[LOGSWI]) + 8 + 3;    // 8: logswi len, 3: for =,

        tmp_cmdline = (char *)malloc(len);
        if (!tmp_cmdline)
        {
                printf("[ERR] %s: Malloc failed\n", __func__);
                return -1;
        }

        if(value == NULL){
                logrc=factory_read("tmp/factory/rtdlog",&logbuffer,&loglength);

                if(!logrc){
                        sprintf(tmp_cmdline, "%s %s=%s", commandline, logbuf_arg_type[LOGSWI], logbuf_close);
                }else{
                        sprintf(tmp_cmdline, "%s %s=%s", commandline, logbuf_arg_type[LOGSWI], RTD_LOGBUF_LOGSWI_DEFAULT);
                }
        }else{
                sprintf(tmp_cmdline, "%s %s=%s", commandline, logbuf_arg_type[LOGSWI],value);
        }

        setenv("bootargs", tmp_cmdline);
        free(tmp_cmdline);

        return 0;
}
#endif
int rtk_plat_boot_prep_logbuffer_value(void)
{
        rtk_plat_boot_prep_logbuffer_bufsize();
        return 0;
}

#else   // #ifdef Config_RTD_LOGBUF_TRUE
int init_rtd_logbuf(void)
{
        return 0;
}

int rtk_plat_boot_prep_logbuffer_value(void)
{
        return 0;
}
#endif  //#ifdef Config_RTD_LOGBUF_TRUE

