#include <common.h>
#include <platform_lib/board/gpio.h>
#include <asm/arch/io.h>

#define gpio_assert(x)		printf("assert assert assert!!!! GPIO_NUM=%d\n", GPIO_NUM)
#define iso_gpio_assert(x)	printf("assert assert assert!!!! ISOGPIO_NUM=%d\n", ISOGPIO_NUM)
#define gpi_assert(x)   printf("assert assert assert!!!! GPI_NUM=%d\n", GPI_NUM)

void SHOW_GPIO_HELP(void)
{
    printf("GPIO command description/usage\n");
    printf("Usage:\n");
    printf("gpio mis g <num> - Get MISC GPIO\n");
    printf("Example: gpio mis g 80\n");
    printf("gpio mis s <num> <value> - Set MISC GPIO\n");
    printf("Example: gpio mis s 80 1\n");
    printf("gpio iso g <num> - Get ISO GPIO\n");
    printf("Example: gpio iso g 80\n");
    printf("gpio iso s <num> <value> - Set ISO GPIO\n");
    printf("Example: gpio iso s 80 1\n");

}

typedef enum {
    ISO_GPIO      = 0,
    MISC_GPIO     = 1,
    ISO_GPI       = 2,
    ISO_GPO       = 3,
    MISC_GPI      = 4,
    MISC_GPO      = 5,
    UNKONWN_GPIO,
}GPIOTYPE;

typedef enum {
    INPUT = 0,
    OUTPUT = 1,
}GPIO_DIR;

#define ioread_reg_bit(reg, bits)   ((rtd_inl(reg) >> (bits)) & 0x1)

static inline void iowrite_reg_bit(unsigned long reg, unsigned char bit,
                   unsigned char val)
{
    val  = (val)? 1 : 0;
    if ((rtd_inl(reg) & (0x1 << bit)) != (val << bit))
    {
        rtd_outl(reg, val | (0x1 << bit));
    }
}

static int _get_reg_ofst_bit(int id,unsigned int *pOffset, unsigned char *pBit)
{
   *pOffset = ((id/31)<<2);
   *pBit = ((id%31)+1);
   return 0;

}

static int set_gpio_dir(int GPIO_NUM,GPIOTYPE type,int value)
{
   unsigned int offset;
   unsigned char bit;

   if( _get_reg_ofst_bit(GPIO_NUM,&offset,&bit) !=0)
       return -1;

   switch (type)
   {
   case MISC_GPIO:
       iowrite_reg_bit(MIS_GP0DIR + offset,bit, value);
       return 0;
   
   case ISO_GPIO:
       iowrite_reg_bit(ISO_GPDIR + offset,bit, value);
       return 0;

   default:
       break;
   }
   return -1;

}
static int get_gpio_data(int GPIO_NUM,GPIOTYPE type)
{
    unsigned int offset;
    unsigned char bit;

    if (_get_reg_ofst_bit(GPIO_NUM,&offset,&bit) !=0 )
        return -1;

    switch (type) 
    {
    case MISC_GPIO:
        return ioread_reg_bit(MIS_GP0DATI + offset,bit);

    case ISO_GPIO:
        return ioread_reg_bit(ISO_GPDATI + offset,bit);

    case MISC_GPI:
        return ioread_reg_bit(MIS_GPI_DATI + offset,bit);

    default:
        break;
    }

    return -1;

}
static int set_gpio_data(int GPIO_NUM,GPIOTYPE type,int value)
{
    unsigned int offset;
    unsigned char bit;

    if (_get_reg_ofst_bit(GPIO_NUM,&offset,&bit) !=0 )
        return -1;

    switch (type) 
    {
    case MISC_GPIO:
        iowrite_reg_bit(MIS_GP0DATO + offset,bit, value);
        return 0;
    
    case ISO_GPIO:
        iowrite_reg_bit(ISO_GPDATO + offset,bit, value);
        return 0;

    default:
        break;
    }

    return -1;

}
int getGPIO(int GPIO_NUM) {
    if(GPIO_NUM >= MIS_GPIO_MAX_CNT)
    {
        printf("getGPIO GPIO_NUM:%d out of range\n",GPIO_NUM);
        return -1;
    }
    set_gpio_dir(GPIO_NUM,MISC_GPIO,INPUT);
    return get_gpio_data(GPIO_NUM,MISC_GPIO);
}

int setGPIO(int GPIO_NUM, int value) {
    int ret = 0;
    if(GPIO_NUM >= MIS_GPIO_MAX_CNT)
    {
        printf("setGPIO GPIO_NUM:%d out of range\n",GPIO_NUM);
        return -1;
    }
    ret = set_gpio_data(GPIO_NUM,MISC_GPIO,value);
    ret |= set_gpio_dir(GPIO_NUM,MISC_GPIO,OUTPUT);
    return ret;
}

int getISOGPIO(int ISOGPIO_NUM)
{
    if(ISOGPIO_NUM >= ISO_GPIO_MAX_CNT)
    {
        printf("getISOGPIO ISOGPIO_NUM:%d out of range\n",ISOGPIO_NUM);
        return -1;
    }
    set_gpio_dir(ISOGPIO_NUM,ISO_GPIO,INPUT);
    return get_gpio_data(ISOGPIO_NUM,ISO_GPIO);
}

int setISOGPIO(int ISOGPIO_NUM, int value)
{
    if(ISOGPIO_NUM >= ISO_GPIO_MAX_CNT)
    {
        printf("setISOGPIO ISOGPIO_NUM:%d out of range\n",ISOGPIO_NUM);
        return -1;
    }
    int ret = 0;
    ret = set_gpio_data(ISOGPIO_NUM,ISO_GPIO,value);
    ret |= set_gpio_dir(ISOGPIO_NUM,ISO_GPIO,OUTPUT);
    return ret;
}

int getGPI(int GPI_NUM) {
    if(GPI_NUM >= GPI_MIS_MAX_CNT)
    {
        printf("getGPI GPI_NUM:%d out of range\n",GPI_NUM);
        return -1;
    }
    return get_gpio_data(GPI_NUM,MISC_GPI);
}

