/******************************************************************************
 * $Id: nand_base_rtk.c 2014-08-04 07:38:49Z kui_rong $
 * drivers/mtd/nand/nand_base_rtk.c
 * Overview: Realtek MTD NAND Driver
 * Copyright (c) 2008 Realtek Semiconductor Corp. All Rights Reserved.
 * Modification History:
 *    #000 2008-06-10 Ken-Yu   create file
 *    #001 2008-09-10 Ken-Yu   add BBT and BB management
 *    #002 2008-09-28 Ken-Yu   change r/w from single to multiple pages
 *    #003 2008-10-09 Ken-Yu   support single nand with multiple dies
 *    #004 2008-10-23 Ken-Yu   support multiple nands
 *
 *******************************************************************************/

 /************************************************************************
 *  Include files
 ************************************************************************/
#include <common.h>
#include <malloc.h>
#include <linux/types.h>
#include <linux/mtd/mtd.h>
#include <asm/io.h>
#include <linux/bitops.h>
#include <mtd/mtd-abi.h>
#include <linux/mtd/rtk_nand.h>
#include "nand_reg.h"
#include "rtk_nand_list.h"
#include <asm/arch/sys_flash_layout.h>
#include <watchdog.h>

/************************************************************************
 *  Definitions
 ************************************************************************/
 NF_DEBUG_SET_LEVEL(NF_DEBUG_LEVEL_INFO);
#define NOTALIGNED(mtd, x) (x & (mtd->oobblock-1)) != 0

#define check_end(mtd, addr, len)					\
do {									\
	if (mtd->size == 0) 						\
		return -ENODEV;						\
	else								\
	if ((addr + len) > mtd->size) {					\
		printk (				\
			"%s: attempt access past end of device\n",	\
			__FUNCTION__);					\
		return -EINVAL;						\
	}								\
} while(0)

#define check_page_align(mtd, addr)					\
do {									\
	if (addr & (mtd->oobblock - 1)) {				\
		printk (				\
			"%s: attempt access non-page-aligned data\n",	\
			__FUNCTION__);					\
		printk (				\
			"%s: mtd->oobblock = 0x%x\n",			\
			__FUNCTION__,mtd->oobblock);			\
		return -EINVAL;						\
	}								\
} while (0)

#define check_block_align(mtd, addr)					\
do {									\
	if (addr & (mtd->erasesize - 1)) {				\
		printk (				\
			"%s: attempt access non-block-aligned data\n",	\
			__FUNCTION__);					\
		return -EINVAL;						\
	}								\
} while (0)

#define check_len_align(mtd,len)					\
do {									\
	if (len & (512 - 1)) {          	 			\
		printk (				\
               	      "%s: attempt access non-512bytes-aligned mem\n",	\
			__FUNCTION__);					\
		return -EINVAL;						\
	}								\
} while (0)

#ifdef CONFIG_MTD_RBT_TEST
#define READ_RETRY_COUNT 1  //for test read fail case
#else
#define READ_RETRY_COUNT 3
#endif
 /************************************************************************
  *  Public variables
  ************************************************************************/
/*block-tag for bootcode-area & factory-area*/
static unsigned char SpecialBlockTag[] = {
	BLOCK_CLEAN,
	BLOCK_OTHER_DATA,
	BLOCK_BBT,
	BLOCK_IBT,
	BLOCK_EMCU_CERT,
	BLOCK_EMCU,
	BLOCK_SYS_INFO,
	BLOCK_HWSETTING,
	BLOCK_KEY_SIG,
	BLOCK_AUXCODE,
	BLOCK_BOOTCODE,
	BLOCK_KBOOT,
	BLOCK_DATA,
	BLOCK_FACTORY_SETTING,
	BLOCK_ENVPARAM_MAGICNO,
	BLOCK_QC_GPU1,
	BLOCK_QC_GPU2,
	BLOCK_QC_VE_ENCODE,
	BLOCK_QC_VE,
	BLOCK_QC_VDE,
	BLOCK_QC_VP9,
	/*add your define tag*/
	BLOCK_UNDETERMINE
};

#define SPECIAL_BLOCK_TAG_N	(sizeof(SpecialBlockTag)/sizeof(unsigned char))
/************************************************************************
 *  External variables
 ************************************************************************/

/************************************************************************
 *  Static function prototypes
 ************************************************************************/
/* NAND low-level MTD interface functions */
extern int rtk_nand_reload_bbt(struct mtd_info *mtd);
extern int nf_get_phyblock(struct mtd_info *mtd,int logblock);
extern int nf_erase_remap_block(struct mtd_info *mtd,int logblock,int bad_block);
extern int nf_remap_write_block_content(struct mtd_info *mtd,int log_block,int page,const u_char * page_content);
extern int nf_remap_read_block_content(struct mtd_info *mtd,int log_block,int page,
	unsigned int skip_bad_page);
/************************************************************************
 *  Function body
 ************************************************************************/
 static int rtk_block_status_get(struct mtd_info *mtd,  loff_t ofs)
{
	struct nand_chip *this = (struct nand_chip *)mtd->priv;
	unsigned int page = 0, block = 0;
	u16 chipnr = 0;
	unsigned char block_status_p1 = 0;
	this->active_chip=chipnr=0;
	page = ((int) ofs) >> this->page_shift;
	block = page/this->ppb;
	memset(this->g_oobbuf, 0xff, this->oob_size);
	
	if (this->isLastPage ){
		page = block*this->ppb + (this->ppb-1);
	}
	if ( this->read_oob(mtd, chipnr, page, this->oob_size, this->g_oobbuf)){
		NAND_ERR ("read_oob page=%d failed\n", page);
		return 0;
	}
	block_status_p1 = this->g_oobbuf[0];
	return block_status_p1;
}

int rtk_block_isbad(struct mtd_info *mtd, u16 chipnr, loff_t ofs)
{
	struct nand_chip *this = (struct nand_chip *)mtd->priv;
	unsigned int page = 0, block = 0,i = 0;
	unsigned char block_status_p1 = 0;
	if((block_status_p1=(unsigned char)rtk_block_status_get(mtd,ofs)) == 0){
		return 1;
	}
	page = ((int) ofs) >> this->page_shift;
	block = page/this->ppb;
	if(block < (SYS_PARAM_BOOTCODE_SIZE/mtd->erasesize)){
		for(i = 0; i< SPECIAL_BLOCK_TAG_N; i++)
		{
			if(block_status_p1 == SpecialBlockTag[i])
				break;
		}

		if(i >= SPECIAL_BLOCK_TAG_N){
			NAND_ERR("Bootcode/Secure-store/Factory area: block tag error!!! "
				"block_status_p1 = [0x%x], block_no = [0x%x]\n",
				block_status_p1,block);
			return  1;		//error block tag, bad block
		}else{
			//check special block
			if(block == 0x0){
				if (( block_status_p1 != BLOCK_CLEAN
					)&&(block_status_p1 != BLOCK_OTHER_DATA)){
					NAND_ERR("Profile area: block tag error!!! "
					  "block_status_p1 = [0x%x], block_no = [0x%x]\n",
						block_status_p1,block);
					return  1;		//error block tag, bad block
				}
			}
		}
	}else{
		if((block_status_p1 != BLOCK_CLEAN) && \
			(block_status_p1 != BLOCK_OTHER_DATA)){
			NAND_ERR("block tag error!!! block_status_p1 = [%x],"
				" block_no = [0x%x]\n",block_status_p1,block);
			return  1;		//error block tag, bad block
		}
	}
	return 0;
}

static int nand_block_isbad (struct mtd_info *mtd, loff_t ofs)
{
	int rc = 0;
	/* Check for invalid offset */
	if (ofs > mtd->size){
		NAND_ERR("ofs 0x%llx out of size 0x%llx \n",ofs,mtd->size);
		return -EINVAL;
	}
	rc = rtk_block_isbad(mtd, 0, ofs);
	return rc;
}

void mark_as_badblock(struct mtd_info *rtk_mtd,int block_id)
{
	struct nand_chip *this = (struct nand_chip *)rtk_mtd->priv;
	unsigned int page_0 = 0,page_1 = 0;
	memset(this->g_oobbuf, 0xff, this->oob_size);
	this->g_oobbuf[0] = 0x00;
	if (this->isLastPage ){
		page_0 = block_id*this->ppb+this->ppb-1;
		page_1 = block_id*this->ppb+this->ppb-2;
	}else{
		page_0 = block_id*this->ppb;
		page_1 = block_id*this->ppb+1;
	}
	this->write_oob(rtk_mtd, 0, page_0,this->oob_size, this->g_oobbuf);
	this->write_oob(rtk_mtd, 0, page_1,this->oob_size, this->g_oobbuf);
}

static int nand_block_markbad (struct mtd_info *mtd, loff_t ofs)
{
	struct nand_chip *this = (struct nand_chip *)mtd->priv;
	unsigned int page = 0, block = 0;
	int chipnr = 0, chipnr_remap = 0,phy_block = 0;
	page = ((int) ofs) >> this->page_shift;
	this->active_chip = chipnr = chipnr_remap = (int)(ofs >> this->chip_shift);
	block = page/this->ppb;
	this->active_chip=chipnr=chipnr_remap=0;
	this->select_chip(mtd, chipnr);
	
	phy_block = nf_get_phyblock(mtd,block);
	NAND_INFO("INIT(block0x%x,phyblock0x%x)....", block,phy_block); 
	mark_as_badblock(mtd, phy_block);
	return 0;
}

static int nf_scan_block_tag(struct mtd_info *mtd, unsigned int from, unsigned int depth, 
	unsigned int page_count,unsigned char block_tag)
{
	struct nand_chip *this = mtd->priv;
	int res = 0,stop_block = 0, i=0;
	int chipnr=0,page = 0,ret_value = 0;
	unsigned int spare_tags = 0;
	spare_tags = ((page_count & 0xffff) << 16) | block_tag;
	stop_block = from + depth;
	this->active_chip=chipnr=0;
	this->select_chip(mtd, chipnr);
	
	for(i = from; i < stop_block; i++ )
	{
		page = i*this->ppb;
		//reset buf
		memset(this->g_oobbuf, 0xff, this->oob_size);
		if((res = this->read_oob(mtd, chipnr, page, 
			this->oob_size, this->g_oobbuf)) < 0){
			//read fail, jump next block
			continue;
		}else{
			//check spare value
			if(spare_tags == ((unsigned int *)this->g_oobbuf)[0]){
				ret_value = i;
				break;
			}
		}
	}
	
	if(spare_tags == ((unsigned int *)this->g_oobbuf)[0]){
		return ret_value;
	}else{
		//return fail
		return 0;
	}
}

static int nand_rescue_image_read(struct mtd_info *mtd, unsigned char* dst_buf,
	unsigned int from, unsigned int length, unsigned int page_count,
	const unsigned char magic)
{
	struct nand_chip *this = mtd->priv;
	int res = 0;
	unsigned int count = 0,retry = 0x100;
	unsigned int spare_tags = 0,current_page = 0;
	int chipnr = 0,data_len = 0;

	if ((from + length) > mtd->size) {
		NAND_ERR("Attempt read beyond end of device!\n");
		return -1;
	}

	if (NOTALIGNED (mtd, from)) {
		NAND_ERR("Attempt to read not page aligned data\n");
		return -1;
	}

	this->active_chip=chipnr=0;
	this->select_chip(mtd, chipnr);
	current_page = from/this->page_size;
	
	if (length % this->page_size) {
		// align length to page boundary
		length = (length / this->page_size + 1) * this->page_size;
	}

	while (count < length) {
		memset(this->g_oobbuf, 0xff, this->oob_size);
		if((res = this->read_ecc_page (mtd, this->active_chip, current_page, 
			&dst_buf[data_len], this->g_oobbuf))){	// ECC error happened
jump_to_next_blk:
			if (--retry) {
				// jump to the same offset of next block
				current_page += this->ppb;
				continue;
			}
			else	// retry too many times, return failed
				return -1;
		}

		// check spare area to see if this correct page
		spare_tags = ((page_count & 0xffff) << 16) | magic;
		if (spare_tags != ((unsigned int *)this->g_oobbuf)[0]) {
			// maybe page counter is not correct, or magic number not match, or just an empty page (it's not the page we want anyway)
			goto jump_to_next_blk;
		}

		// read ok, go to next page
		current_page++;
		data_len += this->page_size;
		count += this->page_size;
		page_count++;	// remember to increment page counter in spare area
	}

	return 0;
}

static int nand_read_oob_ext (struct mtd_info *mtd, loff_t from, size_t len,
	size_t *retlen, u_char *oob_buf)
{
	struct nand_chip *this = mtd->priv;
	unsigned int page = 0, realpage = 0;
	int rc=0,oob_len = 0, thislen = 0,old_page =0,retry_count = 0;
	int page_offset = 0, block = 0,chipnr=0, chipnr_remap=0,phy_block = 0;

	if ((from + len) > mtd->size) {
		NAND_ERR ("Attempt read beyond end of device\n");
		*retlen = 0;
		rc = -EINVAL;
		goto end;
	}

	if (NOTALIGNED (mtd, from)) {
		NAND_ERR("Attempt to read not page aligned data\n");
		rc = -EINVAL;
		goto end;
	}

	realpage = (int)(from >> this->page_shift);
	this->active_chip = chipnr = chipnr_remap = (int)(from >> this->chip_shift);
	old_page = page = realpage & this->pagemask;
	page_offset = page & (this->ppb-1);
	block = page/this->ppb;
	this->active_chip=chipnr=chipnr_remap=0;		
	this->select_chip(mtd, chipnr);

	if ( retlen ) 
		*retlen = 0;
	thislen = this->oob_size;

	while (oob_len < len) {
		if (thislen > (len - oob_len)) 
			thislen = (len - oob_len);
		/*get phy block*/
		phy_block = nf_get_phyblock(mtd,block);
		page = phy_block*this->ppb + page_offset;
		
		for(retry_count = 0;retry_count < READ_RETRY_COUNT;retry_count++)
		{
			if((rc = this->read_oob (mtd, this->active_chip, page,
				thislen, &oob_buf[oob_len])) >=0){
				break;
			}		
		}
		if (rc < 0) {
			NAND_ERR("Un-correctable HW ECC Error at page=%d\n", page); 	
			break;
		}
				
		oob_len += thislen;
		old_page++;
		page_offset = old_page & (this->ppb-1);
		if ( oob_len<len && !(old_page & this->pagemask)) {
			old_page &= this->pagemask;
			chipnr++;
			this->active_chip = chipnr;
			this->select_chip(mtd, chipnr);
		}
		block = old_page/this->ppb;
	}

	if ( retlen ){
		if ( oob_len == len )
			*retlen = oob_len;
		else{
			NAND_ERR("error: oob_len %d != len %d\n", oob_len, len);
			rc =  -1;
		}	
	}
end:
	return rc;
}

static int nand_read_ecc (struct mtd_info *mtd, loff_t from, size_t len,
	size_t *retlen, u_char *buf, u_char *oob_buf, struct nand_oobinfo *oobsel)
{
	struct nand_chip *this = mtd->priv;
	unsigned long long page = 0, realpage = 0,page_ppb = 0;
	int rc = 0,data_len = 0, oob_len = 0,retry_count = 0;
	int chipnr = 0, chipnr_remap = 0,tmp_isCPdisable_R = mtd->isCPdisable_R;
	unsigned int old_page = 0, page_offset = 0, block = 0,phy_block = 0,skip_bad_page = 0;

	if ((from + len) > mtd->size) {
		NAND_ERR("nand_read_ecc: Attempt read beyond end of device\n");
		*retlen = 0;	
		return -EINVAL;
	}
	
	if (NOTALIGNED (mtd, from) || NOTALIGNED(mtd, len)) {
		NAND_ERR("nand_read_ecc: Attempt to read not page aligned data\n");	
		return -EINVAL;
	}

	realpage = (int)(from >> this->page_shift);
	this->active_chip = chipnr = chipnr_remap = (int)(from >> this->chip_shift);
	old_page = page = realpage & this->pagemask;
	page_offset = page & (this->ppb-1);
	page_ppb = page;
	do_div(page_ppb,this->ppb);
	block = (unsigned int)page_ppb;
	this->active_chip=chipnr=chipnr_remap=0;		

	if(oobsel){	
		if(oobsel->useecc==MTD_ECC_RTK_HW ){
			mtd->ecctype = MTD_ECC_RTK_HW;
		}else{
			mtd->ecctype = MTD_ECC_NONE;
		}
	}else{
		mtd->ecctype = mtd->oobinfo.useecc;
	}
	
	this->select_chip(mtd, chipnr);
	
	if (retlen)
		*retlen = 0;
	
	while (data_len < len) 
	{
		/*get phy block*/
		phy_block = nf_get_phyblock(mtd,block);
		if(phy_block >= this->block_num){
			NAND_ERR("out of range phy block %d,max block %d !\n",
				phy_block,this->block_num);
			return -1;
		}
		page = phy_block*this->ppb + page_offset;
		mtd->isCPdisable_R =tmp_isCPdisable_R;
		for(retry_count = 0;retry_count < READ_RETRY_COUNT;retry_count++)
		{
			if((rc = this->read_ecc_page (mtd, this->active_chip, page, 
				&buf[data_len], &oob_buf[oob_len])) == 0){
				break;
			}
		}

		if((rc < 0)||(this->ecc_err_flag == 1)||(retry_count > 1)){
			if (rc < 0) {
				NAND_ERR("page:%d: Un-correctable HW ECC!\n",(int)page);		
				skip_bad_page = 1;
			}
			NAND_ERR ("Enter remap page %d skip_bad_page %d!\n",
					(int)page,skip_bad_page);
			nf_remap_read_block_content(mtd,block, page,skip_bad_page);
			skip_bad_page = 0;
		}		
		data_len += this->page_size;
		if(oob_buf)
			oob_len += this->oob_size;
		
		old_page++;
		page_offset = old_page & (this->ppb-1);
		if ( data_len<len && !(old_page & this->pagemask)) {
			old_page &= this->pagemask;
			chipnr++;
			this->active_chip = chipnr;
			this->select_chip(mtd, chipnr);
		}
		block = old_page/this->ppb;
	}

	if ( retlen ){
		if ( data_len == len )
			*retlen = data_len;
		else{
			NAND_ERR("data_len %d != len %d\n", data_len, len);
			return -1;
		}	
	}
	return 0;
}

static int nand_read (struct mtd_info *mtd, loff_t from, size_t len, size_t * retlen, 
	u_char * buf)
{
	int rc=0;
	unsigned int rd_len = len, tmp_len = len;
	// align read length to page boundary
	if (rd_len & (mtd->oobblock - 1)){
		rd_len = (rd_len / mtd->oobblock + 1) * mtd->oobblock;
	}

	len = rd_len;
	rc = nand_read_ecc (mtd, from, len, retlen, buf, NULL, NULL);

	if(*retlen == len)
		*retlen=tmp_len;	
	return rc;
}

static int nand_read_oob (struct mtd_info *mtd, loff_t from, struct mtd_oob_ops *ops)
{
	int rc = 0;
	mtd->oobinfo.useecc = ops->mode;
	if(ops->len==ops->ooblen){// read oob
		 rc = nand_read_oob_ext(mtd, from, ops->len, &ops->retlen,ops->oobbuf);
	}else{
		 rc = nand_read_ecc(mtd, from, ops->len, &ops->retlen,ops->datbuf,
		 	ops->oobbuf, NULL);
	}
	return rc;
}

static int nand_write_ecc (struct mtd_info *mtd, loff_t to, size_t len, size_t *retlen,
			const u_char * buf, const u_char *oob_buf, struct nand_oobinfo *oobsel)
{
	struct nand_chip *this = mtd->priv;
	unsigned long long page = 0, realpage = 0,page_ppb = 0;
	int rc = 0,data_len = 0, oob_len  = 0,chipnr = 0,chipnr_remap = 0;
	unsigned int old_page = 0, page_offset = 0,block = 0,phy_block = 0;

	if ((to + len) > mtd->size) {
		NAND_ERR ("Attempt write beyond end of device\n");
		*retlen = 0;
		return -EINVAL;
	}
	
	if (NOTALIGNED (mtd, to) || NOTALIGNED(mtd, len)) {
		NAND_ERR ("Attempt to write not page aligned data\n");
		return -EINVAL;
	}
	
	realpage = (unsigned long long)(to >> this->page_shift);
	this->active_chip = chipnr = chipnr_remap = (int)(to >> this->chip_shift);
	old_page = page = realpage & this->pagemask;
	page_offset = page & (this->ppb-1);
	page_ppb = page;
	do_div(page_ppb,this->ppb);
	block = (unsigned int)page_ppb;
	this->active_chip=chipnr=chipnr_remap=0;
	this->select_chip(mtd, chipnr);

	if ( retlen )
		*retlen = 0;

	while ( data_len < len) 
	{
		/*get phy block*/
		phy_block = nf_get_phyblock(mtd,block);
		if(phy_block >= this->block_num){
			NAND_ERR("out of range phy block %d,max block %d !\n",
				phy_block,this->block_num);
			return -1;
		}
		page = phy_block*this->ppb + page_offset;

		if(oob_buf)
			rc = this->write_ecc_page (mtd, this->active_chip, page, &buf[data_len], &oob_buf[oob_len]);
		else
			rc = this->write_ecc_page (mtd, this->active_chip, page, &buf[data_len], NULL);

		if (rc < 0) {
			if((rc = nf_remap_write_block_content(mtd,block,page,buf)) < 0){
				NAND_ERR("not free block to remap! \n"); 
				rc = -1;
				goto end;
			}
		}
		data_len += this->page_size;
		oob_len  += this->oob_size;

		old_page++;
		page_offset = old_page & (this->ppb-1);
		if ( data_len<len && !(old_page & this->pagemask)) {
			old_page &= this->pagemask;
			chipnr++;
			this->active_chip = chipnr;
			this->select_chip(mtd, chipnr);
		}
		block = old_page/this->ppb;
	}
	
	if (retlen){
		if ( data_len == len){
			*retlen = data_len;
		}else{
			NAND_ERR("data_len %d != len %d\n", data_len, len); 		
			rc = -1;
		}	
	}
end: 
	return rc;
}

static int nand_write (struct mtd_info *mtd, loff_t to, size_t len, size_t *retlen,
	const u_char *buf)
{
	int rc = 0;
	rc = (nand_write_ecc (mtd, to, len, retlen, buf, NULL, NULL));
	return rc;
}

static int nand_write_oob (struct mtd_info *mtd, loff_t to, struct mtd_oob_ops *ops)//for 2.6.34 YAFFS-->mtd
{
	int rc = 0;
	rc =  nand_write_ecc (mtd, to, ops->len, &ops->retlen,ops->datbuf, ops->oobbuf, NULL);
	return rc;
}

int nand_erase_nand (struct mtd_info *mtd, struct erase_info *instr, int allowbbt)
{ 
	struct nand_chip *this = (struct nand_chip *)mtd->priv;
	u_int32_t addr = instr->addr;
	u_int32_t len = instr->len;
	int page = 0, chipnr = 0,old_page = 0, block = 0,elen = 0;
	int rc = 0,realpage = 0, chipnr_remap = 0,phy_block = 0;

	check_end (mtd, addr, len);
	check_block_align (mtd, addr);

	instr->fail_addr = 0xffffffff;
	realpage = ((int) addr) >> this->page_shift;
	this->active_chip = chipnr = chipnr_remap = ((int) addr) >> this->chip_shift;
	old_page = page = realpage & this->pagemask;
	block = page/this->ppb;
	
	this->active_chip=chipnr=chipnr_remap=0;		
	this->select_chip(mtd, chipnr);
	
	instr->state = MTD_ERASING;
	while (elen < len) {
		/*get phy block*/
		phy_block = nf_get_phyblock(mtd,block);

		page = phy_block*this->ppb;
		if((rc = this->erase_block (mtd, this->active_chip, page))!= 0){
			if((rc = nf_erase_remap_block(mtd,block,phy_block))!= 0){
				NAND_ERR("not enough free block\n");
				goto end;
			}
		}

		elen += mtd->erasesize;
		old_page += this->ppb;
		
		if ( elen<len && !(old_page & this->pagemask)) {
			old_page &= this->pagemask;
			chipnr++;
			this->active_chip = chipnr;
			this->select_chip(mtd, chipnr);
		}
		block = old_page/this->ppb;
	}
	instr->state = MTD_ERASE_DONE;
end:
	return rc;
}

static int nand_erase (struct mtd_info *mtd, struct erase_info *instr)
{
	return nand_erase_nand (mtd, instr, 0);
}

void nand_mtd_function_init(struct mtd_info *mtd)
{
	mtd->erase			= nand_erase;
	mtd->point			= NULL;
	mtd->unpoint			= NULL;
	mtd->read			= nand_read;
	mtd->write			= nand_write;
	mtd->read_ecc			= nand_read_ecc;
	mtd->write_ecc			= nand_write_ecc;
	mtd->read_oob			= nand_read_oob;
	mtd->write_oob			= nand_write_oob;
	mtd->sync			= NULL;
	mtd->lock			= NULL;
	mtd->unlock			= NULL;
	mtd->block_isbad		= nand_block_isbad;
	mtd->block_markbad		= nand_block_markbad;
	mtd->block_status_get 		= rtk_block_status_get;
	mtd->reload_bbt 		= rtk_nand_reload_bbt;
	mtd->scan_block_tag 	    	= nf_scan_block_tag;
	mtd->rescue_image_read 	    	= nand_rescue_image_read;
	mtd->set_secu_nand_prot_reg 	= NULL;
}

int find_nf_dev_index(struct mtd_info *mtd, int maxchips,
	device_type_t *nf_dev_array, unsigned int array_num,
	unsigned int *num_chips_probe)
{
	struct nand_chip *this = (struct nand_chip *)mtd->priv;
	int index = -1;
	unsigned int loop = 0,nand_id = 0;
	unsigned char id[6] = {0},maker_code = 0,device_code = 0,B5th = 0,B6th = 0;
	device_type_t *nf_dev = NULL;
	this->active_chip = 0;
	this->select_chip(mtd, 0);
	this->read_id(mtd, id);
	NAND_INFO("READ ID:0x%x 0x%x 0x%x 0x%x 0x%x 0x%x\n",
		id[0],id[1],id[2],id[3],id[4],id[5]);
	maker_code = id[0];
	device_code = id[1];
	B5th = id[4];
	B6th = id[5];
	nand_id = maker_code<<24 | device_code<<16 | id[2]<<8 | id[3];

	for (loop = 1; loop < maxchips; loop++) 
	{
		this->select_chip(mtd, loop);
		this->read_id(mtd, id);
		if (maker_code !=  id[0] ||device_code != id[1] )
			break;
	}
	*num_chips_probe = loop;
	NAND_INFO("NAND Flash Controller detects %d dies\n", *num_chips_probe);
	for (loop = 0; loop < array_num; loop++)
	{
		nf_dev = &(nf_dev_array[loop]);
		if ( (nf_dev->id==nand_id) 
		&&((nf_dev->CycleID5th==0xff)?1:(nf_dev->CycleID5th==B5th))
		&&((nf_dev->CycleID6th==0xff)?1:(nf_dev->CycleID6th==B6th))){
			//setting timing
			this->set_timing(nf_dev->T1, nf_dev->T2,nf_dev->T3,nf_dev->nf_div);
			if (nand_id != HY27UT084G2M ){
				REG_WRITE_U32( REG_MULTI_CHNL_MODE,0x20);
			}
			if (nf_dev->size == (*num_chips_probe)*nf_dev->chipsize){
				if ( *num_chips_probe == nf_dev->num_chips ){
					NAND_INFO("One %s chip has %d die(s) on board\n",
						nf_dev->name,nf_dev->num_chips);
					index = loop;
					break;
				}
			}else{
				if ( !strcmp(nf_dev->name, "HY27UF084G2M"))
					continue;
				else{
					NAND_INFO("We have %d the same %s chips on board\n",
						(*num_chips_probe)/nf_dev->num_chips, 
						nf_dev->name);
					index = loop;
					break;
				}
			}
		}
	}
	
	if(index != -1){
		NAND_INFO("nand part=%s, id=%x, device_size=%lu, chip_size=%lu, num_chips=%d, isLastPage=%d, eccBits=%d\n",
		nf_dev->name, nf_dev->id, (unsigned long)nf_dev->size, (unsigned long)nf_dev->chipsize,
		nf_dev->num_chips, nf_dev->isLastPage, nf_dev->eccSelect);
	}else{
		NAND_INFO("Warning: Lookup Table do not have this nand flash !!\n");
		NAND_INFO ("Manufacture ID=0x%02x, Chip ID=0x%02x, "
			"3thID=0x%02x, 4thID=0x%02x, 5thID=0x%02x, 6thID=0x%02x\n",
			 id[0], id[1], id[2], id[3], id[4], id[5]);
	}
	this->select_chip(mtd, 0);
	return index;
}

int nand_vaule_init(struct mtd_info *mtd,device_type_t *nf_device,
	unsigned int num_chips_probe)
{
	struct nand_chip *this = (struct nand_chip *)mtd->priv;
	mtd->name 		= "rtk_nand";
	mtd->type		= MTD_NANDFLASH;
	mtd->flags		= MTD_CAP_NANDFLASH;
	mtd->ecctype		= MTD_ECC_NONE;
	mtd->PartNum 		= nf_device->name;
	mtd->oobsize 		= nf_device->OobSize;
	mtd->writesize 		= nf_device->PageSize;
	mtd->oobblock 		= nf_device->PageSize;
	mtd->erasesize 		= nf_device->BlockSize;
	mtd->eccsize 		= 512;
	this->maker_code	= nf_device->id&0xFF;
	this->device_code	= (nf_device->id>>8)&0xFF;
	this->ecc_safe_value    = nf_device->ecc_safe_value;
	this->page_shift 	= generic_ffs(nf_device->PageSize)-1;
	this->phys_erase_shift 	= generic_ffs(nf_device->BlockSize)-1;
	this->oob_shift 	= generic_ffs(nf_device->OobSize)-1;
	this->ppb 		= nf_device->BlockSize >> this->page_shift;
	this->oob_size		= nf_device->OobSize;
	this->block_num 	= nf_device->chipsize >> this->phys_erase_shift;
	this->page_num 		= nf_device->chipsize >> this->page_shift;
	this->chipsize 		= nf_device->chipsize;
	this->device_size 	= nf_device->size;
	this->chip_shift 	= generic_ffs(this->chipsize )-1;
	this->pagemask 		= (this->chipsize >> this->page_shift) - 1;
	this->page_size 	= nf_device->PageSize;
	this->block_size 	= nf_device->BlockSize;	
	this->ecc_select 	= nf_device->eccSelect;
	this->eccmode 		= MTD_ECC_RTK_HW;
	this->bootcode_size 	= SYS_PARAM_BOOTCODE_SIZE;
	this->numchips 		= num_chips_probe;
	this->isLastPage	= nf_device->isLastPage;
	mtd->size = this->numchips * this->chipsize;
	if((this->g_databuf = kmalloc(this->page_size, GFP_KERNEL))== NULL){
		NAND_ERR("no enough memory for g_databuf\n");
		return -ENOMEM;
	}
	memset(this->g_databuf, 0xff, this->page_size);
	if(((this->g_pagebuf = kmalloc(this->page_size, GFP_KERNEL))== NULL)||
		(((unsigned int)this->g_pagebuf&0x7)!=0)){
		NAND_ERR("no enough memory for g_pagebuf,or not 8 byte aligment\n");
		return -ENOMEM;
	}
	memset(this->g_pagebuf, 0xff, this->page_size);
	if((this->g_oobbuf = kmalloc( this->oob_size, GFP_KERNEL)) == NULL){
		NAND_ERR("no enough memory for g_oobbuf\n");
		free(this->g_databuf);
		return -ENOMEM;
	}
	memset(this->g_oobbuf, 0xff, this->oob_size);
	NAND_INFO("-------------------------infomation--------------------------\n");
	NAND_INFO("this->page_shift: %lu\n", (unsigned long)this->page_shift);
	NAND_INFO("this->oob_shift: 0x%x\n", this->oob_shift);
	NAND_INFO("this->pagemask: %lu\n", (unsigned long)this->pagemask);
	NAND_INFO("this->chipsize: %lu\n",(unsigned long)this->chipsize);
	NAND_INFO("this->ecc_safe_value: %d\n", this->ecc_safe_value);
	return 0;
}

int rtk_nand_scan(struct mtd_info *mtd, int maxchips)
{
	struct nand_chip *this = (struct nand_chip *)mtd->priv;
	int index = 0;
	unsigned int num_chips_probe = 1;
	/*init mtd function*/
	nand_mtd_function_init(mtd);
	/*get nand device index and num chip*/
	if((index = find_nf_dev_index(mtd,maxchips,nand_device,
		sizeof(nand_device)/sizeof(nand_device[0]),&num_chips_probe)) == -1){
		return -1;
	}

	/*iinit nand value*/
	if(nand_vaule_init(mtd,&(nand_device[index]),num_chips_probe) < 0){
		return -1;
	}

	//init g_edo_mode_enable value
	this->get_edo_mode();

	/* =========== WE Over spec: Underclocking lists: ========== */
	switch(nand_device[index].id){
		case HY27UF081G2A:	//WE: 15 ns
		case HY27UF082G2A:
		case K9G4G08U0A:
		case K9G8G08U0M:
		case TC58NVG0S3C:
		case TC58NVG1S3C:
		case HY27UT084G2A:
			this->clock_select(mtd->PartNum, 0x03);//54 MHz
			break;
		default:
			;
	}
	WATCHDOG_KICK();
	return 0;
}
