;;----------------------------------------------------------------------------
;;
;;The confidential and proprietary information contained in this file may
;;only be used by a person authorised under and to the extent permitted
;;by a subsisting licensing agreement from ARM Limited.
;;
;;        (C) COPYRIGHT 2008-2009,2011,2013 ARM Limited.
;;             ALL RIGHTS RESERVED
;;             
;;This entire notice must be reproduced on all copies of this file
;;and copies of this file may only be made by a person if such person is
;;permitted to do so under the terms of a subsisting license agreement
;;from ARM Limited.
;;
;;    SVN Information
;;
;;    Checked In          : $Date$
;;
;;    Revision            : $Revision$
;;
;;    Release Information : r0p0-00rel0
;;
;;-----------------------------------------------------------------------------
;; 
;;-----------------------------------------------------------------------------
;; Abstract :Cache initialisation
;;-----------------------------------------------------------------------------
;; Overview
;;----------
;; 
;;
;; 
;; 
;;----------------------------------------------------------------------------

; Turn on D-cache
; We do this at the start of a test, after scatterloading.  	

    PRESERVE8
    AREA    cache, CODE
	
    EXPORT initialize_cpu
    EXPORT invalidate_dcache_raw
    EXPORT enable_caches_tlb
    EXPORT clean_dcache
    EXPORT clean_dcache_line
    EXPORT clean_invalidate_dcache_line
    EXPORT flush_tlb
    EXPORT instr_sync_barrier
    EXPORT flush_branch_pred_array
    EXPORT invalidate_icache
    EXPORT invalidate_icache_line
    EXPORT data_sync_barrier
    
    IMPORT  ttb_first_level


;------------------------------------------------------------------------------------
; CPU initialization - part1
; - Turn off caches/mmu/branch prediction
; - Invalidate Icache;branch prediction/mmu
; - Enable icache + prediction      
;------------------------------------------------------------------------------------
initialize_cpu
        ; Disable caches, MMU and branch prediction
        MRC     p15, 0, r0, c1, c0, 0           ; System control register
        BIC     r0, r0, #1 << 12                ; Instruction cache
        BIC     r0, r0, #1 << 11                ; Program flow prediction
        BIC     r0, r0, #1 << 2                 ; Data cache
        BIC     r0, r0, #1 << 0                 ; MMU
        MCR     p15, 0, r0, c1, c0, 0           ; System control register

        ; Invalidate I cache; BTAC; TLB
        MOV     r0, #0
        MCR     p15, 0, r0, c7, c5, 0           ; Invalidate Instruction Cache
        MCR     p15, 0, r0, c7, c5, 6           ; Invalidate branch prediction array
        MCR     p15, 0, r0, c8, c7, 0           ; Invalidate entire Unified Main TLB

        [ ARMv6
        MOV     r0,#0
        MCR     p15, 0, r0, c7, c5, 4           ; prefetch flush
        |        
        ISB                                     ; instr sync barrier
        ]
        
        ; Enable I cache + prediction to speed up cleaning of D-Cache
        MRC     p15, 0, r0, c1, c0, 0           ; System control register
        ORR     r0, r0, #1 << 12                ; Instruction cache enable
        ORR     r0, r0, #1 << 11                ; Program flow prediction
        MCR     p15, 0, r0, c1, c0, 0           ; System control register

        ; Return
        BX      lr

;------------------------------------------------------------------------------------
; CPU initialization - part 2 [you must have run invalidate_dcache_raw before this]
; - Setup TLB
; - Enable DCache/MMU
; - Flush ICache/BTAC
;------------------------------------------------------------------------------------
enable_caches_tlb        
        ; Set ASID (flushes MicroTLBs)
        MOV     r0, #0                          ; Using ID 0 for init process
        MCR     p15, 0, r0, c13, c0, 1          ; Write Context ID register

        ; Set up TTBR0/TTBCR
        MOV     r0, #0                          ; 16 kB TTBR0 boundary size
        MCR     p15, 0, r0, c2, c0, 2           ; write TTBCR

        LDR     r0, =ttb_first_level            ; Begining of TTB
        ORR     r0, r0, #9                      ; Outer write back, write allocate, inner cacheable
        MCR     p15, 0, r0, c2, c0, 0           ; write TTBR0

        ; Set full access for all domains
        MVN     r0, #0
        MCR     p15, 0, r0, c3, c0, 0           ; Domain access control register

        ; Enable caches, and branch prediction, MMU
        MRC     p15, 0, r0, c1, c0, 0           ; System control register
        ORR     r0, r0, #1 << 12                ; Instruction cache
        ORR     r0, r0, #1 << 11                ; Program flow prediction
        ORR     r0, r0, #1 << 2                 ; Data cache
        ORR     r0, r0, #1 << 0                 ; MMU
        MCR     p15, 0, r0, c1, c0, 0           ; System control register

        ; Since I-Cache may have been on before this point
        ; Invalidate I cache; BTAC to flush out any old mappings
        MOV     r0, #0
        MCR     p15, 0, r0, c7, c5, 0           ; Invalidate Instruction Cache
        MCR     p15, 0, r0, c7, c5, 6           ; Invalidate branch prediction array

        [ ARMv6
        MCR     p15, 0, r0, c7, c10, 4          ; drain write buffer
        MCR     p15, 0, r0, c7, c5, 4           ; prefetch flush
        |        
        DSB
        ISB
        ]

        ; Restore link register
        BX      lr

;------------------------------------------------------------------------------------
; Invalidate data cache prior to enabling.
;------------------------------------------------------------------------------------
invalidate_dcache_raw     PROC
	
  [ ARMv6
      MRC     p15,0,R0,c0,c0,1       ; Read cache type reg
      AND     R0,R0,#0x1C0000        ; Extract D cache size
      MOV     R0,R0, LSR #18         ; Move to bottom bits
      ADD     R0,R0,#7               ; Get Index loop max
      MOV     r1,#3:SHL:30           ; Set up Set = 3
      MOV     R2,#0                  ; Set up Index counter
      MOV     R3,#1
      MOV     R3,R3, LSL R0          ; Set up Index loop max
loop1_armv6
      ORR     R5,R2,r1               ; Set and Index format
      MCR     p15,0,R5,c7,c5,2       ; Invalidate D cache line
      ADD     R2,R2,#1:SHL:5         ; Increment Index
      CMP     R2,R3                  ; Done all index values?
      BNE     loop1_armv6            ; Loop until done
      MOV     r0,#0
      MCR     p15, 0, r0, c7, c10, 4  ; drain write buffer

        |
        
 ; Must iterate over the caches in order to synthesise a complete clean
 ; of data/unified cache
	mrc	p15, 1, r0, c0, c0, 1		; read clidr
	ands 	r3, r0, #0x7000000		; extract loc from clidr
	mov 	r3, r3, lsr #23			; left align loc bit field
	beq	finished			; if loc is 0, then no need to clean
	mov	r10, #0				; start clean at cache level 0 (in r10)
loop1
	add	r2, r10, r10, lsr #1		; work out 3x current cache level
	mov 	r12, r0, lsr r2			; extract cache type bits from clidr
	and	r12, r12, #7			; mask of the bits for current cache only
	cmp	r12, #2                 	; see what cache we have at this level
	blt 	skip				; skip if no cache, or just i-cache
	mcr 	p15, 2, r10, c0, c0, 0		; select current cache level in cssr
        isb
	mrc  	p15, 1, r12, c0, c0, 0		; read the new csidr
	and 	r2, r12, #7			; extract the length of the cache lines
	add	r2, r2, #4			; add 4 (line length offset)
	ldr     r6, =0x3ff
	ands    r6, r6, r12, lsr #3		; find maximum number on the way size
	clz     r5, r6				; find bit position of way size increment
	ldr	r7, =0x7fff
	ands    r7, r7, r12, lsr #13		; extract max number of the index size
loop2
	mov	r8, r6				; create working copy of max way size
loop3
	orr	r11, r10, r8, lsl r5		; factor way and cache number into r11
	orr	r11, r11, r7, lsl r2		; factor index number into r11
	mcr	p15, 0, r11, c7, c6, 2		; invalidate by set/way
	subs 	r8, r8, #1			; decrement the way
	bge 	loop3
	subs	r7, r7, #1			; decrement the index
	bge 	loop2
skip
	add	r10, r10, #2			; increment cache number
	cmp	r3, r10
	bgt 	loop1
finished
        DSB                
    ]
        
        bx      lr

        
        ENDP
	
;------------------------------------------------------------------------------------
; Functions that can be generically used for cache management                
;------------------------------------------------------------------------------------
                	
clean_dcache_line
    MCR     p15, 0, r0, c7, c10, 2
    bx      lr

clean_invalidate_dcache_line
    MCR     p15, 0, r0, c7, c10, 2 ; clean line
    MCR     p15, 0, r0, c7, c6,  2 ; invalidate line
    bx      lr

flush_tlb
    MOV     r0, #0
    MCR     p15, 0, r0, c8, c7, 0
    bx      lr

instr_sync_barrier
    [ ARMv6
    MOV     r0, #0
    MCR     p15, 0, r0, c7, c5, 4  ; prefetch flush
    |
    ISB
    ]
    bx      lr

flush_branch_pred_array
    MOV     r0, #0
    MCR     p15, 0, r0, c7, c5, 6
    bx      lr

invalidate_icache
    mov	    r0, #0
    mcr	    p15, 0, r0, c7, c5, 0		; invalidate I cache
    bx      lr

invalidate_icache_line
    MCR     p15, 0, r0, c7, c5, 2
    bx      lr

data_sync_barrier
    [ ARMv6
    MOV     r0, #0
    MCR     p15, 0, r0, c7, c10, 4  ; drain write buffer
    |
    DSB
    ]
    bx      lr
    
clean_dcache
  PUSH {r5, r6, r7, r8, r10, r11, r12, lr}
	
  [ ARMv6
      MRC     p15,0,R0,c0,c0,1       ; Read cache type reg
      AND     R0,R0,#0x1C0000        ; Extract D cache size
      MOV     R0,R0, LSR #18         ; Move to bottom bits
      ADD     R0,R0,#7               ; Get Index loop max
      MOV     r1,#3:SHL:30           ; Set up Set = 3
      MOV     R2,#0                  ; Set up Index counter
      MOV     R3,#1
      MOV     R3,R3, LSL R0          ; Set up Index loop max
loop1_armv6_d
      ORR     R5,R2,r1               ; Set and Index format
      MCR     p15,0,R5,c7,c10,2      ; Clean D cache line
      ADD     R2,R2,#1:SHL:5         ; Increment Index
      CMP     R2,R3                  ; Done all index values?
      BNE     loop1_armv6_d          ; Loop until done
      MOV     r0,#0
      MCR     p15, 0, r0, c7, c10, 4  ; drain write buffer

   |      

        mov     r10, #0
 ; Must iterate over the caches in order to synthesise a complete clean
 ; of data/unified cache
	mrc	p15, 1, r0, c0, c0, 1		; read clidr
	ands 	r3, r0, #0x7000000		; extract loc from clidr
	mov 	r3, r3, lsr #23			; left align loc bit field
	beq	finished_d			; if loc is 0, then no need to clean
	mov	r10, #0				; start clean at cache level 0 (in r10)
loop1_d
	add	r2, r10, r10, lsr #1		; work out 3x current cache level
	mov 	r12, r0, lsr r2			; extract cache type bits from clidr
	and	r12, r12, #7			; mask of the bits for current cache only
	cmp	r12, #2                 	; see what cache we have at this level
	blt 	skip_d				; skip if no cache, or just i-cache
	mcr 	p15, 2, r10, c0, c0, 0		; select current cache level in cssr
        ISB
	mrc  	p15, 1, r12, c0, c0, 0		; read the new csidr
	and 	r2, r12, #7			; extract the length of the cache lines
	add	r2, r2, #4			; add 4 (line length offset)
	ldr     r6, =0x3ff
	ands    r6, r6, r12, lsr #3		; find maximum number on the way size
	clz     r5, r6				; find bit position of way size increment
	ldr	r7, =0x7fff
	ands    r7, r7, r12, lsr #13		; extract max number of the index size
loop2_d
	mov	r8, r6				; create working copy of max way size
loop3_d
	orr	r11, r10, r8, lsl r5		; factor way and cache number into r11
	orr	r11, r11, r7, lsl r2		; factor index number into r11
	mcr	p15, 0, r11, c7, c10, 2		; clean by set/way
	subs 	r8, r8, #1			; decrement the way
	bge 	loop3_d
	subs	r7, r7, #1			; decrement the index
	bge 	loop2_d
skip_d
	add	r10, r10, #2			; increment cache number
	cmp	r3, r10
	bgt 	loop1_d
        
finished_d
        DSB
   ]
        POP {r5, r6, r7, r8, r10, r11, r12, pc}
        END
