/*----------------------------------------------------------------------------
*
* The confidential and proprietary information contained in this file may
* only be used by a person authorised under and to the extent permitted
* by a subsisting licensing agreement from ARM Limited.
*
*        (C) COPYRIGHT 2008-2009,2011-2013 ARM Limited.
*             ALL RIGHTS RESERVED
*             
* This entire notice must be reproduced on all copies of this file
* and copies of this file may only be made by a person if such person is
* permitted to do so under the terms of a subsisting license agreement
* from ARM Limited.
*
*     SVN Information
*
*     Checked In          : $Date$
*
*     Revision            : $Revision$
*
*     Release Information : r0p0-00rel0
*
*-----------------------------------------------------------------------------*/
#include "MaliFns.h"
#include "MaliDefines-t720.h"


int RunMaliTest_adv5_t720_part0 (void);

static int mali_step;

int RunMaliTest_adv5_t720 (int init) {
  if (init) { mali_step = 0; };
  switch (mali_step) {
    case 0  : return RunMaliTest_adv5_t720_part0();
    default : return 0;
  };
};
//=================================================================
int RunMaliTest_adv5_t720_part0 (void) {
  int res = 0;

  int gpuid, num_cores, l2_size, axi_width, i, as_present, js_present, core_bitmap;
  int active;

  printf("adv5 : RUNNING TEST: adv5\n");
  printf("adv5 :   Purpose: Test correct integration of power domain controllers\n");
  printf("adv5 :            Each power domain is accessed to ensure the core is actually powered up & down\n");

  // Get current configuration to allow testing all registers
  gpuid         = Mali_RdReg(0x20, 0, 0x0000);
  core_bitmap   = Mali_RdReg(0x20, 0, 0x0100);
  l2_size       = ((Mali_RdReg(0x20, 0, 0x0004) >> 16) & 0xFF);
  axi_width     = (1 <<((Mali_RdReg(0x20, 0, 0x0004) >> 24) & 0xFF));
  as_present    = Mali_RdReg(0x20, 0, 0x0018);
  js_present    = Mali_RdReg(0x20, 0, 0x001c);
  
  i = core_bitmap;
  num_cores = 0;
  while(i) {
    num_cores++;
    i >>= 1;
  }
  printf("adv5 : Selected configuration:\n");
  printf("adv5 :   Bus width                           : %d\n", axi_width);
  printf("adv5 :   Number of shader cores              : %d\n", num_cores);
  printf("adv5 :   L2 cache size                       : %d kB\n", ((1<<l2_size)/1024));


  //Mali_Identify();
  Mali_Reset();

  res |= Mali_test_reg(0, 0, 0x000, 0xffff0000, 0xffff0000, GPU_ID_VALUE, 1, "GPU_ID");           
  res |= Mali_test_reg(0, 0, 0x100, 0xffffffff, 0xffffffff, core_bitmap,1, "SHADER_PRESENT_LO");
  res |= Mali_test_reg(0, 0, 0x110, 0xffffffff, 0xffffffff, 0x00000001, 1, "TILER_PRESENT_LO"); 
  if (num_cores >= 6)
  {
    res |= Mali_test_reg(0, 0, 0x120, 0xffffffff, 0xffffffff, 0x00000011, 1, "L2_PRESENT_LO");    
  }
  else
  {
    res |= Mali_test_reg(0, 0, 0x120, 0xffffffff, 0xffffffff, 0x00000001, 1, "L2_PRESENT_LO");    
  }
  
  res |= Mali_test_reg(0, 0, 0x140, 0xffffffff, 0xffffffff, 0x00000000, 1, "SHADER_READY_LO");  
  res |= Mali_test_reg(0, 0, 0x150, 0xffffffff, 0xffffffff, 0x00000000, 1, "TILER_READY_LO");   
  res |= Mali_test_reg(0, 0, 0x160, 0xffffffff, 0xffffffff, 0x00000000, 1, "L2_READY_LO");      

  // --------------------------------------------------------
  printf("adv5 : POWERING OFF \n");  
  for(i=0; i<num_cores; i++) 
  {
    printf("adv5 : Power OFF Shader Core #%d\n", i);
    Mali_WrReg(0, 0, SHADER_PWROFF_LO, (1<<i)); 
    while( Mali_RdReg(0, 0, SHADER_PWRTRANS_LO) ) {}
  }

  printf("adv5 : Power OFF Tiler\n", i);
  Mali_WrReg(0, 0, TILER_PWROFF_LO, 1); 
  while( Mali_RdReg(0, 0, 0x210) ) {}
  res |= Mali_test_reg(0, 0, 0x150, 0xffffffff, 0xffffffff, 0, 1, "TILER_READY_LO"); 

  printf("adv5 : Power OFF L2\n", i);
  if (num_cores>=6)
  {
    Mali_WrReg(0, 0, L2_PWROFF_LO, 17); 
    while( Mali_RdReg(0, 0, 0x220) ) {}
  }
  else
  {
    Mali_WrReg(0, 0, L2_PWROFF_LO, 1); 
    while( Mali_RdReg(0, 0, 0x220) ) {}
  }
  res |= Mali_test_reg(0, 0, 0x160, 0xffffffff, 0xffffffff, 0, 1, "L2_READY_LO");

  active = GPU_activity();
  if (active)
  {
     printf("adv5 : FAILURE: GPU Active when Powered OFF!\n");
  }
  else
  {
     printf("adv5 : SUCCESS: GPU Inactive when Powered OFF\n");
  }

  // --------------------------------------------------------
  printf("adv5 : POWERING ON \n");
  for(i=0; i<num_cores; i++) 
  {   
    printf("adv5 : Power ON  Shader Core #%d\n", i);
    Mali_WrReg(0, 0, SHADER_PWRON_LO, (1<<i)); 
    while( Mali_RdReg(0, 0, SHADER_PWRTRANS_LO) ) {}
  }
  
  printf("adv5 : Power ON  Tiler\n", i);
  Mali_WrReg(0, 0, TILER_PWRON_LO, 1); 
  while( Mali_RdReg(0, 0, 0x210) ) {}
  res |= Mali_test_reg(0, 0, 0x150, 0xffffffff, 0xffffffff, 1, 1, "TILER_READY_LO"); 

  printf("adv5 : Power ON  L2\n", i);
  if (num_cores>=6)
  {
    Mali_WrReg(0, 0, L2_PWRON_LO, 17); 
    while( Mali_RdReg(0, 0, 0x220) ) {}
    res |= Mali_test_reg(0, 0, 0x160, 0xffffffff, 0xffffffff, 17, 1, "L2_READY_LO");
  }
  else
  {
    Mali_WrReg(0, 0, L2_PWRON_LO, 1); 
    while( Mali_RdReg(0, 0, 0x220) ) {}
    res |= Mali_test_reg(0, 0, 0x160, 0xffffffff, 0xffffffff, 1, 1, "L2_READY_LO");
  }

  active = GPU_activity();
  if (!active)
  {
     printf("adv5 : FAILURE: GPU Inactive when Powered ON!\n");
  }
  else
  {
     printf("adv5 : SUCCESS: GPU Active when Powered ON\n");
  }



  // --------------------------------------------------------
  printf("adv5 : POWERING OFF \n");  
  for(i=0; i<num_cores; i++) 
  {
    printf("adv5 : Power OFF Shader Core #%d\n", i);
    Mali_WrReg(0, 0, SHADER_PWROFF_LO, (1<<i)); 
    while( Mali_RdReg(0, 0, SHADER_PWRTRANS_LO) ) {}
  }

  printf("adv5 : Power OFF Tiler\n", i);
  Mali_WrReg(0, 0, TILER_PWROFF_LO, 1); 
  while( Mali_RdReg(0, 0, 0x210) ) {}
  res |= Mali_test_reg(0, 0, 0x150, 0xffffffff, 0xffffffff, 0, 1, "TILER_READY_LO"); /* TILER_READY_LO */

  printf("adv5 : Power ON  L2\n", i);
  if (num_cores>=6)
  {
    Mali_WrReg(0, 0, L2_PWRON_LO, 17); 
    while( Mali_RdReg(0, 0, 0x220) ) {}
    res |= Mali_test_reg(0, 0, 0x160, 0xffffffff, 0xffffffff, 17, 1, "L2_READY_LO");
  }
  else
  {
    Mali_WrReg(0, 0, L2_PWRON_LO, 1); 
    while( Mali_RdReg(0, 0, 0x220) ) {}
    res |= Mali_test_reg(0, 0, 0x160, 0xffffffff, 0xffffffff, 1, 1, "L2_READY_LO");
  }

  printf("adv5 : Power OFF L2\n", i);
  if (num_cores>=6)
  {
    Mali_WrReg(0, 0, L2_PWROFF_LO, 17); 
    while( Mali_RdReg(0, 0, 0x220) ) {}
  }
  else
  {
    Mali_WrReg(0, 0, L2_PWROFF_LO, 1); 
    while( Mali_RdReg(0, 0, 0x220) ) {}
  }
    res |= Mali_test_reg(0, 0, 0x160, 0xffffffff, 0xffffffff, 0, 1, "L2_READY_LO");

  active = GPU_activity();
  if (active)
  {
     printf("adv5 : FAILURE: GPU Active when Powered OFF!\n");
  }
  else
  {
     printf("adv5 : SUCCESS: GPU Inactive when Powered OFF\n");
  }


  // --------------------------------------------------------

  mali_step++;
  return res;
}


/* Mali_test_reg
   Test the register.
   access: 0=RW, 1=RO
*/
int Mali_test_reg(  int unit, 
                    int core, 
                    int regnum, 
                    int read_mask, 
                    int write_mask, 
                    int check_value, 
                    int access,
                    char * reg_name) 
{
  int value = Mali_RdReg(unit, core, regnum);
  if( value != check_value ) 
  {
    printf("adv5 : FAILURE: Wrong value. Addr: 0x%08x Value: 0x%08x Expected: 0x%08x  for  %s\n",
           (unit<<28)+(core<<16)+regnum, value, check_value, reg_name);
    return -1;
  }
  if( access == 0 ) 
  {
    Mali_WrReg(unit, core, regnum, (0xffffffff & write_mask));
    value = Mali_RdReg(unit, core, regnum) & read_mask;
    if( value != (0xffffffff & write_mask & read_mask) ) 
    {
      printf("adv5 : FAILURE: Wrong value. Addr: 0x%08x Value: 0x%08x Expected: 0x%08x  for  %s\n",
             (unit<<28)+(core<<16)+regnum, value, (0xffffffff & write_mask & read_mask), reg_name);
      return -2;
    }
    Mali_WrReg(unit, core, regnum, (0x12345678 & write_mask));
    value = Mali_RdReg(unit, core, regnum) & read_mask;
    if( value != (0x12345678 & write_mask & read_mask) ) 
    {
      printf("adv5 : FAILURE: Wrong value. Addr: 0x%08x Value: 0x%08x Expected: 0x%08x  for  %s\n",
             (unit<<28)+(core<<16)+regnum, value, (0x12345678 & write_mask & read_mask), reg_name);
      return -2;
    }
  }
  printf("adv5 : Register %08x: Success!  for  %s\n", (unit<<28)+(core<<16)+regnum, reg_name);
  return 0;
}


//=================================================================
int GPU_activity()
{
  int gpu_status = Mali_RdReg(0x20,0,GPU_IRQ_STATUS);
  /* ********************************** */
  printf ("adv5 : Starting GPU activity \n");
  gpu_status = Mali_RdReg(0x20,0,GPU_STATUS);
  printf ("adv5 : GPU_STATUS = %x\n", gpu_status);
  printf ("adv5 : Starting flush operation....\n");
  Mali_WrReg(0, 0, GPU_COMMAND, 8); 
  gpu_status = Mali_RdReg(0x20,0,GPU_STATUS);
  printf ("adv5 : GPU_STATUS = %x\n", gpu_status);
  /* ********************************** */
  return gpu_status;
  

}

