/*----------------------------------------------------------------------------
*
* The confidential and proprietary information contained in this file may
* only be used by a person authorised under and to the extent permitted
* by a subsisting licensing agreement from ARM Limited or its affiliates.
*
*        (C) COPYRIGHT 2008-2009, 2011-2017 ARM Limited or its affiliates.
*             ALL RIGHTS RESERVED
*             
* This entire notice must be reproduced on all copies of this file
* and copies of this file may only be made by a person if such person is
* permitted to do so under the terms of a subsisting license agreement
* from ARM Limited or its affiliates.
*
* Release   : r0p0-00eac0
*-----------------------------------------------------------------------------*/

/* 

BRIEF DESCRIPTION, PURPOSE AND STRATEGY
=======================================

This test requires an external Q-Channel Controller to be connected to the GPU Q-Channel signals. This test checks whether GPU comes out from quiescent state after Q-Channel Controller puts it into quiescent state.


DEBUG HINTS IN CASE OF FAILURE
===========================
a) Check the AXI bus connections as this is the bus that is used to communicate with the Job manager.
b) Check that the GPU's reset signal is not asserted.
c) Check that the GPU input clock is toggling as expected.
d) Check that the DFT signals are disabled.
e) Check that MBIST is disabled.
f) Check the GPU interrupts signals are connected correctly i.e. IRQGPU, IRQMMU, IRQJOB and IRQEVENT.

*/

#include "MaliFns.h"

int RunMaliTest_qchannel_DVALIN_part0 (void);
int RunMaliTest_qchannel_DVALIN_part1 (void);

static int mali_step;

int RunMaliTest_qchannel_DVALIN (int init) {
  if (init) { mali_step = 0; };
  switch (mali_step) {
    case 0  : return RunMaliTest_qchannel_DVALIN_part0();
    case 1  : return RunMaliTest_qchannel_DVALIN_part1();
    default : {printf ("mali_step=%d", mali_step); return 0;};
  };
};

int RunMaliTest_qchannel_DVALIN_part0 (void) {
  int res = 0;
  int gpu_status = 0;

  printf("RUNNING TEST: qchannel\n");
  printf("This test requires an external Q-Channel Controller to be connected to the GPU Q-Channel signals\n");

  gpu_status   = Mali_RdReg(0x0034);
  printf("GPU_STATUS is %08x\n", gpu_status);

  if (gpu_status & 0x1) {
        printf("GPU is Active\n");
    } else {
        printf("GPU is NOT Active\n");
    }

  // Clear all interrupt registers
  Mali_WrReg(0x0024,0xFFFFFFFF); // GPU_IRQ_CLEAR
  Mali_WrReg(0x1004,0xFFFFFFFF); // JOB_IRQ_CLEAR
  Mali_WrReg(0x2004,0xFFFFFFFF); // MMU_IRQ_CLEAR
  Mali_WrReg(0x1024,0xFFFFFFFF); // EVENT_IRQ_CLEAR 

  // Enable GPU interrupt source, disable all other qchannel sources
  Mali_WrReg(0x0028,0x00000001); // GPU_IRQ_MASK
  Mali_WrReg(0x1008,0x00000000); // JOB_IRQ_MASK
  Mali_WrReg(0x2008,0x00000000); // MMU_IRQ_MASK
  Mali_WrReg(0x1028,0x00000000); // EVENT_IRQ_MASK

  // Trigger GPU interrupt
  Mali_WrReg(0x0020,0x00000001); // GPU_IRQ_RAWSTAT

  mali_step++;
  if (res == 0) {
   return 255;
  }
  return res;
}

int RunMaliTest_qchannel_DVALIN_part1 (void) {
  int gpu_irq_status, job_irq_status, mmu_irq_status, event_irq_status;
  gpu_irq_status   = Mali_RdReg(0x002c);
  job_irq_status   = Mali_RdReg(0x100c);
  mmu_irq_status   = Mali_RdReg(0x200c);
  event_irq_status = Mali_RdReg(0x102c);
  if( (gpu_irq_status == 1) && (job_irq_status == 0) && (mmu_irq_status == 0) && (event_irq_status == 0) ) {
  printf("GPU Interrupt asserted\n");
  printf("GPU is out of Quiescent state and CLK is enabled again\n");
  Mali_WrReg(0x0024,0xFFFFFFFF); // GPU_IRQ_CLEAR
  Mali_WrReg(0x1004,0xFFFFFFFF); // JOB_IRQ_CLEAR
  Mali_WrReg(0x2004,0xFFFFFFFF); // MMU_IRQ_CLEAR
  Mali_WrReg(0x1024,0xFFFFFFFF); // EVENT_IRQ_CLEAR 
  Mali_WrReg(0x0028,0x00000000); // GPU_IRQ_MASK
  Mali_WrReg(0x1008,0x00000000); // JOB_IRQ_MASK
  Mali_WrReg(0x2008,0x00000000); // MMU_IRQ_MASK
  Mali_WrReg(0x1028,0x00000000); // EVENT_IRQ_MASK
  mali_step++;
  return 0;
  }
  printf("GPU is stuck in Quiescent state and CLK is NOT running\n");
  Mali_WrReg(0x0024,0xFFFFFFFF); // GPU_IRQ_CLEAR
  Mali_WrReg(0x1004,0xFFFFFFFF); // JOB_IRQ_CLEAR
  Mali_WrReg(0x2004,0xFFFFFFFF); // MMU_IRQ_CLEAR
  Mali_WrReg(0x1024,0xFFFFFFFF); // EVENT_IRQ_CLEAR 
  Mali_WrReg(0x0028,0x00000000); // GPU_IRQ_MASK
  Mali_WrReg(0x1008,0x00000000); // JOB_IRQ_MASK
  Mali_WrReg(0x2008,0x00000000); // MMU_IRQ_MASK
  Mali_WrReg(0x1028,0x00000000); // EVENT_IRQ_MASK
  return 1;
}
