#include "util.h"
#include "../ve_plat.h"
#include <ve_mbist_reg.h>
#include <topbist_reg.h>
#include <sys_reg_reg.h>
#include <vde_reg.h>

//reg define
#define  SYS_REG_SYS_SRST0_reg                                                   0xB8000100
#define  SYS_REG_SYS_SRST1_reg                                                   0xB8000104
#define  SYS_REG_SYS_CLKEN0_reg                                                  0xB8000110
#define  SYS_REG_SYS_CLKEN1_reg                                                  0xB8000114

int do_ve_bisr(void)
{
	unsigned int iMaxCount = 1000;
	unsigned bit_flags;
	int ret = 0;

	prints("VE: BISR\n");

	// ve clk sel
	rtd_setbits(SYS_REG_SYS_CLKSEL_reg, SYS_REG_SYS_CLKSEL_ve_sel_mask);

	// ve_cah, ve_ngh, ve_ngh_2 bisr rstn
	bit_flags = TOPBIST_ve_bisr_rstn_ve_cah_bisr_rstn_mask |
				TOPBIST_ve_bisr_rstn_ngh_bisr_rstn_mask |
				TOPBIST_ve_bisr_rstn_ngh_2_bisr_rstn_mask;

	// clear the previous setting
	rtd_clearbits(TOPBIST_ve_bisr_remap_reg, bit_flags);
	rtd_clearbits(TOPBIST_ve_bisr_rstn_reg, bit_flags);

	// start bisr
	rtd_setbits(TOPBIST_ve_bisr_rstn_reg, bit_flags);

	// wait for done
	while((rtd_inl(TOPBIST_ve_bisr_done_reg) & bit_flags) != bit_flags) {
		iMaxCount--;
		udelay(100);
		if( iMaxCount == 0 ) {
			prints("VE: timeout\n");
			ret = -1;
			return ret;
		}
	}

	// hold remap
	rtd_setbits(TOPBIST_ve_bisr_remap_reg, bit_flags);

	// check fail
	bit_flags = VE_MBIST_cah_bisr_fail_group_ve_ngh_2_bisr_fail_group_mask |
				VE_MBIST_cah_bisr_fail_group_ve_ngh_bisr_fail_group_mask |
				VE_MBIST_cah_bisr_fail_group_ve_cah_bisr_fail_group_mask;

	if (rtd_inl(VE_MBIST_cah_bisr_fail_group_reg) & bit_flags)
	{
//		enable_console();
		prints("VE: fail\n");
		ret = -1;
		while(1);
	}
	else
	{
		prints("VE: ok\n");
	}

	// check repair
	if((rtd_inl(VE_MBIST_ve_ngh_bisr_fail_reg) & VE_MBIST_ve_ngh_bisr_fail_ve_ngh_2_bisr_repaired_mask) != 0) {
		prints("VE_ngh_2: repair\n");
	}
	if((rtd_inl(VE_MBIST_ve_ngh_bisr_fail_reg) & VE_MBIST_ve_ngh_bisr_fail_ve_ngh_bisr_repaired_mask) != 0) {
		prints("VE_ngh: repair\n");
	}
	if((rtd_inl(VE_MBIST_ve_cah_bsir_fail_reg) & VE_MBIST_ve_cah_bsir_fail_ve_cah_bisr_repaired_mask) != 0) {
		prints("VE_cah: repair\n");
	}

	return ret;
}

/*
 * enable video related clock
 */
void enable_video_clock(void)
{
	// rst = 0
	//VE_BLK_WRAP, VE_LGCY_P2, VE_LGCY, VDE, VE_P2, VE, VCPU
	// VDEC, ME, SE
	rtd_outl(SYS_REG_SYS_SRST0_reg, _BIT9 | _BIT8 | _BIT6 | _BIT5 | _BIT4 | _BIT3);
	rtd_outl(SYS_REG_SYS_SRST1_reg, _BIT22 | _BIT12 | _BIT5 | _BIT4);

	// clk = 1
	// VE_LGCY_P2, VE_LGCY, VE_P2, VE, VCPU
	// VDEC, ME, SE
	rtd_outl(SYS_REG_SYS_CLKEN0_reg, _BIT8   | _BIT7  | _BIT6 | _BIT5 | _BIT4 | _BIT3 | _BIT0);
	rtd_outl(SYS_REG_SYS_CLKEN1_reg, _BIT22 | _BIT12 | _BIT5 | _BIT0);

	// clk = 0
	rtd_outl(SYS_REG_SYS_CLKEN0_reg, _BIT8  | _BIT7 | _BIT6 | _BIT5 | _BIT4 | _BIT3);
	rtd_outl(SYS_REG_SYS_CLKEN1_reg, _BIT22 | _BIT12 | _BIT5);

	// rst = 0
	rtd_outl(SYS_REG_SYS_SRST0_reg, _BIT9 | _BIT8 | _BIT6 | _BIT5 | _BIT4 | _BIT3);
	rtd_outl(SYS_REG_SYS_SRST1_reg, _BIT22 | _BIT12 | _BIT5 | _BIT4);
	udelay(10);

	// rst = 1
	rtd_outl(SYS_REG_SYS_SRST0_reg, _BIT9 | _BIT8 | _BIT6 | _BIT5 | _BIT4 | _BIT3 | _BIT0);
	rtd_outl(SYS_REG_SYS_SRST1_reg, _BIT22 | _BIT12 | _BIT5 | _BIT4 |  _BIT0);
	udelay(10);

	// clk = 1
	rtd_outl(SYS_REG_SYS_CLKEN0_reg, _BIT8 | _BIT7 | _BIT6 | _BIT5 | _BIT4 | _BIT0); // set _BIT3(vcpu) after bisr done
	rtd_outl(SYS_REG_SYS_CLKEN1_reg, _BIT22 | _BIT12 | _BIT5 | _BIT0);

	// do ve bisr before enable vcpu clock
	do_ve_bisr();

	rtd_outl(SYS_REG_SYS_CLKEN0_reg, _BIT3 | _BIT0); // set vcpu

#if 0 //cklai@move ssb7 priority setting from hw_setting to uboot due to clken here
	rtd_outl(SBX_ssb7_misc_ctrl_reg, 0x00000004);
#endif

}

#if 0
void enable_video_clock(int id)
{
	if(id == 1)
	{
		// rst = 0
		//VE_BLK_WRAP, VE_LGCY_P2, VE_LGCY, VDE, VE_P2, VE, VCPU
		rtd_outl(SYS_REG_SYS_SRST0_reg, _BIT9 | _BIT8 | _BIT6 | _BIT5 | _BIT4 | _BIT3);
		rtd_outl(SYS_REG_SYS_SRST1_reg, _BIT4);

		// clk = 1
		// VE_LGCY_P2, VE_LGCY, VE_P2, VE, VCPU
		rtd_outl(SYS_REG_SYS_CLKEN0_reg, _BIT8   | _BIT7  | _BIT6 | _BIT5 | _BIT4 | _BIT3 | _BIT0);

		// clk = 0
		rtd_outl(SYS_REG_SYS_CLKEN0_reg, _BIT8  | _BIT7 | _BIT6 | _BIT5 | _BIT4 | _BIT3);

		// rst = 0
		rtd_outl(SYS_REG_SYS_SRST0_reg, _BIT9 | _BIT8 | _BIT6 | _BIT5 | _BIT4 | _BIT3);
		rtd_outl(SYS_REG_SYS_SRST1_reg, _BIT4);
		udelay(10);

		// rst = 1
		rtd_outl(SYS_REG_SYS_SRST0_reg, _BIT9 | _BIT8 | _BIT6 | _BIT5 | _BIT4 | _BIT3 | _BIT0);
		rtd_outl(SYS_REG_SYS_SRST1_reg, _BIT4 |  _BIT0);
		udelay(10);

		// clk = 1
		rtd_outl(SYS_REG_SYS_CLKEN0_reg, _BIT8 | _BIT7 | _BIT6 | _BIT5 | _BIT4 | _BIT3 | _BIT0);
	}
	if(id == 2)
	{
#if 0		
		// clk = 0
		rtd_clearbits(CRT_SYS_CLKEN0_reg, _BIT16 | _BIT15 | _BIT14);
		udelay(10);

		// rst = 0
		// VDE2, VE2_P2, VE2, VCPU2
		rtd_clearbits(CRT_SYS_SRST0_reg, _BIT17 | _BIT16 | _BIT15 | _BIT14);
		udelay(10);

		// clk = 1
		// VE2_P2, VE2, VCPU2
		rtd_setbits(CRT_SYS_CLKEN0_reg, _BIT16 | _BIT15 | _BIT14);
        	udelay(10);

		// clk = 0
		rtd_clearbits(CRT_SYS_CLKEN0_reg, _BIT16 | _BIT15 | _BIT14);
		udelay(10);

		// rst = 1
		rtd_setbits(CRT_SYS_SRST0_reg, _BIT17 | _BIT16 | _BIT15 | _BIT14);
		udelay(10);

		// clk = 1
		rtd_setbits(CRT_SYS_CLKEN0_reg, _BIT16 | _BIT15 | _BIT14);
		udelay(10);
#endif	
	        rtd_outl(CRT_SYS_SRST0_reg, _BIT17 | _BIT16 | _BIT15 | _BIT14);

		// clk = 1
		// VE2_P2, VE2, VCPU2
		rtd_outl(CRT_SYS_CLKEN0_reg, _BIT16 | _BIT15 | _BIT14 | _BIT0);

		// clk = 0
		rtd_outl(CRT_SYS_CLKEN0_reg, _BIT16 | _BIT15 | _BIT14);

		// rst = 0
		rtd_outl(CRT_SYS_SRST0_reg, _BIT17 | _BIT16 | _BIT15 | _BIT14);
		udelay(10);

		// rst = 1
		rtd_outl(CRT_SYS_SRST0_reg, _BIT17 | _BIT16 | _BIT15 | _BIT14 | _BIT0);
		udelay(10);

		// clk = 1
		rtd_outl(CRT_SYS_CLKEN0_reg, _BIT16 | _BIT15 | _BIT14 | _BIT0);
	}
}
#endif

__attribute__ ((unused)) static void hold_hw_semaphore(void)
{
	//printf("## Hold HW semaphore\n");
	while(!(rtd_inl(REG_HW_SEMAPHORE)))
	{
		udelay(100);
	}
}

void enable_ve_clock(int id)
{
	flush_dcache_all();
#ifdef ENABLE_VCPU_BISR		
	hold_hw_semaphore();
#endif
	enable_video_clock();
	mdelay(1);
	/* Release HW Semaphore */
	rtd_outl(REG_HW_SEMAPHORE, 0x0);
}

void disable_ve_clock(int id)
{
	if(id == 1)
	{
                // rst = 0
                // VDE, VE_P2, VE, VCPU
                rtd_outl(CRT_SYS_SRST0_reg,  _BIT6 | _BIT5 | _BIT4 | _BIT3);

                // clk = 0
                // VE_P2, VE, VCPU
                rtd_outl(CRT_SYS_CLKEN0_reg, _BIT5 | _BIT4 | _BIT3);
	}
	else
	{
		// clk = 0
		// VE2_P2, VE2, VCPU2
		rtd_outl(CRT_SYS_CLKEN0_reg, _BIT16 | _BIT15 | _BIT14);

		// rst = 0
		// VDE2, VE2_P2, VE2, VCPU2
		rtd_outl(CRT_SYS_SRST0_reg, _BIT17 | _BIT16 | _BIT15 | _BIT14);
	}
}

