#include <qc_verify.h>
#include <rtd_types_qc.h>
#include <uart_qc.h>

#define KEYCODE_BS		0x08
#define KEYCODE_TAB		0x09
#define KEYCODE_ESC		0x1B
#define KEYCODE_SP		0x20
#define KEYCODE_CR		0x0D
#define KEYCODE_LF		0x0A
#define MAX_ARGV		20
#define TAB				8

#define U1RBR_THR_DLL 0xb801B100
#define U1IIR_FCR 0xb801B108
#define U1LCR     0xb801B10C

#if (UART_PORT == 0) // use uart0
#define GetChar()	UartDataRead()
#define PutChar(x)	UartDataWrite(x)
#elif (UART_PORT == 1) // use uart1
#define GetChar()	Uart1DataRead()
#define PutChar(x)	Uart1DataWrite(x)
#endif

static char*			ArgvArray[MAX_ARGV];

/*-----------------------------------------------
	Lower case string to upper case string
-----------------------------------------------*/

char* StrUpr( char* string )
{
	char*		p ;
	const int	det = 'a' - 'A';

	p = string ;
	while( *p ){
		if( *p >= 'a'  &&  *p <= 'z' ){
			*p -= det ;
		}
		p++ ;
	}
	return string ;
}

/*
---------------------------------------------------------------------------
; void GetLine(char * buffer, const unsigned int size, int EchoFlag)
:   Description:
;	    Input line from stdio
;	    Function not return until a line input complete.
;	    End input line only by <ENTER> key (0x0d)
;   Input:
;	    buffer: pointer to buffer for getline
;	    size  : maximum buffer size
;			EchoFlag: 1: Echo character when get character
;					: 0: No Echo character
;   Output:
;	    buffer: pointer to input data, end by zero character
;	    none
;   Destroy:
---------------------------------------------------------------------------
*/
void GetLine( char * buffer, const unsigned int size, int EchoFlag )
{
	char		ch = 0 ;
	int		c;
	char*		p = buffer ;
	unsigned int	n = 0L ;
	int		i ;

	while( n < size )
	{
		c = GetChar() ;
		
		if( c == -1 )
		{
		    continue;
		}
		ch = c;
		if( ch == KEYCODE_LF )
		{
		    #ifdef USE_LF
		    *--p = 0 ;
		    n-- ;
		    #endif // USE_LF
		    break ;
		}
		else if( ch == KEYCODE_CR )
		{
		    *p = 0 ;
		    n-- ;
		    break ;
		}
		else if( ch == KEYCODE_BS )
		{
		    if( p != buffer )
		    {
			    p-- ;
			    n-- ;
							if(EchoFlag)
							{
								PutChar(KEYCODE_BS);
								PutChar(' ');
								PutChar(KEYCODE_BS);
							}
		    }
		}
		else if( ch == KEYCODE_TAB )
		{				
		    for( i=0 ; i < TAB ; i++ )
		    {
			    *p++ = ' ' ;
			    n++ ;
							if(EchoFlag) PutChar(' ');
		    }
		}
		else
		{
		    *p++ = ch ;
		    n++ ;
					if(EchoFlag) PutChar(ch);
		}
	}
}

/*
---------------------------------------------------------------------------
; int GetArgc(const char* string)
:   Description:
;			Get argument number for a input string
;			Input string separate by space key
;   Input:
;	    string : pointer to buffer
;   Output:
;			argument number
;   Destroy:
---------------------------------------------------------------------------
*/
int GetArgc( const char* string )
{
	int			argc;
	char*		p ;

	argc = 0 ;
	p = (char* )string ;
	while( *p )
	{
		if( *p != ' '  &&  *p )
		{
			argc++ ;
			while( *p != ' '  &&  *p ) p++ ;
			continue ;
		}
		p++ ;
	}
	if (argc >= MAX_ARGV) argc = MAX_ARGV - 1;
	return argc ;
}

/*
---------------------------------------------------------------------------
; int GetArgv(const char* string)
:   Description:
;			Get argument for a input string
;			Input string separate by space key
;   Input:
;	    string : pointer to buffer
;   Output:
;			argument string array, argv[0], argv[1],...
;			argument string end by zero character
;   Destroy:
---------------------------------------------------------------------------
*/
char** GetArgv(const char* string)
{
	char*			p ;
	int				n;

	n = 0 ;
	memset( ArgvArray, 0, MAX_ARGV*sizeof(char *) );
	p = (char* )string ;
	while( *p )
	{
		ArgvArray[n] = p ;
		while( *p != ' '  &&  *p ) p++ ;
		*p++ = '\0';
		while( *p == ' '  &&  *p ) p++ ;
		n++ ;
		if (n == MAX_ARGV) break;
	}
	return (char** )&ArgvArray ;
}


void Set_GPIO_for_TVE_VDC(UINT8 Sts)
{
	rtd_maskl(0xb806022c,0x0fffffff,0xf0000000);//Pinshare to pad_st_GPIO_55[31:28]
	rtd_maskl(0xb8061104,0xfdfffffe,0x02000001);//pad_st_GPIO_55[25] IO set to 1  
	
	if(Sts == VDC_EQC)
	{
		rtd_maskl(0xb806110c,0xfdffffff,0x02000000);//pad_GPIO_55[25] set low for VDC
		printf("Set pad_GPIO_55 to low\n");
	}
	else
	{
		rtd_maskl(0xb806110c,0xfdfffffe,0x02000001);//pad_GPIO_55[25] set high for TVE
		printf("Set pad_GPIO_55 to high\n");
	}
}


void Set_ST_GPIO3(UINT8 Sts)
{
#ifdef CONFIG_RTD299O_QC
	//for VDC and TVE QC test control
	rtd_maskl(0xb8060200,0xffffff0f,0x000000f0);//Pinshare to ST_GPIO3[7:4]
	rtd_maskl(0xb8061100,0xfffffff7,0x00000008);//ST_GPIO3[3] IO set to O	
	if(Sts == VDC_EQC)
	{
		rtd_maskl(0xb8061108,0xfffffff7,0x00000000);//ST_GPO3[3] set low for VDC
		printf("Set ST_GPIO3 to low\n");
	}
	else
	{
		rtd_maskl(0xb8061108,0xfffffff7,0x00000008);//ST_GPO3[3] set high for TVE
		printf("Set ST_GPIO3 to high\n");
	}
#endif

#if defined(CONFIG_RTD289X_QC)||defined(CONFIG_RTD289XP_QC)



               if(Sts == VDC_EQC)
	{
	                rtd_maskl(0xb8060204,0x0fffffff,0xf0000000);
                              rtd_maskl  (0xb8061100,~_BIT4,_BIT4);
                              rtd_maskl  (0xb8061108,~_BIT4,0);
                                rtd_outl(0xb8061108,0x0);

		printf("Set ST_GPIO3 to low\n");
	}
	else
	{
	                rtd_maskl(0xb8060204,0x0fffffff,0xf0000000);
                              rtd_maskl  (0xb8061100,~_BIT4,_BIT4);
                              rtd_maskl  (0xb8061108,~_BIT4,_BIT4);
		printf("Set ST_GPIO3 to high\n");
	}


#endif

}

int UART1_to_MAC2(UINT8 item, UINT8 branch_num)
{
	UINT32 ch = 0;
	UINT8  flag = 0;
	UINT8  flag_end = 0;
	UINT8  i=0;

    //input mux & pinshare
	rtd_maskl(0xb80008a8, 0xffff1fff, 0x00002000);  //[15:13] = 0x01   src1:input mux
    rtd_maskl(0xb8060210, 0xff0f0fff, 0x00707000); //RX:[23:20]=0x07, TX:[15:12]=0x07
    //uart settings
    rtd_outl(0xB801B10C,0x83);            //Line CTRL: DLAB:1; Word length:8    
    rtd_outl(0xB801B100,0x0f);            //U1RBR_THR_DLL: (DLAB=1) --> Timing Setting(DLL), clock =27MHz
    rtd_outl(0xB801B10C,0x03);            //Line CTRL: DLAB:0; DLL --> RX/TX FIFO
    rtd_outl(0xB801B108,0xc6);            //FIFO CTRL: FIFO Reset:FIFO En; receiver line status
    rtd_outl(0xB801B108,0xc1);            //FIFO CTRL: FIFO Enable:FIFO En; no interrupt pending;
    rtd_outl(0xB801B10C,0x03);            //Line CTRL: DLAB:0; DLL --> RX/TX FIFO	
	
	//TEST uart1
	rtd_outl(U1RBR_THR_DLL,KEYCODE_LF); //enter key
	rtd_outl(U1RBR_THR_DLL,KEYCODE_LF); //enter key	
	rtd_outl(U1RBR_THR_DLL,0x71);	//q
	rtd_outl(U1RBR_THR_DLL,0x63);	//c
	rtd_outl(U1RBR_THR_DLL,KEYCODE_LF); //enter key	

	for (i=0;i<5; i++)
		udelay(50000);
	rtd_outl(U1RBR_THR_DLL, (0x30+item)); //item number
	rtd_outl(U1RBR_THR_DLL,KEYCODE_LF); //enter key	

	for (i=0;i<6; i++)
		udelay(50000);
	rtd_outl(U1RBR_THR_DLL, (0x30+branch_num)); //branch number	
	rtd_outl(U1RBR_THR_DLL,KEYCODE_LF); //enter key	
	
	udelay(1000);
	while(TRUE)
	{
		ch = GetChar();
		PutChar(ch);

		//fail check	
		if (ch == 0x52) //R
			flag = 1;
		else if ((flag == 1) && (ch == 0x58)) //X
			flag = 2;
		else if ((flag == 2) && (ch == 0x5F)) //_
			flag = 3;
		else if ((flag == 3) && (ch == 0x46)) //F
			flag = 4;
		else if ((flag == 4) && (ch == 0x41)) //A
			flag = 5;
		else if ((flag == 5) && (ch == 0x49)) //I
			flag = 6;
		else if ((flag == 6) && (ch == 0x4C)) //L
			flag = 7;
		else if (flag != 7)
			flag = 0;					

		//test finish
		if (ch == 0x3C) //<
			flag_end = 1;
		else if ((flag_end == 1) && (ch == 0x65)) //e
			flag_end = 2;
		else if ((flag_end == 2) && (ch == 0x6E)) //n
			flag_end = 3;
		else if ((flag_end == 3) && (ch == 0x64)) //d
			flag_end = 4;		
		else if ((flag_end == 4) && (ch == 0x3E)) //>
			flag_end = 5;		
		else if (flag_end != 5)
			flag_end = 0;

		if (flag_end == 5)
		{				
			rtd_outl(U1RBR_THR_DLL,0x33); //3
			rtd_outl(U1RBR_THR_DLL,KEYCODE_LF); //enter key
			udelay(1000);	
			
			if (flag == 7)  
				return FAILED;		 
			else 
				return SUCCESS; 
			break;
		}			
	}
}
