#include <linux/string.h>
#include <common.h>
#include <command.h>
#include <malloc.h>
#include <linux/compiler.h>
#include <asm/io.h>
#include "alc1311.h"
#include <drivers/i2c/rtk_i2c_lib.h>
#include <drivers/i2c/rtk_g2c_lib.h>




//*************************************************************
// PUBLIC FUNCTIONS
//*************************************************************
#define END_FLAG 	(0xff)
#define AMP_DEBUG 	(0)

typedef struct
{
    unsigned char bAddr;
    unsigned char bAddr1;
    unsigned char bLength;
    unsigned char bArray[2];
    unsigned int bdelay;
}ALC1311_REG;//markchen add for 1311

static ALC1311_REG ALC1311_InitTbl[] =
{
	/////////////////initial

	{ 0x0,0x00,2,{0xFF,0xFF}, 0 },
	//{ 0x0004, 0x1000, 0 },

	// enable RC clock
	{ 0x00,0x04, 2,{0xC0,0x00}, 0 },

	{ 0x00,0x06, 2,{0x35,0x70}, 0 },
	{ 0x00,0x08, 2,{0x30,0x00}, 0 },
	{ 0x00,0x0A, 2,{0x10,0x5E}, 0 },
	{ 0x00,0x0C, 2,{0x00,0x00}, 0 },
	{ 0x03,0x22, 2,{0x40,0x0F}, 0 },
	{ 0x03,0x26, 2,{0x00,0x0F}, 0 },
	{ 0x03,0x50, 2,{0x01,0x88}, 0 },
	{ 0x03,0x24, 2,{0xC0,0xA0}, 0 },
	{ 0x00,0x6A, 2,{0x05,0x10}, 0 },
	{ 0x00,0x6C, 2,{0x25,0x00}, 0 },
	{ 0x01,0x03, 2,{0x03,0x3E}, 0 },	//Master Volume L = 0dB
	{ 0x01,0x04, 2,{0x03,0x3E}, 0 },	//Master Volume L = 0dB
	//{ 0x0106, 0xA4A4, 0 },	//DAGAIN = -4.125dB
	{ 0x03,0x00, 2,{0xB8,0x03}, 0 },  //400K HV32dB
	{ 0x00,0x6A, 2,{0x00,0x02}, 0 },
	{ 0x00,0x6C, 2,{0x65,0x05}, 0 },
	{ 0x00,0x6A, 2,{0x03,0x00}, 0 },
	{ 0x00,0x6C, 2,{0x01,0x45}, 0 },
	{ 0x00,0x6A, 2,{0x03,0x02}, 0 },
	{ 0x00,0x6C, 2,{0x28,0x02}, 0 },
	{ 0x00,0x6A, 2,{0x03,0x0C}, 0 },
	{ 0x00,0x6C, 2,{0x0F,0xC0}, 0 },
	{ 0x00,0x6A, 2,{0x01,0x0C}, 0 },
	{ 0x00,0x6C, 2,{0xC0,0x00}, 0 },
	// Turn on Efuse & Auto Readback mode
	{ 0x03,0x26, 2,{0x00,0xAF}, 0 },
	{ 0x03,0x26, 2,{0x00,0xFF}, 0 },
	{ 0x00,0x80, 2,{0x80,0x00}, 0 },
	{ 0x00,0x80, 2,{0x80,0x01}, 0 },
	{ 0x00,0x6A, 2,{0x02,0x20}, 0 },
	{ 0x00,0x6C, 2,{0x00,0x00}, 0 },

	// Turn off Efuse
	{ 0x03,0x26, 2,{0x00,0xAF}, 0 },
	{ 0x03,0x26, 2,{0x00,0x0F}, 50 },
	// Delay 0.5

	// enable PLL
	{ 0x00,0x04, 2,{0x10,0x00}, 0 },

	// Turn on PWM
	{ 0x00,0x6A, 2,{0x02,0x16}, 0 }, // DC detect H_threshold
	{ 0x00,0x6C, 2,{0x04,0x70}, 0 },
	{ 0x00,0x6A, 2,{0x02,0x17}, 0 }, // DC detect L_threshold
	{ 0x00,0x6C, 2,{0x04,0x00}, 0 },
	{ 0x02,0x14, 2,{0x80,0x00}, 0 }, // DC detect enable
	{ 0x03,0x50, 2,{0x01,0x08}, 0 },

	{END_FLAG,END_FLAG ,2,{0x00,0x00},0},
};

void alc1311_amp_mute(int enable)
{
    IO_Set(PIN_AMP_RESET, enable);
}

void alc1311_func(int bus_id, unsigned short addr)
{
    unsigned char data[8];
    unsigned char regaddr;
    unsigned char index;
    //printf("--alc1311_func--\n");

    for(index = 0; ; index ++)
    {
        regaddr = ALC1311_InitTbl[index].bAddr;
        data[0] = ALC1311_InitTbl[index].bAddr;
        data[1] = ALC1311_InitTbl[index].bAddr1;
        if(regaddr == END_FLAG)
        {
            break;
        }
        data[2] = ALC1311_InitTbl[index].bArray[0];
        data[3] = ALC1311_InitTbl[index].bArray[1];
#if AMP_DEBUG
        printf("\n 1311 write addr:0x%x%x----data[2]=0x%x----data[3]=0x%x\n",data[0],data[1],data[2],data[3]);
#endif

        if (I2C_Write_EX(bus_id, addr,  4, data ) < 0)
            printf("program ALC1311 failed\n");

	mdelay(ALC1311_InitTbl[index].bdelay);
#if AMP_DEBUG
        unsigned char readArray[2] = {0,0};
        if (I2C_Read_EX(bus_id,addr, 1,  &regaddr, 2, readArray)!= S_OK)
        {
            printf("\n i2c read fail \n"  );
        }
        printf("\nread addr:0x%x----data[0]=0x%x----data[1]=0x%x\n",regaddr,readArray[0],readArray[1]);
#endif
        if(index == 0){
            //After 10msec, Relase PD Pin
            mdelay(10);
            alc1311_amp_mute(0);
        }
    }
}

