#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include "auxfun.h"
#include "otp_bitmap.h"
#include "util.h"
#include "ver.h"
#include "crc.h"

#define OPTION_NON	0
#define OPTION_GEN	1
#define OPTION_PRT	2
#define OPTION_MGE	3
#define OPTION_CRC8	4
#define OPTION_CRC16	5

int print_bitmap(otp_bitmap_t *map);
int parse_config(const char *f_name, otp_bitmap_t *map);


otp_bitmap_t bitmap,bitmap2,bitmap3;

static void xor_bitmap(char* inBuf1, char* inBuf2, char*outBuf, unsigned int size)
{
    while(size--)
    {
        *outBuf++ = *inBuf1++ ^ *inBuf2++;
    }
}

static int load_bitmap(const char *f_name,otp_bitmap_t *bitmap)
{
	FILE *fp;
	int num;

	debug_print("%s(%s)\n", __FUNCTION__, f_name);

	fp = fopen(f_name, "rb");
	if (fp == NULL)
		return -1;

	num = fread(bitmap, 1, sizeof(*bitmap), fp);
	debug_print("fread: %d, otp_bitmap_t: %ld\n", num, sizeof(*bitmap));

	fclose(fp);
	return (num - sizeof(*bitmap));	// should be 0
}



static int save_bitmap(const char *f_name, otp_bitmap_t *bitmap)
{
	FILE *fp;
	int num;

	debug_print("%s(%s)\n", __FUNCTION__, f_name);

	fp = fopen(f_name, "wb");
	if (fp == NULL)
		return -1;

	num = fwrite(bitmap, 1, sizeof(*bitmap), fp);
	debug_print("fwrite: %d, otp_bitmap_t: %ld\n", num, sizeof(*bitmap));

	fclose(fp);
	return (num - sizeof(*bitmap));	// should be 0
}

static void show_usage(char *bin_file)
{
#define INDENT_ "\n\t"
	printf("\nUsage:\n"
		"\n"
		"%s print <bitmap_file>\n"
		INDENT_
		"- print OTP fields in the <bitmap_file>\n"
		"\n"
		"%s generate <config_file> <bitmap_file>\n"
		INDENT_
		"- generate OTP bitmap to <bitmap_file> according to <config_file>\n"
		"\n"
		"%s merge <bitmap_file> <bitmap_file_2> <bitmap_file_3>\n"
		INDENT_
		"- merge OTP bitmap  <bitmap_file> and <bitmap_file_2> to <bitmap_file_3>\n"
		"\n"
		"%s crc8 <bitmap_file>\n"
		INDENT_
		"- Count the crc8 value of the <bitmap_file>\n"
		"\n"
		"%s crc16 <bitmap_file>\n"
		INDENT_
		"- Count the crc16 value of the <bitmap_file>\n"
		"\n"
		,bin_file, bin_file, bin_file, bin_file, bin_file);
#undef INDENT_
}

static int get_option(int argc, char **argv, int *option, char **config_file, char **bitmap_file, char **bitmap_file_2, char **bitmap_file_3)
{
	int i, res;
	char *ptr;

	*option = OPTION_NON;
	*config_file = NULL;
	*bitmap_file = NULL;
	*bitmap_file_2 = NULL;
	*bitmap_file_3 = NULL;
	res = -1;

	for (i = 0; i <argc; i++) {
		ptr = "generate";
		if (strncmp(ptr, argv[i], strlen(ptr)) == 0) {
			if (argc != 4) {
				printf("wrong argument\n");
				return -1;
			}

			*option = OPTION_GEN;
			*config_file = argv[i+1];
			*bitmap_file = argv[i+2];
			res = 0;
			break;
		}

		ptr = "merge";
		if (strncmp(ptr, argv[i], strlen(ptr)) == 0) {
			if (argc != 5) {
				printf("wrong argument\n");
				return -1;
			}
			*option = OPTION_MGE;
			*bitmap_file = argv[i+1];
			*bitmap_file_2 = argv[i+2];
			*bitmap_file_3 = argv[i+3];
			res = 0;
			break;
		}

		ptr = "print";
		if (strncmp(ptr, argv[i], strlen(ptr)) == 0) {
			if (argc != 3) {
				printf("wrong argument\n");
				return -1;
			}

			*option = OPTION_PRT;
			*bitmap_file = argv[i+1];
			res = 0;
			break;
		}

		ptr = "crc8";
		if (strncmp(ptr, argv[i], strlen(ptr)) == 0) {
			if (argc != 3) {
				printf("wrong argument\n");
				return -1;
			}

			*option = OPTION_CRC8;
			*bitmap_file = argv[i+1];
			res = 0;
			break;
		}

		ptr = "crc16";
		if (strncmp(ptr, argv[i], strlen(ptr)) == 0) {
			if (argc != 3) {
				printf("wrong argument\n");
				return -1;
			}

			*option = OPTION_CRC16;
			*bitmap_file = argv[i+1];
			res = 0;
			break;
		}
	}

	debug_print("config_file: %s, bitmap_file: %s, bitmap_file_2: %s, bitmap_file_3: %s\n", *config_file, *bitmap_file, *bitmap_file_2, *bitmap_file_3);
	return res;
}

int main(int argc, char **argv)
{
	int res;
	int option;
	char *config_file;
	char *bitmap_file;
	char *bitmap_file_2;
	char *bitmap_file_3;
	printf("\n%s\n", argv[0]);
	printf("version %s (compiled at %s)\n", VERSION_OTP_BITMAP_GENERATOR, __DATE__);
	
    printf("sizeof(bitmap)=%d\n", sizeof(bitmap)*8);
    
	// check argument
	if (get_option(argc, argv, &option, &config_file, &bitmap_file, &bitmap_file_2, &bitmap_file_3)) {
		show_usage(argv[0]);
		return 0;
	}
	
	memset(&bitmap, 0, sizeof(bitmap));

	switch (option) {
		case OPTION_PRT:
			res = load_bitmap(bitmap_file,&bitmap);
			if (res) {
				printf("load bitmap error: %d\n", res);
				return -1;	
			}

			print_bitmap(&bitmap);
			break;

		case OPTION_GEN:
			res = parse_config(config_file, &bitmap);
			if (res) {
				printf("parse config error: %d\n", res);
				return -1;	
			}
			print_bitmap(&bitmap);
			
			res = save_bitmap(bitmap_file,&bitmap);
			if (res) {
				printf("save bitmap error: %d\n", res);
				return -1;	
			}

			break;
			
		case OPTION_MGE:
			res = load_bitmap(bitmap_file, &bitmap);
			if (res) {
				printf("load bitmap error: %d\n", res);
				return -1;	
			}
			res = load_bitmap(bitmap_file_2, &bitmap2);
			if (res) {
				printf("load bitmap_2 error: %d\n", res);
				return -1;	
			}

			xor_bitmap((char*)&bitmap,(char*)&bitmap2,(char*)&bitmap3,sizeof(otp_bitmap_t));
			res = save_bitmap(bitmap_file_3,&bitmap3);
			if (res) {
				printf("save bitmap error: %d\n", res);
				return -1;	
			}

			break;
		case OPTION_CRC8:
			res = count_file_crc(CRC_TYPE_CRC8, bitmap_file);
			if (res < 0) {
				printf("count the crc8 fail\n");
				return -1;	
			}
			printf("The crc8 value of %s is: 0x%02x\n", bitmap_file, res & 0xff);
			break;
		case OPTION_CRC16:
			res = count_file_crc(CRC_TYPE_CRC16, bitmap_file);
			if (res < 0) {
				printf("count the crc16 fail\n");
				return -1;	
			}
			printf("The crc16 value of %s is: 0x%04x\n", bitmap_file, res & 0xffff);
			break;
			
		default:
			printf("unknown option: %d\n", option);
			show_usage(argv[0]);
			break;
	}

	return 0;
}

